@@ -19,10 +19,8 @@
 #endif
 
 
-DEFUN_NEW("CHECK-FD-FOR-INPUT",object,fScheck_fd_for_input,
-      SI,2,2,NONE,OI,IO,OO,OO,(fixnum fd,fixnum timeout),
-
-"Check FD a file descriptor for data to read, waiting TIMEOUT clicks \
+DEFUN("CHECK-FD-FOR-INPUT",object,fScheck_fd_for_input,SI,2,2,NONE,OI,IO,OO,OO,(fixnum fd,fixnum timeout),
+      "Check FD a file descriptor for data to read, waiting TIMEOUT clicks \
 for data to become available.  Here there are \
 INTERNAL-TIME-UNITS-PER-SECOND in one second.  Return is 1 if data \
 available on FD, 0 if timeout reached and -1 if failed.")
@@ -183,28 +181,30 @@ int m;
     { bcopy(sfd->valid_data,sfd->read_buffer,sfd->valid_data_size);
       sfd->valid_data=sfd->read_buffer;}
    /* there is at least a packet size of space available */   
-  if ((fix(FFN(fScheck_fd_for_input)(sfd->fd,sfd->write_timeout))>0))
-     again:
-        {char *start = sfd->valid_data+sfd->valid_data_size;
-        nread = SAFE_READ(sfd->fd,start,
-		     sfd->read_buffer_size - (start -  sfd->read_buffer));
-       }
-        if (nread<0)
-          {if (errno == EAGAIN) goto again;
-           return -1;}
-        if (nread == 0)
-	  { 
-	    return 0;
-	  }
-        sfd->total_bytes_received +=  nread;
-        sfd->bytes_received_not_confirmed +=  nread;
-        sfd->valid_data_size += nread; 
-        if(sfd->bytes_received_not_confirmed > MUST_CONFIRM)
-          send_confirmation(sfd);
-        scan_headers(sfd); 
-        goto TRY_PACKET;
-      }
+  if ((fix(FFN(fScheck_fd_for_input)(sfd->fd,sfd->write_timeout))>0)) {
 
+    char *start = sfd->valid_data+sfd->valid_data_size;
+  again:
+    nread = SAFE_READ(sfd->fd,start,sfd->read_buffer_size - (start -  sfd->read_buffer));
+    if (nread<0) {
+      if (errno == EAGAIN) goto again;
+      return -1;
+    }
+    if (nread == 0)  { 
+      return 0;
+    }
+    sfd->total_bytes_received +=  nread;
+    sfd->bytes_received_not_confirmed +=  nread;
+    sfd->valid_data_size += nread; 
+    if(sfd->bytes_received_not_confirmed > MUST_CONFIRM)
+      send_confirmation(sfd);
+    scan_headers(sfd); 
+    goto TRY_PACKET;
+  }
+
+  return 0;
+
+}
 
 /* send BYTES chars from buffer P to CONNECTION.
    They are packaged up with a hdr */
@@ -262,15 +262,16 @@ write1(sfd,p,bytes)
       
 }	  
 
-DEFUN_NEW("CLEAR-CONNECTION",object,fSclear_connection,SI,1,1,NONE,OI,OO,OO,OO,(fixnum fd),
-      "Read on FD until nothing left to read.  Return number of bytes read")
-{char buffer[0x1000];
- int n=0;
- while (fix(FFN(fScheck_fd_for_input)(fd,0)))
-   { n+=read(fd,buffer,sizeof(buffer));
-   }
- 
- return make_fixnum1(n);
+DEFUN("CLEAR-CONNECTION",object,fSclear_connection,SI,1,1,NONE,OI,OO,OO,OO,(fixnum fd),
+      "Read on FD until nothing left to read.  Return number of bytes read") {
+  
+  char buffer[0x1000];
+  int n=0;
+  while (fix(FFN(fScheck_fd_for_input)(fd,0)))
+    n+=read(fd,buffer,sizeof(buffer));
+  
+  return make_fixnum1(n);
+
 }
 #ifdef STATIC_FUNCTION_POINTERS
 object
