@@ -84,11 +84,11 @@ extern char *inet_ntoa ( struct in_addr in );
 FILE *pstreamDebug;
 int fDebugSockets;
 
-#ifdef PLATFORM_SUNOS
-static void notice_input( );
-#else
-static void notice_input();
-#endif
+/* #ifdef PLATFORM_SUNOS */
+/* static void notice_input( ); */
+/* #else */
+/* static void notice_input(); */
+/* #endif */
 
 int hdl = -1;
 
@@ -285,7 +285,7 @@ char *envp[];
 #define SET_SESSION_ID() setsid()
 #else
 #ifdef BSD
-#define SET_SESSION_ID() (setpgrp(0,0) ? -1 : 0)
+#define SET_SESSION_ID() (setpgrp() ? -1 : 0)
 #endif
 #endif	  
 #endif
@@ -369,19 +369,19 @@ struct connection_state *sfd;
 }
   
 
-#ifdef PLATFORM_SUNOS
-static void
-notice_input( int sig, int code, struct sigcontext *s, char *a )
-#else
-static void
-notice_input( sig )
-     int sig;
-#endif
-{
-  signal( SIGIO, notice_input );
-  dfprintf(stderr, "\nNoticed input!\n" );
+/* #ifdef PLATFORM_SUNOS */
+/* static void */
+/* notice_input( int sig, int code, struct sigcontext *s, char *a ) */
+/* #else */
+/* static void */
+/* notice_input( sig ) */
+/*      int sig; */
+/* #endif */
+/* { */
+/*   signal( SIGIO, notice_input ); */
+/*   dfprintf(stderr, "\nNoticed input!\n" ); */
 
-}
+/* } */
 
 static int message_id;
 
@@ -455,7 +455,7 @@ struct connection_state *sfd;
   int tot;
   struct message_header *msg;
   msg = (struct message_header *) buf;
-  m= read1(sfd,msg,MESSAGE_HEADER_SIZE,DEFAULT_TIMEOUT_FOR_TK_READ);
+  m= read1(sfd,(void *)msg,MESSAGE_HEADER_SIZE,DEFAULT_TIMEOUT_FOR_TK_READ);
   if (m == MESSAGE_HEADER_SIZE)
     {
      if ( msg->magic1!=MAGIC1
@@ -468,7 +468,7 @@ struct connection_state *sfd;
       if (tot >= bufleng)
          {msg = (void *)malloc(tot+1);
 	  bcopy(buf,msg,MESSAGE_HEADER_SIZE);}
-     m = read1(sfd,&(msg->body),
+      m = read1(sfd,(void *)&(msg->body),
 		   body_length,DEFAULT_TIMEOUT_FOR_TK_READ);
      if (m == body_length)
        { return msg;}}
