@@ -7,7 +7,6 @@
  *	applications.
  *
  * Copyright (c) 1990-1993 The Regents of the University of California.
- * Copyright (c) 2024 Camm Maguire
  * All rights reserved.
  *
  * Permission is hereby granted, without written agreement and without
@@ -37,9 +36,7 @@
 #include <stdlib.h>
 #include <tcl.h>
 #include <tk.h>
-#if TCL_MAJOR_VERSION >= 9
-#include <wordexp.h>
-#endif
+
 
 
 #if (TK_MINOR_VERSION==0 && TK_MAJOR_VERSION==4)
@@ -70,12 +67,12 @@ struct connection_state *dsfd;
  * file out of the Tk source directory to make their own modified versions).
  */
 
-/* extern void		exit _ANSI_ARGS_((int status));
+/* extern void		exit _ANSI_ARGS_((int status)); */
 extern int		isatty _ANSI_ARGS_((int fd));
+/*
 extern int		read _ANSI_ARGS_((int fd, char *buf, size_t size));
-extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
 */
-extern int Tcl_AppInit(Tcl_Interp *interp);
+extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
 
 /*
  * Global variables used by the main program:
@@ -266,7 +263,7 @@ TkX_Wish (argc, argv)
      * specified on the command line.
      */
 
-    args = Tcl_Merge(argc-1, (const char **)argv+1);
+    args = Tcl_Merge(argc-1, (void *)argv+1);
     Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
     ckfree(args);
     sprintf(buf, "%d", argc-1);
@@ -334,17 +331,11 @@ TkX_Wish (argc, argv)
 	 */
 
 	if (tcl_RcFileName != NULL) {
+	    Tcl_DString buffer;
 	    char *fullName;
 	    FILE *f;
-#if TCL_MAJOR_VERSION >= 9
-	    wordexp_t exp_result;
-	    wordexp(tcl_RcFileName, &exp_result, WRDE_NOCMD);
-	    fullName = exp_result.we_wordv[0];
-#else
-	    Tcl_DString buffer;
     
 	    fullName = Tcl_TildeSubst(interp, tcl_RcFileName, &buffer);
-#endif
 	    if (fullName == NULL) {
 		fprintf(stderr, "%s\n", INTERP_RESULT(interp));
 	    } else {
@@ -357,16 +348,12 @@ TkX_Wish (argc, argv)
 		    fclose(f);
 		}
 	    }
-#if TCL_MAJOR_VERSION >= 9
-	    wordfree(&exp_result);
-#else
 	    Tcl_DStringFree(&buffer);
-#endif
 	}
 
 	dfprintf(stderr, "guis : Creating file handler for %d\n", dsfd->fd);
 #ifndef __MINGW32__	
-	Tcl_CreateFileHandler(dsfd->fd, TCL_READABLE, StdinProc, (ClientData) 0);
+	Tk_CreateFileHandler(dsfd->fd, TK_READABLE, StdinProc, (ClientData) 0);
 #endif        
     }
     fflush(stdout);
@@ -479,7 +466,7 @@ StdinProc(clientData, mask)
    */
   dfprintf(stderr, "\nguis : Disabling file handler for %d\n", dsfd->fd);
 
-/*  Tcl_CreateFileHandler(dsfd->fd, 0, StdinProc, (ClientData) 0); */
+/*  Tk_CreateFileHandler(dsfd->fd, 0, StdinProc, (ClientData) 0); */
 
   do
     { 
@@ -491,7 +478,7 @@ StdinProc(clientData, mask)
 	  /*dfprintf(stderr, "Yoo !!! Empty command\n"); */
 	  if (debug)perror("zero message");
 #ifndef __MINGW32__          
-	  Tcl_CreateFileHandler(dsfd->fd, TCL_READABLE, StdinProc, (ClientData) 0);
+	  Tk_CreateFileHandler(dsfd->fd, TK_READABLE, StdinProc, (ClientData) 0);
 #endif          
 	  return;
 	}
@@ -582,7 +569,7 @@ StdinProc(clientData, mask)
 	  dfprintf(stderr, "Error !!! Unknown command %d\n"
 		   , msg->type);
 	}
-      fNotDone = fScheck_dsfd_for_input(dsfd,0);
+      fNotDone = fix(fScheck_dsfd_for_input(dsfd,0));
       
       if (fNotDone > 0)
 	{
@@ -593,7 +580,7 @@ StdinProc(clientData, mask)
     } while (fNotDone > 0);
 
 
-  /* Tcl_CreateFileHandler(dsfd->fd, TCL_READABLE, StdinProc, (ClientData) 0); */
+  /* Tk_CreateFileHandler(dsfd->fd, TK_READABLE, StdinProc, (ClientData) 0); */
   if ((void *)msg != (void *) buf)
     free ((void *) msg);
 }
