@@ -112,8 +112,8 @@
 
 )
 
-(defmacro nth-value (n form)
-  `(multiple-value-bind ,(make-list (+ n 1) :initial-element 'a) ,form  a))
+;(defmacro nth-value (n form)
+;  `(multiple-value-bind ,(make-list (+ n 1) :initial-element 'a) ,form  a))
 
 (defvar *tk-command* nil)
 
@@ -1241,7 +1241,6 @@
 	 (funcall (car funs) string)))))
 
 ;;convert "2c" into screen units or points or something...
-    ))
 
 ;; If loc is suitable for handing to setf,  then
 ;; (setf loc (coerce-result val type)
