@@ -28,8 +28,10 @@
 #endif     
 #endif
 
-#define GET_FAULT_ADDR(sig,code,sv,a) \
-    ((void *)(((struct sigcontext *)(&code))->cr2))
+#undef MPROTECT_ACTION_FLAGS
+#define MPROTECT_ACTION_FLAGS SA_RESTART|SA_SIGINFO
+#define GET_FAULT_ADDR(sig,code,sv,a) ((siginfo_t *)code)->si_addr
+/* #define GET_FAULT_ADDR(sig,code,sv,a) ((void *)(((struct sigcontext *)(&code))->cr2)) */
 #endif
 
 /*#define NULL_OR_ON_C_STACK(x) ((x)==0 || ((unsigned int)x) > (unsigned int)(pagetochar(MAXPAGE+1)))*/
@@ -41,3 +43,5 @@
 
 #define	I386
 #define SGC
+
+#define RELOC_H "elf32_i386_reloc.h"
