@@ -45,11 +45,10 @@
 #include <sys/stat.h>
 #endif
 
-#define LITTLE_ENDIAN
+/* #define LITTLE_END */
 
 /* Begin for cmpinclude */
 
-#define HAVE_ALLOCA
 
 #ifdef __GNUC__
 #undef __BUILTIN_VA_ARG_INCR
