@@ -33,7 +33,7 @@
 #define IEEEFLOAT
 
 /* The exponent and most signif are in the second word for doubles */
-#define LITTLE_END
+/* #define LITTLE_END */
 
 #undef   FILECPY_HEADER
 #define FILECPY_HEADER \
@@ -113,7 +113,7 @@
          {int ll;   \
 	   if ((ll=(int)(sbrk(0) - core_end))  <= PAGESIZE) \
            {sbrk(PAGESIZE - ll); \
-            type_map[page(core_end)] = t_other; \
+            \
             heap_end=core_end = sbrk(0); }\
           else  \
         error("Someone allocated my memory!");} \
