@@ -24,10 +24,38 @@ immnum_andc1(x,y)
 immnum_andc2(x,y)
 immnum_orc1(x,y)
 immnum_orc2(x,y)
-listp(x)
+tp0(x)
+tp1(x)
+tp2(x)
+tp3(x)
+tp4(x)
+tp5(x)
+tp6(x)
+tp7(x)
+tp8(x)
 sf(x)
+lf(x)
+sfc(x)
+lfc(x)
+eql_is_eq(x)
+equal_is_eq(x)
+equalp_is_eq(x)
+listp(x)
+functionp(x)
+rationalp(x)
 atom(x)
+numberp(x)
+realp(x)
 consp(x)
+integerp(x)
+rationalp(x)
+floatp(x)
+realp(x)
+numberp(x)
+vectorp(x)
+arrayp(x)
+compiled_functionp(x)
+pathname_designatorp(x)
 OBJNULL
 Cnil
 Ct
@@ -56,71 +84,11 @@ mpz_odd_p(x)
 mpz_sgn(x)
 NULL
 FALSE
+FEerror(a,b,c)
 TRUE
+MULTIPLE_VALUES_LIMIT
 is_imm_fixnum(x)
-is_unmrkd_imm_fixnum(x)
+is_imm_fix(x)
 aref1
-VFUN_NARGS
-KEYTYPE
-S_DATA(x)
-MMcons(x,y)
-MMcar(x)
-MMcdr(x)
-fcalln
-CMPcar(x)   
-CMPcdr(x)   
-CMPcaar(x)
-CMPcadr(x)
-CMPcdar(x)  
-CMPcddr(x)  
-CMPcaaar(x) 
-CMPcaadr(x) 
-CMPcadar(x) 
-CMPcaddr(x) 
-CMPcdaar(x) 
-CMPcdadr(x) 
-CMPcddar(x) 
-CMPcdddr(x) 
-CMPcaaaar(x)
-CMPcaaadr(x)
-CMPcaadar(x)
-CMPcaaddr(x)
-CMPcadaar(x)
-CMPcadadr(x)
-CMPcaddar(x)
-CMPcadddr(x)
-CMPcdaaar(x)
-CMPcdaadr(x)
-CMPcdadar(x)
-CMPcdaddr(x)
-CMPcddaar(x)
-CMPcddadr(x)
-CMPcdddar(x)
-CMPcddddr(x)
-ALLOCA_CONS(x)
-check_arg(x)
-endp_prop(x)
-fix_imm_fixnum(x)
-make_imm_fixnum(x)
-gethash_with_check(x,y)
-sethash_with_check(x,y,z)
-ON_STACK_LIST_VECTOR_NEW
-ON_STACK_MAKE_LIST
-SAFE_CDR(x)
-Scons
-EQ(x,y)
-aset
-stp_ordinary
-SIGNED_CHAR(x)
-FEerror(x,y...)
-FEwrong_type_argument(x,y)
-BIT_ENDIAN(x)
-integerp(x)
-rationalp(x)
-floatp(x)
-realp(x)
-numberp(x)
-vectorp(x)
-arrayp(x)
-compiled_function_p(x)
-pathname_designatorp(x)
+OBJ_ALIGNED_STACK_ALLOC(x)
+NO_RETURN
