@@ -9,7 +9,7 @@
 #define    unmark_imm_fixnum(a_)        ((a_)=((object)((fixnum)(a_)-(LOW_IM_FIX<<1))))
 #define        is_imm_fixnum(a_)        ((fixnum)(a_)<(fixnum)OBJNULL)
 #define is_unmrkd_imm_fixnum(a_)        ((fixnum)(a_)<LOW_IM_FIX)
-#define is_marked_imm_fixnum(a_)        (is_imm_fixnum(a_)*!is_unmrkd_imm_fixnum(a_))
+#define is_marked_imm_fixnum(a_)       p (is_imm_fixnum(a_)*!is_unmrkd_imm_fixnum(a_))
 #define           is_imm_fix(a_)        INT_IN_BITS(a_,LOW_SHFT-1)
 #elif defined (IM_FIX_BASE) && defined(IM_FIX_LIM)
 #define      make_imm_fixnum(a_)        ((object)((a_)+(IM_FIX_BASE+(IM_FIX_LIM>>1))))
@@ -33,10 +33,11 @@
 /* #define        un_imm_fixnum(a_)         */
 #endif
 
-#define make_fixnum(a_)  ({register fixnum _q1=(a_);register object _q4; \
-      _q4=is_imm_fix(_q1) ? make_imm_fixnum(_q1) : make_fixnum1(_q1);_q4;})
-#define fix(a_)          ({register object _q2=(a_);register fixnum _q3;		\
-      _q3=is_imm_fixnum(_q2) ? fix_imm_fixnum(_q2) :  (_q2)->FIX.FIXVAL;_q3;})
+#define make_fixnum(a_)  ({register fixnum _q1=(a_);register object _q3;\
+                          _q3=is_imm_fix(_q1) ? make_imm_fixnum(_q1) : make_fixnum1(_q1);_q3;})
+#define CMPmake_fixnum(a_) make_fixnum(a_)/*FIXME*/
+#define fix(a_)          ({register object _q2=(a_);register fixnum _q4;\
+                          _q4=is_imm_fixnum(_q2) ? fix_imm_fixnum(_q2) :  (_q2)->FIX.FIXVAL;_q4;})
 #define Mfix(a_)         fix(a_)
 #define small_fixnum(a_) make_fixnum(a_) /*make_imm_fixnum(a_)*/
 #define set_fix(a_,b_)   ((a_)->FIX.FIXVAL=(b_))
