@@ -8,25 +8,21 @@ typedef unsigned long   ufixnum;
 
 #ifndef WORDS_BIGENDIAN
 
-/* high bit must be clear to distinguish from high immediate fixnum*/
-#define FRSTWRD(t_,b_,a_...) ufixnum    e:1,m:1,f:1,    t_:5,t:5,st:3,a_,b_,h:1
-#define FRSTWRDF(t_,a_...)   ufixnum    e:1,m:1,f:1,    t_:5,t:5,st:3,a_,h:1
-#define FIRSTWORD            ufixnum    e:1,m:1,f:1,    tt:5,t:5,st:3,w:LM(17),h:1
-#define FSTPWORD             ufixnum  emf:3,            tp:10,   st:3,w:LM(17),h:1
-#define MARKWORD             ufixnum    e:1,   mf:2,    tt:5,t:5,xx:LM(14),h:1
-#define SGCMWORD             ufixnum    e:1,mf:2,       tt:5,t:5,xx:LM(14),h:1
-#define TYPEWORD             ufixnum  emf:3,            tt:5,t:5,xx:LM(14),h:1
+#define FRSTWRD(t_,a_...) ufixnum    e:1,m:1,f:1,    t_:5,t:5,st:3, ##a_
+#define FIRSTWORD         ufixnum    e:1,m:1,f:1,    tt:5,t:5,st:3,w:LM(16)
+#define FSTPWORD          ufixnum  emf:3,            tp:10,   st:3,w:LM(16)
+#define MARKWORD          ufixnum    e:1,   mf:2,    tt:5,t:5,x:LM(13)
+#define SGCMWORD          ufixnum    e:1,mf:2,       tt:5,t:5,x:LM(13)
+#define TYPEWORD          ufixnum  emf:3,            tt:5,t:5,x:LM(13)
 
 #else
 
-/* high bit must be clear to distinguish from high immediate fixnum*/
-#define FRSTWRD(t_,b_,a_...) ufixnum h:1,b_,a_,   st:3,t:5,t_:5,    f:1,m:1,e:1
-#define FRSTWRDF(t_,a_...)   ufixnum h:1,a_,   st:3,t:5,t_:5,    f:1,m:1,e:1
-#define FIRSTWORD            ufixnum h:1,w:LM(17),st:3,t:5,tt:5,    f:1,m:1,e:1
-#define FSTPWORD             ufixnum h:1,w:LM(17),st:3,tp:10,             emf:3
-#define MARKWORD             ufixnum h:1,xx:LM(14),    t:5,tt:5,       mf:2,e:1
-#define SGCMWORD             ufixnum h:1,xx:LM(14),    t:5,tt:5,       mf:2,e:1
-#define TYPEWORD             ufixnum h:1,xx:LM(14),    t:5,tt:5,          emf:3
+#define FRSTWRD(t_,a_...) ufixnum ##a_,    st:3,t:5,t_:5,    f:1,m:1,e:1
+#define FIRSTWORD         ufixnum w:LM(16),st:3,t:5,tt:5,    f:1,m:1,e:1
+#define FSTPWORD          ufixnum w:LM(16),st:3,tp:10,             emf:3
+#define MARKWORD          ufixnum x:LM(13),     t:5,tt:5,       mf:2,e:1
+#define SGCMWORD          ufixnum x:LM(13),     t:5,tt:5,       mf:2,e:1
+#define TYPEWORD          ufixnum x:LM(13),     t:5,tt:5,          emf:3
 
 #endif
 
@@ -149,13 +145,10 @@ struct character {
 
   object            ch_name;
   ufixnum           pad5;
-  uchar             ch_code;/*need address of this*/
+  uchar             ch_code;
   uchar             ch_font;
   uchar             ch_bits;
-  uchar             cp1;/*cygwin won't take a bit field pad here*/
-#if SIZEOF_LONG == 8
-  uchar             cp2,cp3,cp4,cp5;
-#endif
+  ufixnum           pad:LM(24);
   ufixnum           pad1;
   ufixnum           pad2;
   ufixnum           pad3;
@@ -221,7 +214,7 @@ struct hashtable {           /*  hash table header  */
   ufixnum       ht_size:LM(4);      /*  hash table size  */
   uhfixnum      ht_test:2;          /*  key test function, of enum httest  */
   uhfixnum      ht_pad3:HM(2);      /*  unused */
-  uhfixnum      ht_pad4;            /*  unused */
+  hfixnum       ht_pad4;            /*  unused */
   ufixnum       ht_pad5:4;          /*  unused */
   ufixnum       ht_max_ent:LM(4);   /*  max entries */
   htent         ht_cache;           /*  gethash cache */
@@ -237,31 +230,16 @@ struct hashtable {           /*  hash table header  */
 #define ARRAY_RANK_LIMIT (1UL<<ARRAY_RANK_BITS)
 
 #if SIZEOF_LONG == 8
-#ifdef WORDS_BIGENDIAN
 #define ARRAYWORD(b_,c_)						\
-  FRSTWRDF(J(b_,J(c_,elttype)),						\
-	   J(b_,J(c_,eltmode)):3,					\
-	   J(b_,J(c_,dim)):ARRAY_DIMENSION_BITS,			\
-	   J(b_,J(c_,hasfillp)):1,					\
-	   J(b_,J(c_,writable)):1,					\
-	   J(b_,J(c_,rank)):ARRAY_RANK_BITS,				\
-	   pd1:1,							\
-	   J(b_,J(c_,adjustable)):1,					\
-	   J(b_,J(c_,offset)):3,					\
-	   J(b_,J(c_,eltsize)):3)
-#else
-#define ARRAYWORD(b_,c_)						\
-  FRSTWRDF(J(b_,J(c_,elttype)),						\
-	   J(b_,J(c_,hasfillp)):1,					\
-	   J(b_,J(c_,writable)):1,					\
-	   J(b_,J(c_,rank)):ARRAY_RANK_BITS,				\
-	   pd1:1,							\
-	   J(b_,J(c_,adjustable)):1,					\
-	   J(b_,J(c_,offset)):3,					\
-	   J(b_,J(c_,eltsize)):3,					\
-	   J(b_,J(c_,eltmode)):3,					\
-	   J(b_,J(c_,dim)):ARRAY_DIMENSION_BITS)
-#endif
+  FRSTWRD(J(b_,J(c_,elttype)),						\
+	  J(b_,J(c_,hasfillp)):1,					\
+	  J(b_,J(c_,adjustable)):1,					\
+	  J(b_,J(c_,writable)):1,					\
+	  J(b_,J(c_,offset)):3,						\
+	  J(b_,J(c_,eltsize)):4,					\
+	  J(b_,J(c_,eltmode)):4,					\
+	  J(b_,J(c_,rank)):ARRAY_RANK_BITS,				\
+	  J(b_,J(c_,dim)):ARRAY_DIMENSION_BITS)
 
 #define atem(a_,b_,c_)				\
   ARRAYWORD(b_,c_);				\
@@ -270,21 +248,20 @@ struct hashtable {           /*  hash table header  */
 #else
 
 #define ARRAYWORD(b_,c_)						\
-  FRSTWRDF(J(b_,J(c_,elttype)),						\
-	   J(b_,J(c_,hasfillp)):1,					\
-	   J(b_,J(c_,adjustable)):1,					\
-	   J(b_,J(c_,writable)):1,					\
-	   J(b_,J(c_,offset)):3,					\
-	   J(b_,J(c_,eltsize)):3,					\
-	   J(b_,J(c_,rank)):ARRAY_RANK_BITS)
+  FRSTWRD(J(b_,J(c_,elttype)),						\
+	  J(b_,J(c_,hasfillp)):1,					\
+	  J(b_,J(c_,adjustable)):1,					\
+	  J(b_,J(c_,writable)):1,					\
+	  J(b_,J(c_,offset)):3,						\
+	  J(b_,J(c_,eltsize)):4,					\
+	  J(b_,J(c_,rank)):ARRAY_RANK_BITS)
 
 #define atem(a_,b_,c_)					\
   ARRAYWORD(b_,c_);					\
   a_       *J(b_,J(c_,self));				\
   ufixnum   J(b_,J(c_,dim)):ARRAY_DIMENSION_BITS;	\
-  ufixnum   J(b_,J(c_,eltmode)):3;			\
-  ufixnum   pad1:LM(31)
-/*FIXME SPAD*/
+  ufixnum   J(b_,J(c_,eltmode)):4
+
 #endif
 
 #define dimstempl(b_,c_)			\
@@ -416,16 +393,6 @@ struct structure {  /*  structure header  */
 
 };
 
-struct strstd {  /*  structure header  */
-
-  FIRSTWORD;
-
-  structure  strstd_sdef;  /*  structure definition (a structure)  */
-  object    *strstd_sself; /*  structure self  */
-  SPAD;
-
-};
-
 struct stream {
 
   FIRSTWORD;
@@ -473,17 +440,14 @@ struct pathname {
 struct function {
 
   FRSTWRD(tt,
-#if SIZEOF_LONG == 8
-	  fw:LM(39),
 	  fun_minarg:6,    /* required arguments */
 	  fun_maxarg:6,    /* maximum arguments */
-	  pd:4,
+#if SIZEOF_LONG == 8
 	  fun_neval:5,     /* maximum extra values set */
-	  fun_vv:1         /* variable number of values */
+	  fun_vv:1,        /* variable number of values */
+	  fw:LM(34)
 #else
-	  fw:LM(29),
-	  fun_minarg:6,    /* required arguments */
-	  fun_maxarg:6     /* maximum arguments */
+	  fw:LM(28)
 #endif
 	  );
 
@@ -504,8 +468,8 @@ struct function {
 struct cfdata {
 
   FRSTWRD(tt,
-	  cfw:LM(18),
-	  cfd_prof:1       /* profiling */
+	  cfd_prof:1,       /* profiling */
+	  cfw:LM(17)
 	  );
 
   char   *cfd_start;             /* beginning of contblock for fun */
@@ -565,7 +529,6 @@ union lispunion {
  struct matrix             mt; /*  matrix  */
  struct adjarray         aadj; /*  adjustable array  */
  struct structure         str; /*  structure  */
- struct strstd         strstd; /*  structure  */
  struct stream             sm; /*  stream  */
  struct random            rnd; /*  random-states  */
  struct readtable          rt; /*  read table  */
