@@ -1,5 +1,5 @@
 #define MAYBE_DATA_P(pp) ((char *)(pp)>= (char *) data_start)/*DBEGIN*/
-#define VALID_DATA_ADDRESS_P(pp) (MAYBE_DATA_P(pp) &&  ((char *)(pp) < heap_end))
+#define VALID_DATA_ADDRESS_P(pp) (MAYBE_DATA_P(pp) &&  inheap(pp))
 
 
 #ifndef page
@@ -28,11 +28,13 @@
 #define CPTR_SIZE PTR_ALIGN
 #endif
 
-#define FLR(x,r) (((x))&~(r-1))
+#define FLR(x,r) ((x)&~(r-1))
 #define CEI(x,r) FLR((x)+(r-1),r)
 #define PFLR(x,r) ((void *)FLR((ufixnum)x,r))
 #define PCEI(x,r) ((void *)CEI((ufixnum)x,r))
 
+#define OBJ_ALIGNED_STACK_ALLOC(x) ({void *v=alloca((x)+OBJ_ALIGNMENT-1);PCEI(v,OBJ_ALIGNMENT);})
+
 #ifdef SGC
 
 #define NORMAL_PAGE 0
@@ -102,7 +104,6 @@ extern fixnum writable_pages;
 #define ON_WRITABLE_PAGE_CACHED(x) CACHED_WRITABLE_PAGE_P(page(x))
 
 
-
 EXTER long first_data_page,real_maxpage,phys_pages,available_pages;
 EXTER void *data_start,*initial_sbrk;
 
