@@ -1,6 +1,4 @@
-#include "linux.h"
-
-/*  #define CLEAR_CACHE do {void *v=memory->cfd.cfd_start,*ve=v+memory->cfd.cfd_size; for (;v<ve;v+=32)   asm __volatile__ ("dcbst 0,%0\n\tsync\n\ticbi 0,%0\n\tsync\n\tisync": : "r" (v) : "memory");} while(0) */
+n#include "linux.h"
 
 #ifdef IN_GBC
 #undef MPROTECT_ACTION_FLAGS
@@ -12,3 +10,11 @@
 #endif
 
 #define SGC
+
+#if SIZEOF_LONG == 8
+#define C_GC_OFFSET 4
+#define RELOC_H "elf64_s390_reloc.h"
+#define SPECIAL_RELOC_H "elf64_sparc_reloc_special.h"
+#else
+#define RELOC_H "elf32_s390_reloc.h"
+#endif
