@@ -1,6 +1,5 @@
+typedef void (*handler_function_type)(int,siginfo_t *,void *);
 
-
-typedef void (*handler_function_type)();
 EXTER handler_function_type our_signal_handler[32];
 #ifdef __MINGW32__
 void main_signal_handler (int signo);
