@@ -71,6 +71,8 @@ long starting_hole_div=10;
 long starting_relb_heap_mult=2;
 long resv_pages=0;
 
+void *stack_alloc_start=NULL,*stack_alloc_end=NULL;
+
 #ifdef BSD
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -100,7 +102,7 @@ static inline void
 expand_contblock_array(void) {
 
   if (contblock_array==Cnil) {
-    contblock_array=fSmake_vector1_2(16,aet_fix,Cnil,make_fixnum(0));
+    contblock_array=fSmake_vector(make_fixnum(aet_fix),16,Ct,make_fixnum(0),Cnil,0,Cnil,make_fixnum(0));
     contblock_array->v.v_self[0]=(object)&cb_pointer;
     enter_mark_origin(&contblock_array);
   }
@@ -445,6 +447,7 @@ check_avail_pages(void) {
 
 }
 
+#include <fenv.h>
 
 fixnum
 set_tm_maxpage(struct typemanager *tm,fixnum n) {
@@ -452,7 +455,7 @@ set_tm_maxpage(struct typemanager *tm,fixnum n) {
   fixnum r=tm->tm_type==t_relocatable,j=tm->tm_maxpage,z=(n-j)*(r ? 2 : 1);
   if (z>available_pages) return 0;
   available_pages-=z;
-  tm->tm_adjgbccnt*=((double)j+1)/(n+1);
+  ({fenv_t f;feholdexcept(&f);tm->tm_adjgbccnt*=((double)j+1)/(n+1);fesetenv(&f);});
   tm->tm_maxpage=n;
   /* massert(!check_avail_pages()); */
   return 1;
@@ -467,8 +470,7 @@ type_name(int t) {
 static void
 call_after_gbc_hook(int t) {
   if (sSAafter_gbc_hookA && sSAafter_gbc_hookA->s.s_dbind!= Cnil) {
-    set_up_string_register(tm_table[(int)t].tm_name+1);
-    ifuncall1(sSAafter_gbc_hookA->s.s_dbind,intern(string_register,system_package));
+    ifuncall1(sSAafter_gbc_hookA->s.s_dbind,intern(str((tm_table[(int)t].tm_name+1)),system_package));
   }
 }
 
@@ -554,7 +556,7 @@ static inline void
 expand_contblock_index_space(void) {
 
   if (cbv==Cnil) {
-    cbv=fSmake_vector1_2(16,aet_fix,Cnil,make_fixnum(0));
+    cbv=fSmake_vector(make_fixnum(aet_fix),16,Ct,make_fixnum(0),Cnil,0,Cnil,make_fixnum(0));
     cbv->v.v_self[0]=(object)&cb_pointer;
     enter_mark_origin(&cbv);
   }
@@ -817,7 +819,7 @@ alloc_after_gc(struct typemanager *tm,fixnum n) {
 
     switch (jmp_gmp) {
     case 0: /* not in gmp call*/
-      GBC(tm->tm_type);
+      GBC(tm->tm_calling_type);
       break;
     case 1: /* non-in-place gmp call*/
       longjmp(gmp_jmp,tm->tm_type);
@@ -1017,7 +1019,7 @@ alloc_code_space(size_t sz,ufixnum max_code_address) {
     v=sSAcode_block_reserveA->s.s_dbind->st.st_self;
     sSAcode_block_reserveA->s.s_dbind->st.st_self+=sz;
     sSAcode_block_reserveA->s.s_dbind->st.st_dim-=sz;
-    sSAcode_block_reserveA->s.s_dbind->st.st_fillp=sSAcode_block_reserveA->s.s_dbind->st.st_dim;
+    VSET_MAX_FILLP(sSAcode_block_reserveA->s.s_dbind);
     
   } else
     v=alloc_contblock(sz);
@@ -1058,6 +1060,8 @@ make_cons(object a,object d) {
   static struct typemanager *tm=tm_table+t_cons;/*FIXME*/
   object obj=alloc_mem(tm,tm->tm_size);
 
+  tm->tm_calling_type=t_cons;
+
   load_cons(obj,a,d);
 
   pageinfo(obj)->in_use++;
@@ -1073,20 +1077,23 @@ on_stack_cons(object x, object y) {
   return p;
 }
 
-DEFUNM_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"")
-{ struct typemanager *tm=(&tm_table[t_from_type(typ)]);
-  tm = & tm_table[tm->tm_type];
-  if (tm->tm_type == t_relocatable)
-    { tm->tm_npage = page(rb_size());
-      tm->tm_nfree = rb_limit -rb_pointer;
-    }
-  else if (tm->tm_type == t_contiguous)
-    { int cbfree =0;
-      struct contblock **cbpp;
-      for(cbpp= &cb_pointer; (*cbpp)!=NULL; cbpp= &(*cbpp)->cb_link)
-	  cbfree += (*cbpp)->cb_size ;
-      tm->tm_nfree = cbfree;
-    }
+
+DEFUNM("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"") { 
+
+  struct typemanager *tm=(&tm_table[t_from_type(typ)]);
+  fixnum vals=(fixnum)fcall.valp;
+  object *base=vs_top;
+
+  if (tm->tm_type == t_relocatable) {
+    tm->tm_npage = page(rb_size());
+    tm->tm_nfree = rb_limit -rb_pointer;
+  } else if (tm->tm_type == t_contiguous) { 
+    int cbfree =0;
+    struct contblock **cbpp;
+    for(cbpp= &cb_pointer; (*cbpp)!=NULL; cbpp= &(*cbpp)->cb_link)
+      cbfree += (*cbpp)->cb_size ;
+    tm->tm_nfree = cbfree;
+  }
   
   RETURN(6,object,make_fixnum(tm->tm_nfree),
 	    (RV(make_fixnum(tm->tm_npage)),
@@ -1100,7 +1107,7 @@ DEFUNM_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"
 extern void overlap_check(struct contblock *,struct contblock *);
 #endif
 
-DEFUN_NEW("PRINT-FREE-CONTBLOCK-LIST",object,fSprint_free_contblock_list,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+DEFUN("PRINT-FREE-CONTBLOCK-LIST",object,fSprint_free_contblock_list,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
   
   struct contblock *cbp,*cbp1;
 
@@ -1136,9 +1143,8 @@ init_tm(enum type t, char *name, int elsize, int nelts, int sgc,int distinct) {
   if (!distinct)
     for (i = 0;  i < t_end;  i++)
       if (tm_table[i].tm_size != 0 &&
-	  tm_table[i].tm_size >= elsize &&
-	  !tm_table[i].tm_distinct &&
-	  (j < 0 || tm_table[j].tm_size > tm_table[i].tm_size))
+	  tm_table[i].tm_size == elsize &&
+	  !tm_table[i].tm_distinct)
 	j = i;
   if (j >= 0) {
     tm_table[(int)t].tm_type = (enum type)j;
@@ -1218,7 +1224,7 @@ gcl_init_alloc(void *cs_start) {
 
     massert(!getrlimit(RLIMIT_STACK, &rl));
     if (rl.rlim_cur!=RLIM_INFINITY && (rl.rlim_max == RLIM_INFINITY || rl.rlim_max > rl.rlim_cur)) {
-      rl.rlim_cur = rl.rlim_max;
+      rl.rlim_cur = rl.rlim_max; /* == RLIM_INFINITY ? rl.rlim_max : rl.rlim_max/64; */
       massert(!setrlimit(RLIMIT_STACK,&rl));
     }
     cssize = rl.rlim_cur/sizeof(*cs_org) - sizeof(*cs_org)*CSGETA;
@@ -1289,9 +1295,9 @@ gcl_init_alloc(void *cs_start) {
   init_tm(t_cons, ".CONS", sizeof(struct cons), 0 ,50,0 );
   init_tm(t_fixnum, "NFIXNUM",sizeof(struct fixnum_struct), 0,20,0);
   init_tm(t_structure, "SSTRUCTURE", sizeof(struct structure),0,1,0 );
-  init_tm(t_cfun, "fCFUN", sizeof(struct cfun),0,1,0  );
-  init_tm(t_sfun, "gSFUN", sizeof(struct sfun),0,1,0 );
+  init_tm(t_simple_string, "\'SIMPLE-STRING", sizeof(struct unadjstring),0,1,0);
   init_tm(t_string, "\"STRING", sizeof(struct string),0,1,0  );
+  init_tm(t_simple_array, "ASIMPLE-ARRAY", sizeof(struct unadjarray),0,1,0 );
   init_tm(t_array, "aARRAY", sizeof(struct array),0,1,0 );
   init_tm(t_symbol, "|SYMBOL", sizeof(struct symbol),0,1,0 );
   init_tm(t_bignum, "BBIGNUM", sizeof(struct bignum),0,1,0 );
@@ -1302,17 +1308,15 @@ gcl_init_alloc(void *cs_start) {
   init_tm(t_character,"#CHARACTER",sizeof(struct character),0 ,1,0);
   init_tm(t_package, ":PACKAGE", sizeof(struct package),0,1,0);
   init_tm(t_hashtable, "hHASH-TABLE", sizeof(struct hashtable),0,1,0 );
+  init_tm(t_simple_vector, "VSIMPLE-VECTOR", sizeof(struct unadjvector),0 ,1,0);
   init_tm(t_vector, "vVECTOR", sizeof(struct vector),0 ,1,0);
+  init_tm(t_simple_bitvector, "BSIMPLE-BIT-VECTOR", sizeof(struct unadjbitvector),0 ,1,0);
   init_tm(t_bitvector, "bBIT-VECTOR", sizeof(struct bitvector),0 ,1,0);
   init_tm(t_stream, "sSTREAM", sizeof(struct stream),0 ,1,0);
   init_tm(t_random, "$RANDOM-STATE", sizeof(struct random),0 ,1,0);
   init_tm(t_readtable, "rREADTABLE", sizeof(struct readtable),0 ,1,0);
   init_tm(t_pathname, "pPATHNAME", sizeof(struct pathname),0 ,1,0);
-  init_tm(t_cclosure, "cCCLOSURE", sizeof(struct cclosure),0 ,1,0);
-  init_tm(t_closure, "cCLOSURE", sizeof(struct closure),0 ,1,0);
-  init_tm(t_vfun, "VVFUN", sizeof(struct vfun),0 ,1,0);
-  init_tm(t_gfun, "gGFUN", sizeof(struct sfun), 0 ,1,0);
-  init_tm(t_afun, "AAFUN", sizeof(struct sfun), 0 ,1,0);
+  init_tm(t_function, "xFUNCTION", sizeof(struct function), 85 ,1,0);
   init_tm(t_cfdata, "cCFDATA", sizeof(struct cfdata),0 ,1,0);
   init_tm(t_spice, "!SPICE", sizeof(struct spice),0 ,1,0);
   init_tm(t_relocatable, "%RELOCATABLE-BLOCKS", 0,0,20,1);
@@ -1337,7 +1341,7 @@ gcl_init_alloc(void *cs_start) {
   
 }
 
-DEFUN_NEW("STATICP",object,fSstaticp,SI,1,1,NONE,OO,OO,OO,OO,(object x),"Tell if the string or vector is static") {
+DEFUN("STATICP",object,fSstaticp,SI,1,1,NONE,OO,OO,OO,OO,(object x),"Tell if the string or vector is static") {
   RETURN1((inheap(x->ust.ust_self) ? sLt : sLnil));
 }
 
@@ -1351,13 +1355,14 @@ t_from_type(object type) {
  
   int i;
   check_type_or_symbol_string(&type);
+  type=coerce_to_string(type);
   for (i= t_start ; i < t_other ; i++)
     {struct typemanager *tm = &tm_table[i];
     if(tm->tm_name &&
-       0==strncmp((tm->tm_name)+1,type->st.st_self,type->st.st_fillp)
+       0==strncmp((tm->tm_name)+1,type->st.st_self,VLEN(type))
        )
       return i;}
-  FEerror("Unrecognized type",0);
+  /* FEerror("Unrecognized type",0); */
   return i;
 
 }
@@ -1366,7 +1371,7 @@ t_from_type(object type) {
    when the sgc is turned on.  FREE_PERCENT is an integer between 0 and 100. 
    */
 
-DEFUN_NEW("ALLOCATE-SGC",object,fSallocate_sgc,SI
+DEFUN("ALLOCATE-SGC",object,fSallocate_sgc,SI
       ,4,4,NONE,OO,II,II,OO,(object type,fixnum min,fixnum max,fixnum free_percent),"") {
 
   int t=t_from_type(type);
@@ -1391,11 +1396,12 @@ DEFUN_NEW("ALLOCATE-SGC",object,fSallocate_sgc,SI
 /* Growth of TYPE will be by at least MIN pages and at most MAX pages.
    It will try to grow PERCENT of the current pages.
    */
-DEFUN_NEW("ALLOCATE-GROWTH",object,fSallocate_growth,SI,5,5,NONE,OO,II,II,OO,
+DEFUN("ALLOCATE-GROWTH",object,fSallocate_growth,SI,5,5,NONE,OO,II,II,OO,
       (object type,fixnum min,fixnum max,fixnum percent,fixnum percent_free),"")
 {int  t=t_from_type(type);
- struct typemanager *tm=tm_of(t);
+ struct typemanager *tm=t<t_other ? tm_of(t) : NULL;
  object res,x,x1,x2,x3;
+ if (!tm) RETURN1(Cnil);
  x=make_fixnum(tm->tm_min_grow);
  x1=make_fixnum(tm->tm_max_grow);
  x2=make_fixnum(tm->tm_growth_percent);
@@ -1416,21 +1422,17 @@ DEFUN_NEW("ALLOCATE-GROWTH",object,fSallocate_growth,SI,5,5,NONE,OO,II,II,OO,
 
 
 
-DEFUN_NEW("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
+DEFUN("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI
+	  ,1,2,NONE,OI,OO,OO,OO,(fixnum npages,...),"") {
 
-  int nargs=VFUN_NARGS;
-  object really_do;
+  object really_do,l=Cnil,f=OBJNULL;
   va_list ap;
-  fixnum npages=fixint(onpages);
+  fixnum nargs=INIT_NARGS(1);
   
-  really_do=Cnil;
-  if (nargs>=2) {
-    va_start(ap,onpages);
-    really_do=va_arg(ap,object);
-    va_end(ap);
-  }
-  
-  CHECK_ARG_RANGE(1,2);
+  va_start(ap,npages);
+  really_do=NEXT_ARG(nargs,ap,l,f,Cnil);
+  va_end(ap);
+
   if  (npages  < 0)
     FEerror("Allocate requires positive argument.", 0);
   if (ncbpage > npages)
@@ -1445,35 +1447,29 @@ DEFUN_NEW("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI,1,2,
 
 }
 
-
-DEFUN_NEW("ALLOCATED-CONTIGUOUS-PAGES",object,fSallocated_contiguous_pages,SI
+DEFUN("ALLOCATED-CONTIGUOUS-PAGES",object,fSallocated_contiguous_pages,SI
        ,0,0,NONE,OO,OO,OO,OO,(void),"")
 {
 	/* 0 args */
 	RETURN1((make_fixnum(ncbpage)));
 }
 
-DEFUN_NEW("MAXIMUM-CONTIGUOUS-PAGES",object,fSmaximum_contiguous_pages,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+DEFUN("MAXIMUM-CONTIGUOUS-PAGES",object,fSmaximum_contiguous_pages,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
   /* 0 args */
   RETURN1((make_fixnum(maxcbpage)));
 }
 
 
-DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
+DEFUN("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,2,NONE,OI,OO,OO,OO,(fixnum npages,...),"") {
 
-  int nargs=VFUN_NARGS;
-  object really_do;
+  object really_do,l=Cnil,f=OBJNULL;
   va_list ap;
-  fixnum npages=fixint(onpages);
+  fixnum nargs=INIT_NARGS(1);
   
-  really_do=Cnil;
-  if (nargs>=2) {
-    va_start(ap,onpages);
-    really_do=va_arg(ap,object);
-    va_end(ap);
-  }
+  va_start(ap,npages);
+  really_do=NEXT_ARG(nargs,ap,l,f,Cnil);
+  va_end(ap);
     
-  CHECK_ARG_RANGE(1,2);
   if (npages  <= 0)
     FEerror("Requires positive arg",0);
   if (npages<nrbpage) npages=nrbpage;
@@ -1486,28 +1482,24 @@ DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,
 
 }
 
-DEFUN_NEW("ALLOCATE",object,fSallocate,SI,2,3,NONE,OO,OO,OO,OO,(object type,object onpages,...),"") {
+DEFUN("ALLOCATE",object,fSallocate,SI
+	  ,2,3,NONE,OO,IO,OO,OO,(object type,fixnum npages,...),"") {
 
-  int nargs=VFUN_NARGS;
-  object really_do;
+  object really_do,l=Cnil,f=OBJNULL;
   va_list ap;
   struct typemanager *tm;
-  fixnum npages=fixint(onpages);
   int t;
+  fixnum nargs=INIT_NARGS(2);
   
-  really_do=Cnil;
-  if (nargs>=3) {
-    va_start(ap,onpages);
-    really_do=va_arg(ap,object);
-    va_end(ap);
-  }
+  va_start(ap,npages);
+  really_do=NEXT_ARG(nargs,ap,l,f,Cnil);
+  va_end(ap);
   
-  CHECK_ARG_RANGE(2,3);
   t= t_from_type(type);
   if (t == t_contiguous) 
-    RETURN1(FUNCALL(2,FFN(fSallocate_contiguous_pages)(make_fixnum(npages),really_do)));
+    RETURN1(FUNCALL(2,FFN(fSallocate_contiguous_pages)(npages,really_do)));
   else if (t==t_relocatable) 
-    RETURN1(FUNCALL(2,FFN(fSallocate_relocatable_pages)(make_fixnum(npages),really_do)));
+    RETURN1(FUNCALL(2,FFN(fSallocate_relocatable_pages)(npages,really_do)));
 
 
   if  (npages <= 0)
@@ -1523,29 +1515,31 @@ DEFUN_NEW("ALLOCATE",object,fSallocate,SI,2,3,NONE,OO,OO,OO,OO,(object type,obje
 
 }
 
-DEFUN_NEW("ALLOCATED-RELOCATABLE-PAGES",object,fSallocated_relocatable_pages,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+DEFUN("ALLOCATED-RELOCATABLE-PAGES",object,fSallocated_relocatable_pages,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
   /* 0 args */
   RETURN1(make_fixnum(nrbpage));
 }
 
-DEFUN_NEW("GET-HOLE-SIZE",object,fSget_hole_size,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+DEFUN("GET-HOLE-SIZE",object,fSget_hole_size,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
   /* 0 args */
   RETURN1(make_fixnum((rb_start-heap_end)>>PAGEWIDTH));
 }
 
-DEFUN_NEW("SET-STARTING-HOLE-DIVISOR",object,fSset_starting_hole_divisor,SI,1,1,NONE,II,OO,OO,OO,(fixnum div),"") {
+DEFUN("SET-STARTING-HOLE-DIVISOR",object,fSset_starting_hole_divisor,SI,1,1,NONE,II,OO,OO,OO,(fixnum div),"") {
   if (div>0 && div <100)
     starting_hole_div=div;
   return (object)starting_hole_div;
 }
   
-DEFUN_NEW("SET-STARTING-RELBLOCK-HEAP-MULTIPLE",object,fSset_starting_relb_heap_multiple,SI,1,1,NONE,II,OO,OO,OO,(fixnum mult),"") {
+DEFUN("SET-STARTING-RELBLOCK-HEAP-MULTIPLE",object,fSset_starting_relb_heap_multiple,SI,1,1,NONE,II,OO,OO,OO,(fixnum mult),"") {
   if (mult>=0)
     starting_relb_heap_mult=mult;
   return (object)starting_relb_heap_mult;
 }
   
-DEFUNM_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
+DEFUNM("SET-HOLE-SIZE",object,fSset_hole_size,SI,1,2,NONE,OI,IO,OO,OO,(fixnum npages,...),"") {
+  fixnum vals=(fixnum)fcall.valp;
+  object *base=vs_top;
 
   RETURN2(make_fixnum((rb_start-heap_end)>>PAGEWIDTH),make_fixnum(reserve_pages_for_signal_handler));
 
@@ -1660,9 +1654,9 @@ malloc_internal(size_t size) {
 
   CHECK_INTERRUPT;
   
-  malloc_list = make_cons(alloc_simple_string(size), malloc_list);
+  malloc_list = make_cons(alloc_string(size), malloc_list);
   malloc_list->c.c_car->st.st_self = alloc_contblock(size);
-  malloc_list->c.c_car->st.st_adjustable=writable_malloc;
+  malloc_list->c.c_car->st.st_writable=writable_malloc;
   
   return(malloc_list->c.c_car->st.st_self);
 
@@ -1728,11 +1722,12 @@ realloc(void *ptr, size_t size) {
     if (x->c.c_car->st.st_self == ptr) {
       x = x->c.c_car;
       if (x->st.st_dim >= size) {
-	x->st.st_fillp = size;
+	VFILLP_SET(x,size);
 	return(ptr);
       } else {
 	x->st.st_self = alloc_contblock(size);
-	x->st.st_fillp = x->st.st_dim = size;
+	x->st.st_dim = size;
+	VSET_MAX_FILLP(x);
 	for (i = 0;  i < size;  i++)
 	  x->st.st_self[i] = ((char *)ptr)[i];
 	return(x->st.st_self);
@@ -1772,7 +1767,7 @@ cfree(void *ptr) {
 #ifdef WANT_VALLOC
 static void *
 memalign(size_t align,size_t size) { 
-  object x = alloc_simple_string(size);
+  object x = alloc_string(size);
   x->st.st_self = ALLOC_ALIGNED(alloc_contblock,size,align);
   malloc_list = make_cons(x, malloc_list);
   return x->st.st_self;
