@@ -97,7 +97,7 @@ FFN(Fsetq)(object form)
 			vs_top = top;
 			if (endp(MMcdr(form)))
 			FEinvalid_form("No value for ~S.", form->c.c_car);
-			setq(MMcar(form),ans=Ieval(MMcadr(form)));
+			setq(MMcar(form),ans=Ieval1(MMcadr(form)));
 			form = MMcddr(form);
 		} while (!endp(form));
 		top[0]=ans;
@@ -116,7 +116,7 @@ FFN(Fpsetq)(object arg)
 		if(endp(MMcdr(arg)))
 			FEinvalid_form("No value for ~S.", arg->c.c_car);
 		
-		top[0] = Ieval(MMcadr(arg));
+		top[0] = Ieval1(MMcadr(arg));
 		vs_top = top + 1;
 	}
 	for (arg = argsv, top = old_top; !endp(arg); arg = MMcddr(arg), top++)
@@ -125,62 +125,75 @@ FFN(Fpsetq)(object arg)
 	vs_push(Cnil);
 }
 
-DEFUNO_NEW("SET",object,fLset,LISP
-   ,2,2,NONE,OO,OO,OO,OO,void,Lset,(object symbol,object value),"")
+DEFUN("SET",object,fLset,LISP,2,2,NONE,OO,OO,OO,OO,(object symbol,object value),"") {
+
+  /* 2 args */
+  if (type_of(symbol) != t_symbol)
+    not_a_symbol(symbol);
+  if ((enum stype)symbol->s.s_stype == stp_constant)
+    FEinvalid_variable("Cannot assign to the constant ~S.",
+		       symbol);
+  symbol->s.s_dbind = value;
+  RETURN1(value);
 
-{
-	/* 2 args */
-	if (type_of(symbol) != t_symbol)
-		not_a_symbol(symbol);
-	if ((enum stype)symbol->s.s_stype == stp_constant)
-		FEinvalid_variable("Cannot assign to the constant ~S.",
-				   symbol);
-	symbol->s.s_dbind = value;
-	RETURN1(value);
 }
 
-DEFUNO_NEW("FSET",object,fSfset,SI
-   ,2,2,NONE,OO,OO,OO,OO,void,siLfset,(object sym,object function),"")
+DEFUN("FUNCTION-NAME",object,fSfunction_name,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
-{
-	/* 2 args */
-	if (type_of(sym) != t_symbol)
-		not_a_symbol(sym);
-	if (sym->s.s_sfdef != NOT_SPECIAL) {
-		if (sym->s.s_mflag) {
-			if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-				sym->s.s_sfdef = NOT_SPECIAL;
-		} else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-			FEerror("~S, a special form, cannot be redefined.",
-				1, sym);
-	}
-	sym = clear_compiler_properties(sym,function);
-	if (sym->s.s_hpack == lisp_package &&
-	    sym->s.s_gfdef != OBJNULL && initflag) {
-		ifuncall2(sLwarn,make_simple_string("~S is being redefined."),
-			 sym);
-	}
-	if (type_of(function) == t_cfun ||
-	    type_of(function) == t_sfun ||
-	    type_of(function) == t_vfun ||
-	    type_of(function) == t_gfun ||
-	    type_of(function) == t_cclosure||
-	    type_of(function) == t_closure ||
-	    type_of(function) == t_afun 
-	    ) {
-		sym->s.s_gfdef = function;
-		sym->s.s_mflag = FALSE;
-	} else if (car(function) == sLspecial)
-		FEerror("Cannot define a special form.", 0);
-	else if (function->c.c_car == sLmacro) {
-		sym->s.s_gfdef = function->c.c_cdr;
-		sym->s.s_mflag = TRUE;
-	} else {
-		sym->s.s_gfdef = function;
-		sym->s.s_mflag = FALSE;
-	}
+  switch(type_of(x)) {
+  case t_function: 
+    x=Cnil;
+    break;
+  default:
+    TYPE_ERROR(x,sLfunction);
+    x=Cnil;
+    break;
+  }
+
+  return x;
+
+}
+
+	
+
+
+DEFUN("FSET",object,fSfset,SI,2,2,NONE,OO,OO,OO,OO,(object sym,object function),"") {
+
+  if (type_of(sym)!=t_symbol)
+    sym=ifuncall1(sSfunid_to_sym,sym);
+  
+  if (sym->s.s_sfdef != NOT_SPECIAL) {
+    if (sym->s.s_mflag) {
+      if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+	sym->s.s_sfdef = NOT_SPECIAL;
+    } else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+      FEerror("~S, a special form, cannot be redefined.",
+	      1, sym);
+  }
+  if (sym->s.s_hpack == lisp_package &&
+      sym->s.s_gfdef != OBJNULL && !raw_image && sLwarn->s.s_gfdef)
+    ifuncall2(sLwarn,make_simple_string("~S is being redefined."),sym);
+  sym = clear_compiler_properties(sym,function);
+  if (type_of(function) == t_function) {
+    sym->s.s_gfdef = function;
+    sym->s.s_mflag = FALSE;
+  } else if (car(function) == sLspecial)
+    FEerror("Cannot define a special form.", 0);
+  else if (function->c.c_car == sSmacro) {
+    function=function->c.c_cdr;
+    sym->s.s_gfdef = function;
+    sym->s.s_mflag = TRUE;
+  } else {
+    sym->s.s_gfdef = function;
+    sym->s.s_mflag = FALSE;
+  }
+  
+  sym->s.s_sfdef=NOT_SPECIAL;/*FIXME?*/
+  if (function->fun.fun_plist!=Cnil)
+    function->fun.fun_plist->c.c_cdr->c.c_cdr->c.c_cdr->c.c_cdr->c.c_cdr->c.c_car=sym;/*FIXME*/
+  
+  RETURN1(function);
 
-	RETURN1(function);
 }
 #ifdef STATIC_FUNCTION_POINTERS
 object
@@ -190,102 +203,108 @@ fSfset(object sym,object function) {
 #endif
 
 static void
-FFN(Fmultiple_value_setq)(object form)
-{
-	object vars;
-	int n, i;
+FFN(Fmultiple_value_setq)(object form) {
 
-	if (endp(form) || endp(form->c.c_cdr) ||
-	    !endp(form->c.c_cdr->c.c_cdr))
-	    FEinvalid_form("~S is an illegal argument to MULTIPLE-VALUE-SETQ",
-			   form);
-	vars = form->c.c_car;
+  object vars,*vals;
+  int n, i;
+  
+  if (endp(form) || endp(form->c.c_cdr) ||
+      !endp(form->c.c_cdr->c.c_cdr))
+    FEinvalid_form("~S is an illegal argument to MULTIPLE-VALUE-SETQ",form);
+
+  vars = form->c.c_car;
+  vals=ZALLOCA(MULTIPLE_VALUES_LIMIT*sizeof(*vals));
+
+  vals[0]=Ievaln(form->c.c_cdr->c.c_car,vals+1);
+  for (i=0,n=vs_top-vals;!endp(vars);i++,vars=vars->c.c_cdr)
+    setq(vars->c.c_car,i<n ? vals[i] : Cnil);
+
+  vs_base[0]=vals[0];
+  vs_top=vs_base+1;
 
-	fcall.values[0]=Ieval(form->c.c_cdr->c.c_car);
-	n = fcall.nvalues;
-	
-	for (i = 0;  !endp(vars);  i++, vars = vars->c.c_cdr)
-		if (i < n)
-			setq(vars->c.c_car, fcall.values[i]);
-		else
-			setq(vars->c.c_car, Cnil);
-	vs_base[0]=fcall.values[0];
-	vs_top = vs_base+1;
 }
 
-DEFUNO_NEW("MAKUNBOUND",object,fLmakunbound,LISP
-   ,1,1,NONE,OO,OO,OO,OO,void,Lmakunbound,(object sym),"")
+DEFUN("MAKUNBOUND",object,fLmakunbound,LISP,1,1,NONE,OO,OO,OO,OO,(object sym),"") {
+
+  if (type_of(sym) != t_symbol)
+    not_a_symbol(sym);
+  if ((enum stype)sym->s.s_stype == stp_constant)
+    FEinvalid_variable("Cannot unbind the constant ~S.",
+		       sym);
+  sym->s.s_dbind = OBJNULL;
+  RETURN1(sym);
 
-{
-	/* 1 args */
-	if (type_of(sym) != t_symbol)
-		not_a_symbol(sym);
-	if ((enum stype)sym->s.s_stype == stp_constant)
-		FEinvalid_variable("Cannot unbind the constant ~S.",
-				   sym);
-	sym->s.s_dbind = OBJNULL;
-	RETURN1(sym);
 }
 
 object sStraced;
 
-DEFUNO_NEW("FMAKUNBOUND",object,fLfmakunbound,LISP
-   ,1,1,NONE,OO,OO,OO,OO,void,Lfmakunbound,(object sym),"")
+DEFUN("FMAKUNBOUND",object,fLfmakunbound,LISP,1,1,NONE,OO,OO,OO,OO,(object sym),"") {
+
+  object rsym;
+
+  rsym=type_of(sym)==t_symbol ? sym : ifuncall1(sSfunid_to_sym,sym);
+
+  if (rsym->s.s_sfdef != NOT_SPECIAL) {
+    if (rsym->s.s_mflag) {
+      if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+	rsym->s.s_sfdef = NOT_SPECIAL;
+    } else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
+      FEerror("~S, a special form, cannot be redefined.", 1, rsym);
+  }
+  remf(&(rsym->s.s_plist),sStraced);
+  clear_compiler_properties(rsym,Cnil);
+  if (rsym->s.s_hpack == lisp_package &&
+      rsym->s.s_gfdef != OBJNULL && !raw_image) {
+    ifuncall2(sLwarn, make_simple_string("~S is being redefined."), rsym);
+  }
+
+  rsym->s.s_gfdef = OBJNULL;
+  rsym->s.s_mflag = FALSE;
+  RETURN1(sym);
 
-{
-	/* 1 args */
-	if(type_of(sym) != t_symbol)
-		not_a_symbol(sym);
-	if (sym->s.s_sfdef != NOT_SPECIAL) {
-		if (sym->s.s_mflag) {
-			if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-				sym->s.s_sfdef = NOT_SPECIAL;
-		} else if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
-			FEerror("~S, a special form, cannot be redefined.",
-				1, sym);
-	}
-	remf(&(sym->s.s_plist),sStraced);
-	clear_compiler_properties(sym,Cnil);
-	if (sym->s.s_hpack == lisp_package &&
-	    sym->s.s_gfdef != OBJNULL && initflag) {
-		ifuncall2(sLwarn, make_simple_string(
-			"~S is being redefined."), sym);
-	}
-	sym->s.s_gfdef = OBJNULL;
-	sym->s.s_mflag = FALSE;
-	RETURN1(sym);
 }
 
 static void
-FFN(Fsetf)(object form)
-{
-	object result,*t,*t1;
-	if (endp(form)) {
-		vs_base = vs_top;
-		vs_push(Cnil);
-	} else {
-		object *top = vs_top;
-		do {
-			vs_top = top;
-			if (endp(MMcdr(form)))
-			FEinvalid_form("No value for ~S.", form->c.c_car);
-			result = setf(MMcar(form), MMcadr(form));
-			form = MMcddr(form);
-		} while (!endp(form));
-		t=vs_base;
-		t1=vs_top;
-		vs_top = vs_base = top;
-		for (;t<t1;t++)
-		  vs_push(*t);
+FFN(Fsetf)(object form) {
+
+  object result=Cnil,*top=vs_top;
+
+  for (;!endp(form);form=MMcddr(form)) {
+    vs_top = top;
+    if (endp(MMcdr(form)))
+      FEinvalid_form("No value for ~S.", form->c.c_car);
+    result=setf(MMcar(form), MMcadr(form));
+  }
+  vs_base=top;
+  vs_base[0]=result;
+  vs_top=vs_base+1;
 
-	}
 }
+    
+/*   if (endp(form)) { */
+/*     vs_base = vs_top; */
+/*     vs_push(Cnil); */
+/*   } else { */
+/*     object *top = vs_top; */
+/*     do { */
+/*       vs_top = top; */
+/*       if (endp(MMcdr(form))) */
+/* 	FEinvalid_form("No value for ~S.", form->c.c_car); */
+/*       result = setf(MMcar(form), MMcadr(form)); */
+/*       form = MMcddr(form); */
+/*     } while (!endp(form)); */
+/*     vs_top = vs_base = top; */
+/*     vs_base[0]=result; */
+/*     vs_top=vs_base+1; */
+    
+/*   } */
+/* } */
 
 #define	eval_push(form)  \
 {  \
 	object *old_top = vs_top;  \
   \
-	*old_top = Ieval(form);  \
+	*old_top = Ieval1(form);  \
 	vs_top = old_top + 1;  \
 }
 
@@ -297,7 +316,7 @@ setf(object place, object form)
 	void (*f)();
 	object args;
 	object x,result,y;
-	int i;
+	int i,nka=0;
 /*  	extern void siLaset(void); */
 /*  	extern void siLsvset(void); */
 	extern void siLelt_set();
@@ -305,71 +324,95 @@ setf(object place, object form)
 /*  	extern void siLfill_pointer_set(void); */
 	extern void siLhash_set();
 
-	if (type_of(place) != t_cons) {
-		setq(place, result=Ieval(form));
-		vs_top=vs_base+1;
-		return result;
+	if (!consp(place)) {
+	  setq(place, result=Ieval1(form));
+	  return result;
 	}
+	
 	fun = place->c.c_car;
 	if (type_of(fun) != t_symbol)
 		goto OTHERWISE;
 	args = place->c.c_cdr;
+
+	{
+	  object p=find_package(make_simple_string("COMMON-LISP"));
+	  char *s;
+
+	  if (fun->s.s_hpack==p && fun->s.s_name->st.st_self[0]=='C' && fun->s.s_name->st.st_self[VLEN(fun->s.s_name)-1]=='R' && VLEN(fun->s.s_name)!=3) {
+
+	    s=alloca(VLEN(fun->s.s_name));
+	    s[0]='C';
+	    memcpy(s+1,fun->s.s_name->st.st_self+2,VLEN(fun->s.s_name)-2);
+	    s[VLEN(fun->s.s_name)-1]=0;
+	    
+	    fun=fun->s.s_name->st.st_self[1]=='A' ? sLcar : sLcdr;
+	    args=MMcons(MMcons(find_symbol(make_simple_string(s),p),MMcons(args->c.c_car,Cnil)),Cnil);
+
+	  }
+	} /*FIXME*/
+	  
 	if (fun == sLget) {
-	  object sym,val,key;
-	  sym = Ieval(car(args));
-	  key = Ieval(car(Mcdr(args)));
-	  val = Ieval(form);
+            object sym,val,key,deflt1;
+	  sym = Ieval1(car(args));
+	  key = Ieval1(car(Mcdr(args)));
+          deflt1 = Mcddr(args);
+          if (consp(deflt1))
+	    Ieval1(car(deflt1));
+	  val = Ieval1(form);
 	  return putprop(sym,val,key); 
 	}
-	if (fun == sLgetf) 
-	  Ieval(Mcaddr(args));
-	if (fun == sLaref) { f = siLaset; goto EVAL; }
-	if (fun == sLsvref) { f = siLsvset; goto EVAL; }
+
+	if (fun == find_symbol(str("SYMBOL-FUNCTION"),lisp_package))
+	  return Ieval1(MMcons(find_symbol(str("FSET"),system_package),MMcons(MMcar(args),MMcons(form,Cnil))));
+	if (fun == sLsbit)
+	  return Ieval1(MMcons(find_symbol(str("ASET"),system_package),MMcons(form,args)));
+	if (fun == sLaref) 
+	  return Ieval1(MMcons(find_symbol(str("ASET"),system_package),MMcons(form,args)));
+	if (fun == sLsvref)
+	  return Ieval1(MMcons(find_symbol(str("SVSET"),system_package),append(args,MMcons(form,Cnil))));
 	if (fun == sLelt) { f = siLelt_set; goto EVAL; }
 	if (fun == sLchar) { f = siLchar_set; goto EVAL; }
 	if (fun == sLschar) { f = siLchar_set; goto EVAL; }
-	if (fun == sLfill_pointer) { f = siLfill_pointer_set; goto EVAL; }
-	if (fun == sLgethash) { f = siLhash_set; goto EVAL; }
+	if (fun == sLfill_pointer) 
+	  return Ieval1(MMcons(find_symbol(str("FILL-POINTER-SET"),system_package),append(args,MMcons(form,Cnil))));
+	if (fun == sLgethash) 
+	  return Ieval1(MMcons(find_symbol(str("HASH-SET"),system_package),append(args,MMcons(form,Cnil))));
 	if (fun == sLcar) {
-		x = Ieval(Mcar(args));
-		result = Ieval(form);
-		if (type_of(x) != t_cons)
+		x = Ieval1(Mcar(args));
+		result = Ieval1(form);
+		if (!consp(x))
 			FEerror("~S is not a cons.", 1, x);
 		Mcar(x) = result;
 		return result;
 	}
 	if (fun == sLcdr) {
-		x = Ieval(Mcar(args));
-		result = Ieval(form);
-		if (type_of(x) != t_cons)
+		x = Ieval1(Mcar(args));
+		result = Ieval1(form);
+		if (!consp(x))
 			FEerror("~S is not a cons.", 1, x);
 		Mcdr(x) = result;
 		return result;
 	}
 
 	x = getf(fun->s.s_plist, sSstructure_access, Cnil);
-	if (x == Cnil || type_of(x) != t_cons)
+	if (x == Cnil || !consp(x))
 		goto OTHERWISE;
 	if (getf(fun->s.s_plist, sSsetf_lambda, Cnil) == Cnil)
 		goto OTHERWISE;
 	if (type_of(x->c.c_cdr) != t_fixnum)
 		goto OTHERWISE;
 	i = fix(x->c.c_cdr);
-/*
-	if (i < 0)
-		goto OTHERWISE;
-*/
 	x = x->c.c_car;
-	y = Ieval(Mcar(args));
-	result = Ieval(form);
+	y = Ieval1(Mcar(args));
+	result = Ieval1(form);
 	if (x == sLvector) {
-		if (type_of(y) != t_vector || i >= y->v.v_fillp)
-			goto OTHERWISE;
-		y->v.v_self[i] = result;
+	  if (!TS_MEMBER(type_of(y),TS(t_vector)|TS(t_simple_vector)) || i >= VLEN(y))/*FIXME*/
+	    goto OTHERWISE;
+	  y->v.v_self[i] = result;
 	} else if (x == sLlist) {
 		for (x = y;  i > 0;  --i)
 			x = cdr(x);
-		if (type_of(x) != t_cons)
+		if (!consp(x))
 			goto OTHERWISE;
 		x->c.c_car = result;
 	} else {
@@ -378,24 +421,21 @@ setf(object place, object form)
 	return result;
 
 EVAL:
-	for (;  !endp(args);  args = args->c.c_cdr) {
-		eval_push(args->c.c_car);
+	for (;!endp(args);args=args->c.c_cdr)
+	  eval_push(args->c.c_car);
+	/* if (f!=siLaset) eval_push(form); */
+	if (nka && vs_top-vs>nka) {
+	  vs[nka]=vs_base[0];
+	  vs_top=vs+nka+1;
 	}
-	eval_push(form);
+	nka=0;
 	vs_base = vs;
 	(*f)();
 	return vs_base[0];
 
 OTHERWISE:
 	vs_base = vs_top;
-	vs_push(sLsetf);
-	vs_push(place);
-	vs_push(form);
-	result=vs_top[-1];
-	vs_push(Cnil);
-	stack_cons();
-	stack_cons();
-	stack_cons();
+	vs_push(list(3,sLsetf,place,result=form));
 /***/
 #define VS_PUSH_ENV \
 	if(lex_env[1]){ \
@@ -406,7 +446,7 @@ OTHERWISE:
 	if (!sLsetf->s.s_mflag || sLsetf->s.s_gfdef == OBJNULL)
 		FEerror("Where is SETF?", 0);
 	funcall(sLsetf->s.s_gfdef);
-	return Ieval(vs_base[0]);
+	return Ieval1(vs_base[0]);
 }
 
 static void
@@ -419,7 +459,7 @@ FFN(Fpush)(object form)
 	if (!endp(MMcddr(form)))
 		FEtoo_many_argumentsF(form);
 	var = MMcadr(form);
-	if (type_of(var) != t_cons) {
+	if (!consp(var)) {
 		eval(MMcar(form));
 		form = vs_base[0];
 		eval(var);
@@ -428,9 +468,7 @@ FFN(Fpush)(object form)
 		return;
 	}
 	vs_base = vs_top;
-	vs_push(sLpush);
-	vs_push(form);
-	stack_cons();
+	vs_push(make_cons(sLpush,form));
 /***/
          VS_PUSH_ENV ;
 /***/
@@ -450,16 +488,14 @@ FFN(Fpop)(object form)
 	if (!endp(MMcdr(form)))
 		FEtoo_many_argumentsF(form);
 	var = MMcar(form);
-	if (type_of(var) != t_cons) {
+	if (!consp(var)) {
 		eval(var);
 		setq(var, cdr(vs_base[0]));
 		vs_base[0] = car(vs_base[0]);
 		return;
 	}
 	vs_base = vs_top;
-	vs_push(sLpop);
-	vs_push(form);
-	stack_cons();
+	vs_push(make_cons(sLpop,form));
 /***/
 	VS_PUSH_ENV ;
 /***/
@@ -480,7 +516,7 @@ FFN(Fincf)(object form)
 	if (!endp(MMcdr(form)) && !endp(MMcddr(form)))
 		FEtoo_many_argumentsF(form);
 	var = MMcar(form);
-	if (type_of(var) != t_cons) {
+	if (!consp(var)) {
 		if (endp(MMcdr(form))) {
 			eval(var);
 			vs_base[0] = one_plus(vs_base[0]);
@@ -495,9 +531,7 @@ FFN(Fincf)(object form)
 		return;
 	}
 	vs_base = vs_top;
-	vs_push(sLincf);
-	vs_push(form);
-	stack_cons();
+	vs_push(make_cons(sLincf,form));
 /***/
 	VS_PUSH_ENV ;
 /***/
@@ -518,7 +552,7 @@ FFN(Fdecf)(object form)
 	if (!endp(MMcdr(form)) && !endp(MMcddr(form)))
 		FEtoo_many_argumentsF(form);
 	var = MMcar(form);
-	if (type_of(var) != t_cons) {
+	if (!consp(var)) {
 		if (endp(MMcdr(form))) {
 			eval(var);
 			vs_base[0] = one_minus(vs_base[0]);
@@ -533,9 +567,7 @@ FFN(Fdecf)(object form)
 		return;
 	}
 	vs_base = vs_top;
-	vs_push(sLdecf);
-	vs_push(form);
-	stack_cons();
+	vs_push(make_cons(sLdecf,form));
 /***/
 	VS_PUSH_ENV ;
 /***/
@@ -546,21 +578,9 @@ FFN(Fdecf)(object form)
 }
 
 
-/* object */
-/* clear_compiler_properties(object sym, object code) */
-/* { object tem; */
-/*   VFUN_NARGS=2; fSuse_fast_links(Cnil,sym); */
-/*   tem = getf(sym->s.s_plist,sStraced,Cnil); */
-/*   if (sSAinhibit_macro_specialA && sSAinhibit_macro_specialA->s.s_dbind != Cnil) */
-/*     (void)ifuncall2(sSclear_compiler_properties, sym,code); */
-/*   if (tem != Cnil) return tem; */
-/*   return sym; */
-  
-/* } */
-
 DEF_ORDINARY("CLEAR-COMPILER-PROPERTIES",sSclear_compiler_properties,SI,"");
 
-DEFUN_NEW("CLEAR-COMPILER-PROPERTIES",object,fSclear_compiler_properties,SI
+DEFUN("CLEAR-COMPILER-PROPERTIES",object,fSclear_compiler_properties,SI
    ,2,2,NONE,OO,OO,OO,OO,(object x0,object x1),"")
 
 {
@@ -568,6 +588,20 @@ DEFUN_NEW("CLEAR-COMPILER-PROPERTIES",object,fSclear_compiler_properties,SI
   RETURN1(Cnil);
 }
 
+DEFUN("EMERGENCY-FSET",object,fSemergency_fset,SI
+   ,2,2,NONE,OO,OO,OO,OO,(object sym,object function),"")
+
+{
+
+  if (type_of(sym)!=t_symbol || sym->s.s_sfdef!=NOT_SPECIAL || consp(function)) {
+    printf("Emergency fset: skipping %-.*s\n",(int)VLEN(sym->s.s_name),sym->s.s_name->st.st_self);
+    RETURN1(Cnil);
+  }
+  sym->s.s_gfdef=function;
+  sym->s.s_mflag=FALSE;
+  RETURN1(Ct);
+}
+
 DEF_ORDINARY("AREF",sLaref,LISP,"");
 DEF_ORDINARY("CAR",sLcar,LISP,"");
 DEF_ORDINARY("CDR",sLcdr,LISP,"");
