@@ -44,9 +44,8 @@ read.d: normalize_big_to_object
 
 #define remainder gclremainder
 #define NEED_MP_H
-#include <stdlib.h>
-#include <string.h>
 #include "include.h"
+#include "num_include.h"
 
 #ifdef STATIC_FUNCTION_POINTERS
 static void* alloc_relblock_static (size_t n) {return alloc_relblock (n);}
@@ -56,8 +55,30 @@ static void* alloc_contblock_static(size_t n) {return alloc_contblock(n);}
 void* (*gcl_gmp_allocfun)(size_t)=FFN(alloc_relblock);
 int gmp_relocatable=1;
 
+DEFUN("INTEGER-QUOTIENT-REMAINDER_1",object,fSinteger_quotient_remainder_1,SI,4,4,NONE,OO,OO,IO,OO,(object r,object x,object y,fixnum d),"") {
 
-DEFUN_NEW("SET-GMP-ALLOCATE-RELOCATABLE",object,fSset_gmp_allocate_relocatable,SI,1,1,NONE,OO,OO,OO,OO,
+  integer_quotient_remainder_1(x,y,&r->c.c_car,&r->c.c_cdr,d);
+
+  RETURN1(r);
+
+}
+
+
+
+DEFUN("MBIGNUM2",object,fSbignum2,SI,2,2,NONE,OI,IO,OO,OO,(fixnum h,fixnum l),"") {
+
+  object x = new_bignum();
+
+  mpz_set_si(MP(x),h);
+  mpz_mul_2exp(MP(x),MP(x),8*sizeof(x));
+  mpz_add_ui(MP(x),MP(x),l);
+
+  RETURN1(normalize_big(x));
+
+}
+
+
+DEFUN("SET-GMP-ALLOCATE-RELOCATABLE",object,fSset_gmp_allocate_relocatable,SI,1,1,NONE,OO,OO,OO,OO,
       (object flag),"Set the allocation to be relocatble ")
 {
   if (flag == Ct) {
@@ -70,32 +91,6 @@ DEFUN_NEW("SET-GMP-ALLOCATE-RELOCATABLE",object,fSset_gmp_allocate_relocatable,S
   RETURN1(flag);
 }
 
-#ifndef GMP_USE_MALLOC
-object big_gcprotect;
-object big_fixnum1;
-
-#include "gmp.c"
-
-void
-gcl_init_big1(void) {
-
-  mp_set_memory_functions( gcl_gmp_alloc,gcl_gmp_realloc,gcl_gmp_free);
-  jmp_gmp=0;
-
-#if __GNU_MP_VERSION > 4 || (__GNU_MP_VERSION == 4 && __GNU_MP_VERSION_MINOR >= 2)
-  Mersenne_Twister_Generator_Noseed.b=__gmp_randget_mt;
-  Mersenne_Twister_Generator_Noseed.c=__gmp_randclear_mt;
-  Mersenne_Twister_Generator_Noseed.d=__gmp_randiset_mt;
-#endif
-
-}
-
-#else
-gcl_init_big1()
-{
-}
-#endif
-
 #ifdef GMP
 #include "gmp_big.c"
 #else
@@ -119,6 +114,7 @@ void zero_big(object x)
   ZERO_BIG(x);
 }
 
+
 #ifndef HAVE_MP_COERCE_TO_STRING
 
 double digitsPerBit[37]={ 0,0,
@@ -167,7 +163,7 @@ coerce_big_to_string(x,printbase)
  int sign=big_sign(x);
  object b;
  int size = (int)((ceil(MP_SIZE_IN_BASE2(MP(x))* digitsPerBit[printbase]))+.01);
- char *q,*p = alloca(size+5);
+ char *q,*p = ZALLOCA(size+5);
  q=p;
  if(sign<=0) {
    *q++ = '-';
@@ -181,7 +177,6 @@ coerce_big_to_string(x,printbase)
   object ans = alloc_simple_string(q-p);
   ans->ust.ust_self=alloc_relblock(ans->ust.ust_dim);
   bcopy(ans->ust.ust_self,p,ans->ust.ust_dim);
-  ans->ust.ust_fillp=ans->ust.ust_dim-1;
   return ans;
 }
 
