@@ -334,7 +334,7 @@ SEARCH_DECLARE:
 		form = macro_expand(form);
 		vs_head = form;
 
-		if (type_of(form) == t_string) {
+		if (stringp(form)) {
 			if (endp(body->c.c_cdr))
 				break;
 			continue;
@@ -364,7 +364,9 @@ SEARCH_DECLARE:
 		} else if (optional[i].opt_svar == v) {
 			optional[i].opt_svar_spp = Ct;
 			special_processed = TRUE;
-		}
+		} /* else if (optional[i].opt_init == v) */
+/* 		  special_processed = TRUE; */
+		
 	if (rest_flag && rest->rest_var == v) {
 		rest->rest_spp = Ct;
 		special_processed = TRUE;
@@ -376,12 +378,14 @@ SEARCH_DECLARE:
 		} else if (keyword[i].key_svar == v) {
 			keyword[i].key_svar_spp = Ct;
 			special_processed = TRUE;
-		}
+		} /* else if (keyword[i].key_init == v) */
+/* 		  special_processed = TRUE; */
 	for (i = 0;  i < naux;  i++)
 		if (aux[i].aux_var == v) {
 			aux[i].aux_spp = Ct;
 			special_processed = TRUE;
-		}
+		} /* else if (aux[i].aux_init == v) */
+/* 		  special_processed = TRUE; */
 	if (special_processed)
 		continue;
 	s[0] = MMcons(MMcons(v, Cnil), s[0]);
@@ -524,7 +528,7 @@ REQUIRED_ONLY:
 		/*  MACRO EXPANSION  */
 		vs_head = form = macro_expand(form);
 
-		if (type_of(form) == t_string) {
+		if (stringp(form)) {
 			if (endp(body->c.c_cdr))
 				break;
 			continue;
@@ -651,7 +655,7 @@ find_special(object body, struct bind_temp *start, struct bind_temp *end,object
 		form = macro_expand(form);
 		vs_head = form;
 
-		if (type_of(form) == t_string) {
+		if (stringp(form)) {
 			if (endp(body->c.c_cdr))
 				break;
 			continue;
@@ -970,7 +974,7 @@ parse_key_new_new(int n, object *base, struct key *keys, object first, va_list a
      new = new -2;
      k = *new;
      while(--i >= 0)
-       {if (*(*(ke++)).o == k)
+       {if ((*(ke++)).o == k)
 	  {base[i]= new[1];
 	   n=n-2;
 	   goto top;
@@ -1057,7 +1061,8 @@ parse_key_rest_new(object rest, int n, object *base, struct key *keys, object fi
     
  new = new + n ;
   {int j=keys->n;
-   while (--j >=0) base[j]=*keys->defaults[j].o;
+   object *p= (object *)(keys->defaults);
+   while (--j >=0) base[j]=p[j];
  }
  {if (n==0){ return 0;}
  {int allow = keys->allow_other_keys;
@@ -1077,7 +1082,7 @@ parse_key_rest_new(object rest, int n, object *base, struct key *keys, object fi
      new = new -2;
      k = *new;
      while(--i >= 0)
-       {if (*(*(ke++)).o == k)
+       {if ((*(ke++)).o == k)
 	  {base[i]= new[1];
 	   n=n-2;
 	   goto top;
@@ -1096,19 +1101,17 @@ parse_key_rest_new(object rest, int n, object *base, struct key *keys, object fi
   return -1;
 }}}
 
-static object foo[2]={Cnil,OBJNULL};
-  
 void
 set_key_struct(struct key *ks, object data)
 {int i=ks->n;
  while (--i >=0)
-   {ks->keys[i].o =   data->cfd.cfd_self+ks->keys[i].i;
+   {ks->keys[i].o =   data->cfd.cfd_self[ ks->keys[i].i ];
     if (ks->defaults != (void *)Cstd_key_defaults)
       {fixnum m=ks->defaults[i].i;
         ks->defaults[i].o=
-	  (m==-2 ? foo :
-	   m==-1 ? foo+1 :
-	   data->cfd.cfd_self+m);}
+	  (m==-2 ? Cnil :
+	   m==-1 ? OBJNULL :
+	   data->cfd.cfd_self[m]);}
 }}
 
 #undef AUX
