@@ -29,60 +29,80 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
      
 #include <string.h>
 #include <signal.h>
-#include <stdlib.h>
 #define NEED_MP_H
 #include "include.h"
 #define dcheck_type(a,b) check_type(a,b)
 
 #include "page.h"
 
-#ifdef HAVE_AOUT
-#undef ATT
-#undef BSD
-#ifndef HAVE_ELF
-#ifndef HAVE_FILEHDR
-#define BSD
-#endif
-#endif
-#include HAVE_AOUT
-#endif
-
-DEFUNO_NEW("SPECIALP",object,fSspecialp,SI
-       ,1,1,NONE,OO,OO,OO,OO,void,siLspecialp,(object sym),"")
-{
-	/* 1 args */
-	if (type_of(sym) == t_symbol &&
-	    (enum stype)sym->s.s_stype == stp_special)
-		sym = Ct;
-	else
-		sym = Cnil;
-	RETURN1(sym);
+DEFUN("SPECIALP",object,fSspecialp,SI,1,1,NONE,OO,OO,OO,OO,(object sym),"") {
+  RETURN1((type_of(sym) == t_symbol && (enum stype)sym->s.s_stype == stp_special) ? Ct : Cnil);
 }
 
+DEFUN("BIG-TO-DOUBLE",object,fSbig_to_double,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_bignum)
+    TYPE_ERROR(x,sLbignum);
+  RETURN1(make_longfloat(big_to_double(x)));
+}
+
+DEFUN("LONG-TO-SHORT",object,fSlong_to_short,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_longfloat)
+    TYPE_ERROR(x,sLlong_float);
+  RETURN1(make_shortfloat((float)lf(x)));
+}
+
+DEFUN("COMPLEX-REAL",object,fScomplex_real,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_complex)
+    TYPE_ERROR(x,sLcomplex);
+  RETURN1(x->cmp.cmp_real);
+}
+
+DEFUN("COMPLEX-IMAG",object,fScomplex_imag,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_complex)
+    TYPE_ERROR(x,sLcomplex);
+  RETURN1(x->cmp.cmp_imag);
+}
+
+DEFUN("RATIO-NUMERATOR",object,fSratio_numerator,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_ratio)
+    TYPE_ERROR(x,sLratio);
+  RETURN1(x->rat.rat_num);
+}
+
+DEFUN("RATIO-DENOMINATOR",object,fSratio_denominator,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  if (type_of(x)!=t_ratio)
+    TYPE_ERROR(x,sLratio);
+  RETURN1(x->rat.rat_den);
+}
+
+DEFUN("C-TYPE",object,fSc_type,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  RETURN1(make_fixnum(type_of(x)));
+}
+DEFCONST("C-TYPE-MAX", sSc_type_max,SI,make_fixnum(t_end-1),"");
+
 DEF_ORDINARY("DEBUGGER",sSdebugger,SI,"");
 
-DEFUN_NEW("DEFVAR1",object,fSdefvar1,SI
-       ,2,3,NONE,OO,OO,OO,OO,(object sym,object val,...),"")
-{	int n=VFUN_NARGS;
-	object doc;
-	va_list ap;
-	{ va_start(ap,val);
-	  if (n>=3) doc=va_arg(ap,object);else goto LDEFAULT3;
-	  goto LEND_VARARG;
-	LDEFAULT3: doc = Cnil;
-	LEND_VARARG: va_end(ap);}
+DEFUN("DEFVAR1",object,fSdefvar1,SI
+	  ,2,3,NONE,OO,OO,OO,OO,(object sym,object val,...),"") {
 
-	CHECK_ARG_RANGE(2,3);
-	if(sym->s.s_dbind==0 && n > 1)
-	  sym->s.s_dbind= val;
-	sym->s.s_stype=(short)stp_special;
-	if(n > 2)
-	  putprop(sym,doc,sSvariable_documentation);
-	RETURN1(sym);
-      }
+  object doc,l=Cnil,f=OBJNULL;
+  va_list ap;
+  fixnum n=INIT_NARGS(2);
+  
+  va_start(ap,val);
+  doc=NEXT_ARG(n,ap,l,f,Cnil);
+  va_end(ap);
+  
+  if (sym->s.s_dbind==OBJNULL && n>1)
+    sym->s.s_dbind= val;
+  sym->s.s_stype=(short)stp_special;
+  if (n>2)
+    putprop(sym,doc,sSvariable_documentation);
+  RETURN1(sym);
+}
 
 
-DEFUN_NEW("DEBUG",object,fLdebug,LISP
+DEFUN("DEBUGGER",object,fSdebugger,SI
        ,2,2,NONE,OO,OO,OO,OO,(object sym,object val),"")
 { /* 2 args */
   putprop(sym,val,sSdebugger);
@@ -90,7 +110,7 @@ DEFUN_NEW("DEBUG",object,fLdebug,LISP
 }
 
 
-DEFUN_NEW("SETVV",object,fSsetvv,SI
+DEFUN("SETVV",object,fSsetvv,SI
        ,2,2,NONE,OO,OO,OO,OO,(object index,object val),"")
 { /* 2 args */
   if(type_of(sSPmemory->s.s_dbind)==t_cfdata)
@@ -99,8 +119,8 @@ DEFUN_NEW("SETVV",object,fSsetvv,SI
   RETURN1(index);
 }
 
-DEF_ORDINARY("%MEMORY",sSPmemory,SI,"");
-DEF_ORDINARY("%INIT",sSPinit,SI,"");
+DEFVAR("%MEMORY",sSPmemory,SI,OBJNULL,"");
+DEFVAR("%INIT",sSPinit,SI,OBJNULL,"");
 
 /* void Lidentity(void); */
 void
@@ -168,6 +188,71 @@ gcl_init_cmpaux(void)
 	Conversions to C
 */
 
+
+dcomplex
+object_to_dcomplex(object x) {
+  
+  dcomplex d=0;
+  
+  switch(type_of(x)) {
+  case t_fixnum:
+    d=fix(x);
+    break;
+  case t_bignum:
+    d=mpz_get_si(MP(x));
+    break;
+  case t_character:
+    d=char_code(x);
+    break;
+  case t_ratio:
+    d=number_to_double(x);
+    break;
+  case t_shortfloat:
+    d=sf(x);
+    break;
+  case t_longfloat:
+    d=lf(x);
+    break;
+  case t_complex:
+    d=(double)object_to_dcomplex(x->cmp.cmp_real)+I*(double)object_to_dcomplex(x->cmp.cmp_imag);
+    break;
+  default:
+    FEcannot_coerce(sLfloat,x);
+    break;
+  }
+  
+  return d;
+  
+}
+
+
+void *
+object_to_pointer(object x) {
+  
+  void *d=0;
+  
+  switch(type_of(x)) {
+  case t_simple_vector:
+  case t_simple_bitvector:
+  case t_vector:
+  case t_bitvector:
+  case t_symbol:
+  case t_simple_string:
+  case t_string:
+  case t_array:
+  case t_character:
+    d=x->v.v_self;
+    break;
+  default:
+    FEcannot_coerce(sLfloat,x);
+    break;
+  }
+  
+  return d;
+  
+}
+
+
 char
 object_to_char(object x)
 {
@@ -176,14 +261,11 @@ object_to_char(object x)
 	case t_fixnum:
 		c = fix(x);  break;
 	case t_bignum:
-	  {object *to = vs_top;
-	  vs_push(x);
-	  vs_push(small_fixnum(0xff));
-	  Llogand();
-	  x = vs_base[0];
-	  vs_top = to;
-	  c = (char) fix(x);
-	  break;
+	  {
+	    fixnum fx;
+	    fx=mpz_get_si(MP(x));
+	    c=fx&0xff;
+	    break;
 	  }
 	case t_character:
 		c = char_code(x);  break;
@@ -229,7 +311,7 @@ object_to_fixnum(object x)
 	case t_fixnum:
 	  i = fix(x);  break;
 	case t_bignum:
-	  i = number_to_double(x);
+	  i = mpz_get_si(MP(x)); break;
 	  break;
 	case t_ratio:
 	  i = number_to_double(x);  break;
@@ -238,11 +320,15 @@ object_to_fixnum(object x)
 	case t_longfloat:
 	  i = lf(x);  break;
 	default:
-	  FEerror("~S cannot be coerce to a C int.", 1, x);
+	  FEcannot_coerce(sLinteger,x);
 	}
 	return(i);
 }
 
+fixnum object_to_long(object x) {return object_to_fixnum(x);}
+fixnum object_to_short(object x) {return object_to_fixnum(x);}
+
+
 float 
 object_to_float(object x) 
 { 
@@ -261,7 +347,71 @@ object_to_float(object x)
 	case t_longfloat: 
 		f = lf(x);  break; 
 	default: 
-		FEerror("~S cannot be coerce to a C float.", 1, x); 
+		FEerror("~S cannot be coerce to a C float.", 1, x);
+	} 
+	return(f); 
+} 
+
+object
+make_fcomplex(fcomplex x) {
+
+  return make_complex(make_shortfloat(creal(x)),make_shortfloat(cimag(x)));
+
+}
+
+fcomplex 
+object_to_fcomplex(object x) 
+{ 
+	fcomplex f=0.0; 
+
+	switch (type_of(x)) { 
+	case t_character: 
+		f = char_code(x);  break; 
+	case t_fixnum: 
+		f = fix(x);  break; 
+	case t_bignum: 
+	case t_ratio: 
+		f = number_to_double(x);  break; 
+	case t_shortfloat: 
+		f = sf(x);  break; 
+	case t_longfloat: 
+		f = lf(x);  break; 
+	case t_complex: 
+		f = object_to_float(x->cmp.cmp_real)+object_to_float(x->cmp.cmp_imag)*I;  break; 
+	default: 
+		FEcannot_coerce(sLfloat,x);
+	} 
+	return(f); 
+} 
+
+object
+make_dcomplex(dcomplex x) {
+
+  return make_complex(make_longfloat(creal(x)),make_longfloat(cimag(x)));
+
+}
+
+dcomplex 
+object_to_dcomplex1(object x) 
+{ 
+	dcomplex f=0.0; 
+
+	switch (type_of(x)) { 
+	case t_character: 
+		f = char_code(x);  break; 
+	case t_fixnum: 
+		f = fix(x);  break; 
+	case t_bignum: 
+	case t_ratio: 
+		f = number_to_double(x);  break; 
+	case t_shortfloat: 
+		f = sf(x);  break; 
+	case t_longfloat: 
+		f = lf(x);  break; 
+	case t_complex: 
+		f = object_to_double(x->cmp.cmp_real)+I*object_to_double(x->cmp.cmp_imag);  break; 
+	default: 
+		FEcannot_coerce(sLfloat,x);
 	} 
 	return(f); 
 } 
@@ -284,7 +434,7 @@ object_to_double(object x)
 	case t_longfloat: 
 		d = lf(x);  break; 
 	default: 
-		FEerror("~S cannot be coerce to a C double.", 1, x); 
+		FEerror("~S cannot be coerce to a C double.", 1, x);
 	} 
 	return(d); 
 } 
@@ -299,8 +449,8 @@ object_to_string(object x) {
   unsigned int leng;
   char *res;
 
-  if (type_of(x)!=t_string) FEwrong_type_argument(sLstring,x);
-  leng= x->st.st_fillp;
+  if (!stringp(x)) FEwrong_type_argument(sLstring,x);/*FIXME check_type*/
+  leng= VLEN(x);
   /* user has thoughtfully provided a null terminated string ! */
   if (leng > 0 && leng < x->st.st_dim && x->st.st_self[leng]==0)
     return x->st.st_self;
@@ -317,24 +467,6 @@ object_to_string(object x) {
 
 }
 
-/*  typedef int (*FUNC)(); */
-
-/* perform the actual invocation of the init function durint a fasload
-   init_address is the offset from the place in memory where the code is loaded
-   in.  In most systems this will be 0.
-   The new style fasl vector MUST end with an entry (si::%init f1 f2 .....)
-   where f1 f2 are forms to be evaled.
-*/
-
-/* #ifdef CLEAR_CACHE */
-/* static int */
-/* sigh(int sig,long code,void *scp, char *addr) { */
-
-/*     fprintf(stderr,"Received SIGILL at %p\n",((siginfo_t *)code)->si_addr); */
-/*     exit(1); */
-/* } */
-/* #endif */
-
 
 void
 call_init(int init_address,object memory,object faslfile) {
@@ -359,6 +491,7 @@ call_init(int init_address,object memory,object faslfile) {
 
    */
 
+static int set_min_cfd_self=0;
 object *min_cfd_self=NULL;
 
 void
@@ -377,19 +510,19 @@ do_init(object *statVV) {
   if (ch!='\n') {
     struct fasd * fd;
     faslfile=fSopen_fasd(faslfile,sKinput,OBJNULL,Cnil);
-    fd=(struct fasd *)faslfile->v.v_self;
+    fd=(struct fasd *)faslfile->sv.sv_self;
     n=fix(fd->table_length);
-    fd->table->v.v_self=alloca(n*sizeof(object));
-    memset(fd->table->v.v_self,0,n*sizeof(object));
-    fd->table->v.v_dim=faslfile->v.v_self[1]->v.v_fillp=n;
+    fd->table->sv.sv_self=alloca(n*sizeof(object));
+    memset(fd->table->sv.sv_self,0,n*sizeof(object));
+    fd->table->sv.sv_dim=faslfile->sv.sv_self[1]->sv.sv_dim=n;
   }
 
   n=fix(READ_STREAM_OR_FASD(faslfile));
-  sSPinit->s.s_dbind=fasl_vec=fSmake_vector1_1(n,aet_object,Cnil);
+  sSPinit->s.s_dbind=fasl_vec=fSmake_vector(Ct,n,Ct,Cnil,Cnil,0,Cnil,Cnil);
 
   /* switch SPinit to point to a vector of function addresses */
 
-  fasl_vec->v.v_elttype = aet_fix;
+  set_array_elttype(fasl_vec,aet_fix);
 
   /* swap the entries */
   for (i=0,p=fasl_vec->v.v_self,q=statVV;i<n;i++) {
@@ -399,7 +532,7 @@ do_init(object *statVV) {
   }
 
   data->cfd.cfd_self = statVV;
-  if (!min_cfd_self || data->cfd.cfd_self<min_cfd_self)
+  if (set_min_cfd_self && (!min_cfd_self || data->cfd.cfd_self<min_cfd_self))
     min_cfd_self=data->cfd.cfd_self;
   data->cfd.cfd_fillp= n;
   statVV[n-1] = data;
@@ -416,7 +549,7 @@ do_init(object *statVV) {
 
 }
 
-DEFUN_NEW("MARK-MEMORY-AS-PROFILING",object,fSmark_memory_as_profiling,SI,0,0,
+DEFUN("MARK-MEMORY-AS-PROFILING",object,fSmark_memory_as_profiling,SI,0,0,
 	  NONE,OO,OO,OO,OO,(void),"") {
 
   sSPmemory->s.s_dbind->cfd.cfd_prof=1;
@@ -458,6 +591,8 @@ new_cfdata(void) {
   memory->cfd.cfd_prof=0;
   memory->cfd.cfd_self=0;
   memory->cfd.cfd_start=0;
+  memory->cfd.cfd_dlist=Cnil;
+  memory->cfd.cfd_name=Cnil;
 
   return memory;
 
@@ -471,25 +606,28 @@ gcl_init_or_load1(void (*fn)(void),const char *file) {
 
     object memory;
     object faslfile;
-    file=FIX_PATH_STRING(file);
-    
+    /* file=FIX_PATH_STRING(file); */
+
     memory=new_cfdata();
+    memory->cfd.cfd_name=make_simple_string(file);
     memory->cfd.cfd_start= (char *)fn;
     printf("Initializing %s\n",file); fflush(stdout);
     faslfile=open_stream(make_simple_string(file),smm_input,Cnil,sKerror);
     SEEK_TO_END_OFILE(faslfile->sm.sm_fp);
+    set_min_cfd_self=1;
     call_init(0,memory,faslfile);
+    set_min_cfd_self=0;
     close_stream(faslfile);
 
   } else {
-    printf("loading %s\n",file); 
-    fflush(stdout);  
+    printf("loading %s\n",file);
+    fflush(stdout);
     load(file);
   }
 
 }
 
-DEFUN_NEW("INIT-CMP-ANON", object, fSinit_cmp_anon, SI, 0, 0,
+DEFUN("INIT-CMP-ANON", object, fSinit_cmp_anon, SI, 0, 0,
        NONE, OO, OO, OO,OO,(void),
       "Initialize previously compiled and linked anonymous function from the \
 .text section of the running executable.  This function is inherently \
@@ -531,9 +669,6 @@ find_init_name1(char *s,unsigned len) {
   return find_init_string(tmp);
 
 #else    
-/* These functions have no relevance on Windows
- * as dlopen and friends don't exist in that part of Cyberspace. */
-
   struct stat ss;
   char *tmp,*q;
   FILE *f;
@@ -563,7 +698,7 @@ find_init_name1(char *s,unsigned len) {
 }
  
 
-DEFUN_NEW("FIND-INIT-NAME", object, fSfind_init_name, SI, 1, 1,
+DEFUN("FIND-INIT-NAME", object, fSfind_init_name, SI, 1, 1,
 	  NONE, OO, OO, OO,OO,(object namestring),"") 
 {
 
@@ -572,7 +707,7 @@ DEFUN_NEW("FIND-INIT-NAME", object, fSfind_init_name, SI, 1, 1,
 
 }
 
-DEFUN_NEW("SEEK-TO-END-OFILE",object,fSseek_to_end_ofile,SI,1,1,NONE,OO,OO,OO,OO,(object sm),"") {
+DEFUN("SEEK-TO-END-OFILE",object,fSseek_to_end_ofile,SI,1,1,NONE,OO,OO,OO,OO,(object sm),"") {
   check_type_stream(&sm);
   SEEK_TO_END_OFILE(sm->sm.sm_fp);
   RETURN1(sm);
