@@ -1,6 +1,5 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
- Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -786,7 +785,7 @@ call_applyhook(object fun)
 	object ah;
 
 	ah = symbol_value(siVapplyhook);
-	stack_list();
+	Llist();
 	vs_push(vs_base[0]);
 	vs_base[0] = fun;
 	vs_push(list(3,lex_env[0],lex_env[1],lex_env[2]));
@@ -855,12 +854,7 @@ DEFUNM("APPLY",object,fLapply,LISP,1,MAX_ARGS,NONE,OO,OO,OO,OO,(object fun,...),
   return res;
 
 }
-
-object
-apply_format_function(object x,object y,object z,object a,object b,object c) {
-  return FFN(fLapply)(x,y,z,a,b,c);
-}
-
+	
 
 DEFUNM("EVAL",object,fLeval,LISP,1,1,NONE,OO,OO,OO,OO,(object x0),"") {
 
@@ -976,7 +970,7 @@ DEFUN("FSET-IN",object,fSfset_in,SI,3,3,NONE,OO,OO,OO,OO,(object sym,object src,
   
   object x;
 
-  x=fSinit_function(list(6,Cnil,Cnil,src,Cnil,make_fixnum(0),name),(void *)FFN(fSeval_src),Cnil,Cnil,-1,0,(((1<<6)-1)<<6)|(((1<<5)-1)<<12)|(1<<17));
+  x=fSinit_function(list(6,Cnil,Cnil,src,Cnil,Cnil,name),(void *)FFN(fSeval_src),Cnil,Cnil,-1,0,(((1<<6)-1)<<6)|(((1<<5)-1)<<12)|(1<<17));
   x->fun.fun_env=src_env;
   if (sym!=Cnil) fSfset(sym,x);
   RETURN1(x);
@@ -985,7 +979,7 @@ DEFUN("FSET-IN",object,fSfset_in,SI,3,3,NONE,OO,OO,OO,OO,(object sym,object src,
 #ifdef STATIC_FUNCTION_POINTERS
 object
 fSfset_in(object sym,object src,object name) {
-  RETURN1(FFN(fSfset_in)(sym,src,name));
+  RETURN1(FFN(fSfset_in)(sym,src,object name));
 }
 #endif
 
