@@ -1,5 +1,4 @@
  /* Copyright William F. Schelter  All Rights Reserved.
-    Copyright 2024 Camm Maguire
 
    Utility for writing out lisp objects and reading them in:
    Basically it attempts to write out only those things which could
@@ -596,7 +595,7 @@ DEFUN("OPEN-FASD",object,fSopen_fasd,SI
   if(direction==sKinput)
     {str=coerce_stream(stream,0);
     if (tabl==Cnil)
-      tabl=alloc_simple_vector(0);
+      tabl=alloc_simple_vector(0,aet_object);
     else
       check_type(tabl,t_simple_vector);}
   if(direction==sKoutput)
@@ -605,7 +604,7 @@ DEFUN("OPEN-FASD",object,fSopen_fasd,SI
     else
       check_type(tabl,t_hashtable);}
   massert(str==stream);
-  result=alloc_simple_vector(sizeof(struct fasd)/sizeof(object));
+  result=alloc_simple_vector(sizeof(struct fasd)/sizeof(object),aet_object);
   array_allocself(result,1,Cnil);
   {struct fasd *fd= (struct fasd *)result->sv.sv_self;
   fd->table=tabl;
@@ -1003,16 +1002,34 @@ DEFUN("FIND-SHARING-TOP",object,fSfind_sharing_top,SI
 
 }
 
-static object
-lisp_eval(object x) {
 
-  SAVE_CURRENT_FASD;
-  x=ieval(x);
-  RESTORE_FASD;
 
-  return x;
+/* static object            */
+/* read_fasd(int i) */
+/* {object tem; */
+/*    read_fasd1(i,&tem); */
+/*    return tem;} */
 
-}
+
+     /* I am not sure if saving vs_top,vs_base is necessary */
+static object 
+lisp_eval(object x)
+{  /* object *b,*t; */
+   SAVE_CURRENT_FASD;
+   x=fLeval(x);/*FIXME FFN*/
+   /* b=vs_base; */
+   /* t=vs_top; */
+   /* vs_base=vs_top; */
+   /* vs_push(x); */
+   /* Leval();  */
+   /* x=vs_base[0]; */
+   /* vs_base=b; */
+   /* vs_top=t; */
+   RESTORE_FASD;
+   return x;
+ }
+
+    
 
 #define CHECK_CH(i)    	   do{if ((i)==EOF && stream_at_end(fas_stream)) bad_eof();}while (0)
 /* grow vector AR of general type */
