@@ -20,8 +20,6 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
 
-#include <string.h>
-#include <stdlib.h>
 #include <dlfcn.h>
 #include <sys/param.h>
 #include <time.h>
@@ -34,6 +32,8 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <elf_abi.h>
 #endif
 
+#include "ptable.h"
+
 struct name_list { 
   struct name_list *next;
   char name[1];
@@ -55,60 +55,56 @@ fasload(object faslfile) {
 
   void *dlp ;
   int (*fptr)();
-  char buf[MAXPATHLEN],b[MAXPATHLEN],filename[MAXPATHLEN];
   static int count;
   object memory,data,faslstream;
   struct name_list *nl;
   object x;
 
-  bzero(buf,sizeof(buf)); /*GC partial stack hole closing*/
-  bzero(b,sizeof(b));
-  bzero(filename,sizeof(filename));
 
-  /* this is just to allow reloading in the same file twice.
-   */
-  coerce_to_filename(truename(faslfile), filename);
+  coerce_to_filename(faslfile, FN1);
   if (!count)
     count=time(0);
-  massert(snprintf(buf,sizeof(buf),"/tmp/ufas%dxXXXXXX",count++)>0);
-  massert(mkstemp(buf)>=0);
+  massert(snprintf(FN2,sizeof(FN2),"/tmp/ufas%dxXXXXXX",count++)>0);
+  massert(mkstemp(FN2)>=0);
 
-  massert((nl=(void *) malloc(strlen(buf)+1+sizeof(nl))));
+  massert((nl=(void *) malloc(strlen(FN2)+1+sizeof(nl))));
   massert(loaded_files || !atexit(unlink_loaded_files));
   nl->next = loaded_files;
   loaded_files = nl;
-  strcpy(nl->name,buf);
+  strcpy(nl->name,FN2);
 
-  faslstream = open_stream(faslfile, smm_input, Cnil, sKerror);
-  massert(snprintf(b,sizeof(b),"cc -shared %s -o %s",filename,buf)>0);
-  massert(!psystem(b));
+  /* faslstream = open_stream(faslfile, smm_input, Cnil, sKerror); */
+  massert(snprintf(FN3,sizeof(FN3),"cc -shared %s -o %s",FN1,FN2)>0);
+  massert(!psystem(FN3));
 
-  if (!(dlp = dlopen(buf,RTLD_NOW))) {
+  if (!(dlp = dlopen(FN2,RTLD_NOW))) {
     emsg(dlerror());
-    FEerror("Cannot open for dynamic link ~a",1,make_simple_string(filename));
+    FEerror("Cannot open for dynamic link ~a",1,make_simple_string(faslfile));
   }
   
 
-  x=find_init_name1(buf,0);
-  massert(x->st.st_fillp+1<sizeof(b));
-  memcpy(b,x->st.st_self,x->st.st_fillp);
-  b[x->st.st_fillp]=0;
-  if (!(fptr=dlsym(dlp,b))) {
-    emsg(dlerror());
-    FEerror("Cannot lookup ~a in ~a",2,make_simple_string(b),make_simple_string(filename));
+  x=find_init_name1(FN2,0);
+  coerce_to_filename(x,FN3);
+  if (!(fptr=dlsym(dlp,FN3))) {
+    fputs(dlerror(),stderr);
+    FEerror("Cannot lookup ~a in ~a",2,make_simple_string(b),make_simple_string(faslfile));
   }
 
+  faslstream = open_stream(faslfile, smm_input, Cnil, sKerror);
   SEEK_TO_END_OFILE(faslstream->sm.sm_fp);
 
   data = read_fasl_vector(faslstream);
   memory=new_cfdata();
+  memory->cfd.cfd_name=faslfile->sm.sm_object1;
 
-  if(symbol_value(sLAload_verboseA)!=Cnil)	
-    printf(" start address (dynamic) %p ",fptr);
+  if(symbol_value(sLAload_verboseA)!=Cnil) {
+    printf(" ;; start address (dynamic) %p ",fptr);
+    fflush(stdout);
+  }
 
   call_init(0,memory,data,fptr);
 
-  unlink(buf);
+  unlink(FN2);
   close_stream(faslstream);
 
   return memory->cfd.cfd_size;
