@@ -1,6 +1,5 @@
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
- Copyright (C) 2024 Camm Maguire
 
 This file is part of GNU Common Lisp, herein referred to as GCL
 
@@ -2195,7 +2194,7 @@ fmt_function(bool colon, bool atsign) {
   z=fmt_advance();
 
   VFUN_NARGS=6;
-  apply_format_function(x,fmt_stream,z,colon ? Ct : Cnil,atsign ? Ct : Cnil,y);
+  fLapply(x,fmt_stream,z,colon ? Ct : Cnil,atsign ? Ct : Cnil,y);
 
 }
 
@@ -2694,7 +2693,7 @@ fmt_semicolon(bool colon, bool atsign)
 
 DEFVAR("*FORMAT-UNUSED-ARGS*",sSAformat_unused_argsA,SI,OBJNULL,"");
 
-static object justification_regexp=OBJNULL,logical_block_regexp=OBJNULL;
+static object justification_regexp,logical_block_regexp;
 
 static int
 fmt_pp_string(object control) {
@@ -2705,8 +2704,8 @@ fmt_pp_string(object control) {
   if (logical_block_regexp==OBJNULL)
     logical_block_regexp=fScompile_regexp(make_simple_string("~@?:@?>|~[:@]?[:@]?_|~[0-9]*:?[Ii]|~[0-9]+,[0-9]+[:@]?[:@]?[Tt]|~[:@]?[:@]?[Ww]"));
   VFUN_NARGS=2;
-  just=(fixnum)fSstring_match2(justification_regexp,control);
-  pp=(fixnum)fSstring_match2(logical_block_regexp,control);
+  just=(fixnum)fSstring_match(justification_regexp,control);
+  pp=(fixnum)fSstring_match(logical_block_regexp,control);
   if (just>=0 && pp>=0)
     fmt_error("Mixed justification syntax");
 
