@@ -1,5 +1,4 @@
 /*  Copyright William Schelter. All rights reserved.
-    Copyright 2024 Camm Maguire
 Fast linking method for kcl by W. Schelter University of Texas
    Note there are also changes to 
  cmpcall.lsp and cmptop.lsp */
@@ -229,13 +228,15 @@ clean_link_array(object *ar, object *ar_end) {
 
 DEFVAR("*FAST-LINK-WARNINGS*",sSAfast_link_warningsA,SI,Cnil,"");
 
-#include "pbits.h"
-
-#ifdef WORDS_BIGENDIAN
-typedef struct {ufixnum pad:LM(21),nf:1,pu:1,va:1,vv:1,nv:5,xa:6,ma:6;} fw;
-#else
-typedef struct {ufixnum ma:6,xa:6,nv:5,vv:1,va:1,pu:1,nf:1,pad:LM(21);} fw;
-#endif
+typedef struct {
+  ufixnum ma:6;
+  ufixnum xa:6;
+  ufixnum nv:5;
+  ufixnum vv:1;
+  ufixnum va:1;
+  ufixnum pu:1;
+  ufixnum nf:1;
+} fw;
 
 typedef union {
   ufixnum i;
@@ -328,7 +329,7 @@ call_proc_new(object sym,ufixnum clp,ufixnum vld,void **link,ufixnum argd,object
     if (tp==t_function) {
       fprintf(stderr,"Warning: arg/val mismatch in call to %-.*s (%p) prevents fast linking:\n %ld %ld/%ld %d(%d)  %ld %d  %ld %d  %ld, recompile caller\n",
 	      (int)(type_of(sym)==t_symbol ? VLEN(sym->s.s_name) : 0),sym->s.s_name->st.st_self,sym,
-	      argd,(long)fun->fun.fun_argd,
+	      argd,fun->fun.fun_argd,
 	      vald,fun->fun.fun_neval,fun->fun.fun_vv,
 	      margs,fun->fun.fun_minarg,nargs,fun->fun.fun_maxarg,pushed);
       fflush(stderr);
@@ -417,7 +418,7 @@ call_proc_new(object sym,ufixnum clp,ufixnum vld,void **link,ufixnum argd,object
     } else
       vs_top=base;
 
-    for (;--old_top>=vs_top && vs_top>=vs_org;) *old_top=Cnil;
+    for (;--old_top>=vs_top;) *old_top=Cnil;
     
     switch(result_type) {
     case f_fixnum:
@@ -435,15 +436,6 @@ call_proc_new(object sym,ufixnum clp,ufixnum vld,void **link,ufixnum argd,object
   }
 
 }
-object
-call_proc_new_nval(object sym,ufixnum clp,ufixnum vld,void **link,ufixnum argd,object first,...) {
-  object x;
-  va_list b;
-  va_start(b,first);
-  x=call_proc_new(sym,clp,vld,link,argd,first,b);
-  va_end(b);
-  return x;
-}
 
 object
 call_proc_cs1(object fun,...) {
