@@ -520,7 +520,7 @@ mark_object1(object x) {
     mark_object(x->s.s_plist);
     mark_object(x->s.s_gfdef);
     mark_object(x->s.s_dbind);
-    MARK_LEAF_DATA(x,x->s.s_self,x->s.s_fillp);
+    mark_object(x->s.s_name);
     break;
     
   case t_package:
@@ -540,18 +540,21 @@ mark_object1(object x) {
     mark_object(x->ht.ht_rhthresh);
     if (x->ht.ht_self)
       for (i=0;i<x->ht.ht_size;i++)
-	if (x->ht.ht_self[i].hte_key!=OBJNULL) {
-	  mark_object_address(&x->ht.ht_self[i].hte_key,i);
-	  mark_object_address(&x->ht.ht_self[i].hte_value,i+1);
+	if (x->ht.ht_self[i].c_cdr!=OBJNULL) {/*FIXME*/
+	  mark_object_address(&x->ht.ht_self[i].c_cdr,i);
+	  mark_object_address(&x->ht.ht_self[i].c_car,i+1);
 	}
     i=x->ht.ht_cache-x->ht.ht_self;
     MARK_LEAF_DATA(x,x->ht.ht_self,x->ht.ht_size*sizeof(*x->ht.ht_self));
     if (x->ht.ht_cache) x->ht.ht_cache=x->ht.ht_self+i;
     break;
     
+  case t_simple_array:
   case t_array:
     MARK_LEAF_DATA(x,x->a.a_dims,sizeof(*x->a.a_dims)*x->a.a_rank);
 
+  case t_simple_vector:
+  case t_simple_bitvector:
   case t_vector:
   case t_bitvector:
 
@@ -564,8 +567,7 @@ mark_object1(object x) {
       break;
 
     case aet_bit:
-#define W_SIZE (8*sizeof(fixnum))
-      j= sizeof(fixnum)*((BV_OFFSET(x) + x->bv.bv_dim + W_SIZE -1)/W_SIZE);
+      j=ceil(BV_OFFSET(x)+x->bv.bv_dim,BV_ALLOC)*sizeof(*x->bv.bv_self);
       break;
 
     case aet_char:
@@ -579,7 +581,7 @@ mark_object1(object x) {
       break;
 
     case aet_object:
-      if (x->v.v_displaced->c.c_car==Cnil)
+      if (ADISP(x)->c.c_car==Cnil)
 	mark_object_array(x->v.v_self,x->v.v_self+x->v.v_dim);
 
     default:
@@ -587,32 +589,30 @@ mark_object1(object x) {
 
     }
 
+  case t_simple_string:
   case t_string:/*FIXME*/
     j=j ? j : x->st.st_dim;
 
-    if (x->v.v_displaced->c.c_car==Cnil) {
-      void *p=x->v.v_self;
+    if (ADISP(x)->c.c_car==Cnil)
       MARK_LEAF_DATA(x,x->v.v_self,j);
-      if (x->v.v_displaced!=Cnil) {
-	j=(void *)x->v.v_self-p;
-	x->v.v_self=p;
-	adjust_displaced(x,j);
-      }
-    }
-    mark_object(x->v.v_displaced);
+
+    mark_object(ADISP(x));
+    set_displaced_body_ptr(x);
+
     break;
     
   case t_structure:
     {
-      object def=x->str.str_def;
-      unsigned char *s_type= &SLOT_TYPE(def,0);
-      unsigned short *s_pos= &SLOT_POS(def,0);
       mark_object(x->str.str_def);
-      if (x->str.str_self)
+      if (x->str.str_self) {
+	object def=x->str.str_def;
+	unsigned char *s_type= &SLOT_TYPE(def,0);
+	unsigned short *s_pos= &SLOT_POS(def,0);
 	for (i=0,j=S_DATA(def)->length;i<j;i++)
 	  if (s_type[i]==aet_object)
 	    mark_object_address(&STREF(object,x,s_pos[i]),i);
-      MARK_LEAF_DATA(x,x->str.str_self,S_DATA(def)->size);
+	MARK_LEAF_DATA(x,x->str.str_self,S_DATA(def)->size);
+      }
     }
     break;
     
@@ -625,11 +625,13 @@ mark_object1(object x) {
     case smm_probe:
       mark_object(x->sm.sm_object0);
       mark_object(x->sm.sm_object1);
-      if (x->sm.sm_fp) {
-	MARK_LEAF_DATA(x,x->sm.sm_buffer,BUFSIZ);
-      }
+      /* Only set by malloc, handled by malloc_list */
+      /* if (x->sm.sm_fp) { */
+      /* 	MARK_LEAF_DATA(x,x->sm.sm_buffer,BUFSIZ); */
+      /* } */
       break;
 
+    case smm_file_synonym:
     case smm_synonym:
       mark_object(x->sm.sm_object0);
       break;
@@ -687,39 +689,27 @@ mark_object1(object x) {
     mark_object(x->pn.pn_namestring);
     break;
     
-  case t_closure:
-    mark_object_array(x->cl.cl_env,x->cl.cl_env+x->cl.cl_envdim);
-    MARK_LEAF_DATA(x,x->cl.cl_env,x->cl.cl_envdim*sizeof(object));
-    
-  case t_cfun:
-  case t_sfun:
-  case t_vfun:
-  case t_afun:
-  case t_gfun:	
-    mark_object(x->cf.cf_name);
-    mark_object(x->cf.cf_data);
+  case t_function:	
+    mark_object(x->fun.fun_data);
+    mark_object(x->fun.fun_plist);
+    if (x->fun.fun_env != def_env && x->fun.fun_env != src_env) {
+      mark_object(x->fun.fun_env[0]);
+      x->fun.fun_env--;
+      MARK_LEAF_DATA(x,x->fun.fun_env,(*(ufixnum *)x->fun.fun_env)*sizeof(*x->fun.fun_env));
+      x->fun.fun_env++;
+    }
     break;
     
   case t_cfdata:
     
+    mark_object(x->cfd.cfd_dlist);
+    mark_object(x->cfd.cfd_name);
     mark_object_array(x->cfd.cfd_self,x->cfd.cfd_self+x->cfd.cfd_fillp);
     if (what_to_collect == t_contiguous)
       mark_link_array(x->cfd.cfd_start,x->cfd.cfd_start+x->cfd.cfd_size);
     MARK_LEAF_DATA(NULL,x->cfd.cfd_start,x->cfd.cfd_size);/*Code cannot move*/
     break;
 
- case t_cclosure:
-    mark_object(x->cc.cc_name);
-    mark_object(x->cc.cc_env);
-    mark_object(x->cc.cc_data);
-    if (x->cc.cc_turbo) {
-      x->cc.cc_turbo--;
-      mark_object_array(x->cc.cc_turbo,x->cc.cc_turbo+fix(x->cc.cc_turbo[0]));
-      MARK_LEAF_DATA(x,x->cc.cc_turbo,(1+fix(x->cc.cc_turbo[0]))*sizeof(*x->cc.cc_turbo));
-      x->cc.cc_turbo++;
-    }
-    break;
-    
   case t_spice:
     break;
 
@@ -800,7 +790,9 @@ mark_phase(void) {
   STATIC ihs_ptr ihsp;
   
   mark_object(Cnil->s.s_plist);
+  mark_object(Cnil->s.s_name);
   mark_object(Ct->s.s_plist);
+  mark_object(Ct->s.s_name);
   
   mark_stack_carefully(vs_top-1,vs_org,0);
   mark_stack_carefully(MVloc+(sizeof(MVloc)/sizeof(object)),MVloc,0);
@@ -1113,8 +1105,11 @@ GBC(enum type t) {
   
   what_to_collect = t;
   
-  tm_table[(int)t].tm_gbccount++;
-  tm_table[(int)t].tm_adjgbccnt++;
+  {
+    struct typemanager *tm=tm_of(t);
+    tm->tm_gbccount++;
+    tm->tm_adjgbccnt++;
+  }
   
   if (sSAnotify_gbcA->s.s_dbind != Cnil
 #ifdef DEBUG
@@ -1290,6 +1285,7 @@ FFN(siLheap_report)(void) {
   vs_push(make_fixnum((CSTACK_DIRECTION+1)>>1));
   vs_push(make_fixnum(CSTACK_ALIGNMENT));
   vs_push(make_fixnum(labs(cs_limit-cs_org)));/*CSSIZE*/
+  vs_push(make_fixnum(phys_pages));
 #if defined(IM_FIX_BASE) && defined(IM_FIX_LIM)
 #ifdef LOW_IM_FIX
   vs_push(make_fixnum(-LOW_IM_FIX));
@@ -1302,44 +1298,46 @@ FFN(siLheap_report)(void) {
   vs_push(make_fixnum(0));
   vs_push(make_fixnum(0));
 #endif
-  vs_push(make_fixnum(phys_pages));
 
 }  
 
-static void
-FFN(siLroom_report)(void) {
+DEFUN("ROOM-REPORT",object,fSroom_report,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
 
+  object x=Cnil;
   int i;
   
-  check_arg(0);
-  
-  vs_check_push(make_fixnum(real_maxpage-first_data_page));
-  vs_push(make_fixnum(available_pages));
-  vs_push(make_fixnum(ncbpage));
-  vs_push(make_fixnum(maxcbpage));
-  vs_push(make_fixnum(count_contblocks()));
-  vs_push(make_fixnum(cbgbccount));
-  vs_push(make_fixnum((rb_start-heap_end)>>PAGEWIDTH));
-  vs_push(make_fixnum(rb_pointer - rb_begin()));
-  vs_push(make_fixnum((rb_begin()+rb_size()) - rb_pointer));
-  vs_push(make_fixnum(nrbpage));
-  vs_push(make_fixnum(maxrbpage));
-  vs_push(make_fixnum(rbgbccount));
+  x=make_cons(make_fixnum(real_maxpage-first_data_page),x);
+  x=make_cons(make_fixnum(available_pages),x);
+  x=make_cons(make_fixnum(ncbpage),x);
+  x=make_cons(make_fixnum(maxcbpage),x);
+  x=make_cons(make_fixnum(count_contblocks()),x);
+  x=make_cons(make_fixnum(cbgbccount),x);
+  x=make_cons(make_fixnum((rb_start-heap_end)>>PAGEWIDTH),x);
+  x=make_cons(make_fixnum(rb_pointer - rb_begin()),x);
+  x=make_cons(make_fixnum((rb_begin()+rb_size()) - rb_pointer),x);
+  x=make_cons(make_fixnum(nrbpage),x);
+  x=make_cons(make_fixnum(maxrbpage),x);
+  x=make_cons(make_fixnum(rbgbccount),x);
   for (i = 0;  i < (int)t_end;  i++) {
+    x=make_cons(make_simple_string(tm_table[i].tm_name+1),x);
     if (tm_table[i].tm_type == (enum type)i) {
-      vs_check_push(make_fixnum(TM_NUSED(tm_table[i])));
-      vs_push(make_fixnum(tm_table[i].tm_nfree+tm_table[i].tm_alt_nfree));
-      vs_push(make_fixnum(tm_table[i].tm_npage));
-      vs_push(make_fixnum(tm_table[i].tm_maxpage));
-      vs_push(make_fixnum(tm_table[i].tm_gbccount));
+      x=make_cons(make_fixnum(TM_NUSED(tm_table[i])),x);
+      x=make_cons(make_fixnum(tm_table[i].tm_nfree+tm_table[i].tm_alt_nfree),x);
+      x=make_cons(make_fixnum(tm_table[i].tm_npage),x);
+      x=make_cons(make_fixnum(tm_table[i].tm_maxpage),x);
+      x=make_cons(make_fixnum(tm_table[i].tm_gbccount),x);
+      x=make_cons(make_fixnum(tm_table[i].tm_size/sizeof(fixnum)),x);
     } else {
-      vs_check_push(Cnil);
-      vs_push(make_fixnum(tm_table[i].tm_type));
-      vs_push(Cnil);
-      vs_push(Cnil);
-      vs_push(Cnil);
+      x=make_cons(Cnil,x);
+      x=make_cons(make_simple_string(tm_of(i)->tm_name+1),x);
+      x=make_cons(Cnil,x);
+      x=make_cons(Cnil,x);
+      x=make_cons(Cnil,x);
+      x=make_cons(Cnil,x);
     }
   }
+  RETURN1(nreverse(x));
+
 }
 
 static void
@@ -1393,9 +1391,9 @@ mark_contblock(void *p, int s) {
 #endif
      set_mark_bits(v,x,y);
  }
- 
-DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO,(void),"") {
-  
+
+DEFUN("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO,(void),"") {
+
   struct contblock **cbpp;
   struct pageinfo *v;
   ufixnum i,j,k,s;
@@ -1439,6 +1437,7 @@ DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO
  	switch (type_of(o)) {
  	case t_array:
  	case t_vector:
+ 	case t_simple_vector:
  	  d=o->a.a_self;
  	  s=o->a.a_dim*sizeof(object);
  	  break;
@@ -1446,12 +1445,10 @@ DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO
  	  d=o->ht.ht_self;
  	  s=o->ht.ht_size*sizeof(object)*2;
  	  break;
- 	case t_symbol:
- 	  d=o->s.s_self;
- 	  s=o->s.s_fillp;
- 	  break;
+ 	case t_simple_string:
  	case t_string:
  	case t_bitvector:
+ 	case t_simple_bitvector:
  	  d=o->a.a_self;
  	  s=o->a.a_dim;
  	  break;
@@ -1471,11 +1468,11 @@ DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO
  	  d=o->rnd.rnd_state._mp_seed->_mp_d;
  	  s=o->rnd.rnd_state._mp_seed->_mp_alloc*MP_LIMB_SIZE;
  	  break;
- 	case t_cclosure:
- 	  d=o->cc.cc_turbo;
- 	  s=fix(o->cc.cc_turbo[-1]);
+ 	case t_function:
+ 	  d=o->fun.fun_env;
+ 	  s=o->fun.fun_env!=def_env && o->fun.fun_env!=src_env ? ((ufixnum *)o->fun.fun_env)[-1]*sizeof(object) : 0;
  	  break;
- 	case t_cfdata:
+ 	case t_cfdata:/*FIXME*/
  	  d=o->cfd.cfd_start;
  	  s=o->cfd.cfd_size;
  	  break;
@@ -1487,7 +1484,7 @@ DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO
  	  break;
  	}
  	if (d>=data_start && d<(void *)heap_end && s) {
- 	  emsg("%lu %s bytes at %p\n",s,tm_table[type_of(o)].tm_name,d);
+ 	  emsg("%lu %s bytes at %p, object %p\n",s,tm_table[type_of(o)].tm_name,d,o);
  	  i+=s;
  	  j++;
  	}
@@ -1500,7 +1497,7 @@ DEFUN_NEW("CONTIGUOUS-REPORT",object,fScontiguous_report,SI,1,1,NONE,OO,OO,OO,OO
 
 }
 
-DEFUN_NEW("GBC",object,fSgbc,SI,1,1,NONE,OO,OO,OO,OO,(object x0),"") {
+DEFUN("GBC",object,fSgbc,SI,1,1,NONE,OO,OO,OO,OO,(object x0),"") {
 
   if (x0 == Ct) {
     tm_table[t_contiguous].tm_adjgbccnt--;
@@ -1542,7 +1539,6 @@ void
 gcl_init_GBC(void) {
 
   make_si_function("HEAP-REPORT", siLheap_report);
-  make_si_function("ROOM-REPORT", siLroom_report);
   make_si_function("RESET-GBC-COUNT", siLreset_gbc_count);
   make_si_function("GBC-TIME",siLgbc_time);
 #ifdef SGC
