@@ -52,6 +52,7 @@ gprof_cleanup(void) {
 DEFUN("GPROF-ADDRESSES",object,fSgprof_addresses,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
 
   void *min=heap_end,*max=data_start,*c;
+  static void *mintext;
   struct pageinfo *v;
   object x;
   fixnum i;
@@ -68,20 +69,30 @@ DEFUN("GPROF-ADDRESSES",object,fSgprof_addresses,SI,0,0,NONE,OO,OO,OO,OO,(void),
   if (max<min)
     min=max;
 
+  if (!mintext) {
+
+    mintext=data_start;
+
 #ifdef GCL_GPROF
-  {
-      extern void *min_text;
-
-      if (min_text<data_start)
-	min=min_text;
-
-  }
+    for (i=0;i<c_table.alloc_length;i++)
+      mintext=(void *)c_table.ptable[i].address<mintext ? (void *)c_table.ptable[i].address : mintext;
 #endif
 
+  }
+
+  if (mintext<data_start)
+    min=mintext;
+
   return MMcons(make_fixnum((fixnum)min),make_fixnum((fixnum)max));
 
 }
 
+DEFUN("KCL-SELF",object,fSkcl_self,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
+  return make_simple_string(kcl_self);
+
+}
+
 DEFUN("PTABLE-ALLOC-LENGTH",object,fSptable_alloc_length,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
   return make_fixnum(c_table.alloc_length);
 }
