@@ -46,8 +46,7 @@ FFN(siLdefine_macro)(void)
 	clear_compiler_properties(vs_base[0],MMcaddr(vs_base[1]));
 	if (vs_base[0]->s.s_hpack == lisp_package &&
 	    vs_base[0]->s.s_gfdef != OBJNULL && !raw_image) {
-		vs_push(make_simple_string(
-			"~S is being redefined."));
+		vs_push(make_simple_string("~S is being redefined."));
 		ifuncall2(sLwarn, vs_head, vs_base[0]);
 		vs_popp;
 	}
@@ -80,20 +79,20 @@ FFN(Fdefmacro)(object form)
 	name = MMcar(form);
 	if (type_of(name) != t_symbol)
 		not_a_symbol(name);
-	vs_push(ifuncall3(sSdefmacroA,
+	vs_push(ifuncall3(sSdefmacro_lambda,
 			  name,
 			  MMcadr(form),
 			  MMcddr(form)));
-	if (MMcar(top[0]) != Cnil)
-		name->s.s_plist
-		= putf(name->s.s_plist,
-		       MMcar(top[0]),
-		       sSfunction_documentation);
-	if (MMcadr(top[0]) != Cnil)
-		name->s.s_plist
-		= putf(name->s.s_plist,
-		       MMcadr(top[0]),
-		       sSpretty_print_format);
+	/* if (MMcar(top[0]) != Cnil) */
+	/* 	name->s.s_plist */
+	/* 	= putf(name->s.s_plist, */
+	/* 	       MMcar(top[0]), */
+	/* 	       sSfunction_documentation); */
+	/* if (MMcadr(top[0]) != Cnil) */
+	/* 	name->s.s_plist */
+	/* 	= putf(name->s.s_plist, */
+	/* 	       MMcadr(top[0]), */
+	/* 	       sSpretty_print_format); */
 	if (name->s.s_sfdef != NOT_SPECIAL) {
 		if (name->s.s_mflag) {
 			if (symbol_value(sSAinhibit_macro_specialA) != Cnil)
@@ -102,15 +101,25 @@ FFN(Fdefmacro)(object form)
 			FEerror("~S, a special form, cannot be redefined.",
 				1, name);
 	}
-	clear_compiler_properties(name,MMcaddr(top[0]));
+
+	{
+	  top[0]=fSfset_in(Cnil,top[0],name);/*FIXME fSfset ?*/
+	}
+	/* { */
+	/*   object x=alloc_object(t_ifun); */
+	/*   x->ifn.ifn_self=top[0]; */
+	/*   x->ifn.ifn_name=x->ifn.ifn_call=Cnil; */
+	/*   top[0]=x; */
+	/* } */
+	
+	clear_compiler_properties(name,top[0]);
 	if (name->s.s_hpack == lisp_package &&
 	    name->s.s_gfdef != OBJNULL && !raw_image) {
-		vs_push(make_simple_string(
-			"~S is being redefined."));
+		vs_push(make_simple_string("~S is being redefined."));
 		ifuncall2(sLwarn, vs_head, name);
 		vs_popp;
 	}
-	name->s.s_gfdef = MMcaddr(top[0]);
+	name->s.s_gfdef = top[0];
 	name->s.s_mflag = TRUE;
 	vs_base = vs_top = top;
 	vs_push(name);
@@ -134,10 +143,12 @@ FFN(Fdefmacro)(object form)
 	in VS_BASE[0].
 */
 object
-Imacro_expand1(object exp_fun, object form)
-{
-	return Ifuncall_n(sLAmacroexpand_hookA->s.s_dbind,
-		   3,exp_fun,form,MACRO_EXPAND_ENV);
+Imacro_expand1(object exp_fun, object form) {
+/*   pp(form->c.c_car);printf("\n"); */
+  object b[3]={exp_fun,form,MACRO_EXPAND_ENV};
+  fcall.valp=0;
+  return funcall_vec(coerce_funcall_object_to_function(sLAmacroexpand_hookA->s.s_dbind),3,b);
+
 }
 
 /*
@@ -145,74 +156,72 @@ Imacro_expand1(object exp_fun, object form)
 	the expansion function if the form is a macro form.  Otherwise,
 	MACRO_DEF returns NIL.
 */
-static object
-macro_def(object form)
-{
-	object head, fd;
 
-	if (type_of(form) != t_cons)
-		return(Cnil);
-	head = MMcar(form);
-	if (type_of(head) != t_symbol)
-		return(Cnil);
-	fd = lex_fd_sch(head);
-	if (MMnull(fd))
-		if (head->s.s_mflag)
-			return(head->s.s_gfdef);
-		else
-			return(Cnil);
-	else if (MMcadr(fd) == sSmacro)
-		return(MMcaddr(fd));
-	else
-		return(Cnil);
+object
+macro_def_int(object sym) {
+
+  object fd; 
+
+  if (type_of(sym) != t_symbol)
+    return(Cnil);
+  fd = lex_fd_sch(sym);
+  if (MMnull(fd))
+    if (sym->s.s_mflag)
+      return(sym->s.s_gfdef);
+    else
+      return(Cnil);
+  else if (MMcadr(fd) == sSmacro)
+    return(MMcaddr(fd));
+  else
+    return(Cnil);
 }
 
-DEFUNOM_NEW("MACROEXPAND",object,fLmacroexpand,LISP
-       ,1,2,NONE,OO,OO,OO,OO,void,Lmacroexpand,(object form,...),"")
-{	int n=VFUN_NARGS;
-	object envir;
-	object exp_fun;
-	object *lex=lex_env;
-	object buf[3];
-	
-	va_list ap;
-	{ va_start(ap,form);
-	  if (n>=2) envir=va_arg(ap,object);else goto LDEFAULT2;
-	  goto LEND_VARARG;
-	LDEFAULT2: envir = Cnil;
-	LEND_VARARG: va_end(ap);}
+static object
+macro_def(object form) {
 
-	lex_env = buf;
-	if (n== 1) {buf[0]=sLnil;
-		    buf[1]=sLnil;
-		    buf[2]=sLnil;
-		  }
-	else if (n==2)
-	  { buf[0]=car(envir);
-	    envir=Mcdr(envir);
-	    buf[1]=car(envir);
-	    envir=Mcdr(envir);
-	    buf[2]=car(envir);
-	  }
-	else check_arg_range(1,2);
+  if (!consp(form))
+    return(Cnil);
+  return macro_def_int(MMcar(form));
 
-	exp_fun = macro_def(form);
+}
 
-	if (MMnull(exp_fun)) {
-	  lex_env = lex;
-	  RETURN(2,object,form,(RV(sLnil)));
-	}
-	else
-	  {
-	    object *top = vs_top;
-	    do {
-	      form= Imacro_expand1(exp_fun, form);
-	      vs_top = top;
-	      exp_fun = macro_def(form);
-	    } while (!MMnull(exp_fun));
-	    lex_env = lex;
-	    RETURN(2,object,form,(RV(sLt)));
-	  }
+DEFUNM("MACROEXPAND",object,fLmacroexpand,LISP,1,2,NONE,OO,OO,OO,OO,(object form,...),"") {
+
+  object envir;
+  object exp_fun,l=Cnil,f=OBJNULL;
+  object *lex=lex_env;
+  object buf[3];
+  va_list ap;
+  fixnum n=INIT_NARGS(1);
+  fixnum vals=(fixnum)fcall.valp;
+  object *base=vs_top;
+
+  va_start(ap,form);
+  envir=NEXT_ARG(n,ap,l,f,Cnil);
+  va_end(ap);
+  
+  lex_env = buf;
+  buf[0]=car(envir);
+  envir=Mcdr(envir);
+  buf[1]=car(envir);
+  envir=Mcdr(envir);
+  buf[2]=car(envir);
+  
+  exp_fun = macro_def(form);
+  
+  if (MMnull(exp_fun)) {
+    lex_env = lex;
+    RETURN(2,object,form,(RV(sLnil)));
+  } else {
+    object *top = vs_top;
+    do {
+      form= Imacro_expand1(exp_fun, form);
+      vs_top = top;
+      exp_fun = macro_def(form);
+    } while (!MMnull(exp_fun));
+    lex_env = lex;
+    RETURN(2,object,form,(RV(sLt)));
+  }
 }
 
 LFD(Lmacroexpand_1)(void)
@@ -266,7 +275,7 @@ macro_expand(object form)
 	   immediately.  Macro definitions are superseded by special-
 	   form definitions.
 	*/
-	if (type_of(form) != t_cons)
+	if (!consp(form))
 		return(form);
 	head = MMcar(form);
 	if (type_of(head) != t_symbol)
@@ -303,7 +312,7 @@ LOOP:
 	/* Check if the expanded form is again a macro form.  If not,
 	   reset the stack and return.
 	*/
-	if (type_of(form) != t_cons)
+	if (!consp(form))
 		goto END;
 	head = MMcar(form);
 	if (type_of(head) != t_symbol)
@@ -332,7 +341,8 @@ END:
 
 DEF_ORDINARY("FUNCALL",sLfuncall,LISP,"");
 DEFVAR("*MACROEXPAND-HOOK*",sLAmacroexpand_hookA,LISP,sLfuncall,"");
-DEF_ORDINARY("DEFMACRO*",sSdefmacroA,SI,"");
+/* DEF_ORDINARY("DEFMACRO*",sSdefmacroA,SI,""); */
+DEF_ORDINARY("DEFMACRO-LAMBDA",sSdefmacro_lambda,SI,"");
 DEFVAR("*INHIBIT-MACRO-SPECIAL*",sSAinhibit_macro_specialA,SI,Cnil,"");
 void
 gcl_init_macros(void)
