@@ -24,21 +24,13 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 	IMPLEMENTATION-DEPENDENT
 */
 
-#include <stdlib.h>
 #include <unistd.h>
-#include <string.h>
 #include <fcntl.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 
-static void
-init_main(void);
-
-static void
-initlisp(void);
-
-static int
-multiply_stacks(int);
+static void init_main(void);
+static void initlisp(void);
+static int  multiply_stacks(int);
+void install_segmentation_catcher(void);
 
 #ifdef KCLOVM
 #include <ovm/ovm.h>
@@ -47,11 +39,9 @@ int ovm_process_created;
 void initialize_process();
 #endif
 
-
 #define EXTER
 #define INLINE
 
-
 #include "include.h"
 #include <signal.h>
 #include "page.h"
@@ -112,14 +102,56 @@ int sgc_enabled;
 #endif
 void install_segmentation_catcher(void);
 
+#ifdef NEED_STACK_CHK_GUARD
+
+unsigned long __stack_chk_guard=0;
+
+static unsigned long
+random_ulong() {
+
+  object y;
+
+  vs_top=vs_base;
+  vs_push(Ct);
+  Lmake_random_state();
+  y=vs_pop;
+  vs_push(number_negate(find_symbol(make_simple_string("MOST-NEGATIVE-FIXNUM"),system_package)->s.s_dbind));
+  vs_push(y);
+  Lrandom();
+
+  return fixint(vs_pop);
+
+}
+#endif
+
+int cstack_dir=0;
+
 int
-cstack_dir(fixnum j) {
+get_cstack_dir(VOL fixnum j) {
   static fixnum n;
-  if (!n) {
-    n=1;
-    return cstack_dir((fixnum)&j);
-  }
-  return (fixnum)&j<j ? -1 : 1;
+  fixnum q=n;
+  n=1-n;
+  return q ? ((fixnum)&j<j ? -1 : 1) : get_cstack_dir((fixnum)&j);
+}
+
+void
+wipe_stack(VOL void *l) {
+
+  if (cstack_dir==-1) {
+    if (l>(void *)&l) bzero((void *)&l,l-(void *)&l);
+  } else {
+     l+=sizeof(l);
+     if ((void *)&l>l) bzero((void *)l,(void *)&l-l);
+   }
+}
+
+void
+clear_c_stack(VOL unsigned n) {
+
+  void *v=OBJNULL;
+  alloca(n);
+  wipe_stack(&v);
+
 }
 
 fixnum log_maxpage_bound=sizeof(fixnum)*8-1;
@@ -174,7 +206,7 @@ get_phys_pages_no_malloc(char n) {
   uint64_t s;
   size_t z=sizeof(s);
   int m[2]={CTL_HW,HW_MEMSIZE};
-  
+
   if (sysctl(m,2,&s,&z,NULL,0)==0)
     return s>>PAGEWIDTH;
 
@@ -202,7 +234,7 @@ get_phys_pages_no_malloc(char n) {
   size_t i,len=sizeof(i);
 
   return (sysctlbyname("hw.physmem",&i,&len,NULL,0) ? 0 : i)>>PAGEWIDTH;
-  
+
 }
 
 #else /*Linux*/
@@ -215,7 +247,7 @@ get_phys_pages_no_malloc(char freep) {
   struct sysinfo s;
 
   return sysinfo(&s) ? 0 : ((freep ? s.freeram : s.totalram)>>PAGEWIDTH)*s.mem_unit;
-  
+
 }
 
 #endif
@@ -231,7 +263,7 @@ static void
 get_gc_environ(void) {
 
   const char *e;
-  
+
   mem_multiple=1.0;
   if ((e=getenv("GCL_MEM_MULTIPLE"))) {
     massert(sscanf(e,"%lf",&mem_multiple)==1);
@@ -262,7 +294,7 @@ get_gc_environ(void) {
   wait_on_abort=0;
   if ((e=getenv("GCL_WAIT_ON_ABORT")))
     massert(sscanf(e,"%lu",&wait_on_abort)==1);
-  
+
 }
 
 static void
@@ -279,13 +311,13 @@ setup_maxpages(double scale) {
   maxpages*=scale;
   phys_pages*=scale;
   real_maxpage=maxpages+page(beg);
-  
+
   resv_pages=available_pages=0;
   available_pages=check_avail_pages();
-  
+
   resv_pages=available_pages/100;
   available_pages-=resv_pages;
-  
+
   recent_allocation=0;
 
 }
@@ -324,7 +356,7 @@ update_real_maxpage(void) {
 
   get_gc_environ();
   setup_maxpages(mem_multiple);
-  
+
   return 0;
 
 }
@@ -353,7 +385,7 @@ minimize_image(void) {
   
 }
 
-DEFUN_NEW("SET-LOG-MAXPAGE-BOUND",object,fSset_log_maxpage_bound,SI,1,1,NONE,II,OO,OO,OO,(fixnum l),"") {
+DEFUN("SET-LOG-MAXPAGE-BOUND",object,fSset_log_maxpage_bound,SI,1,1,NONE,II,OO,OO,OO,(fixnum l),"") {
 
   void *end,*dend;
   fixnum def=sizeof(fixnum)*8-1;
@@ -373,27 +405,37 @@ DEFUN_NEW("SET-LOG-MAXPAGE-BOUND",object,fSset_log_maxpage_bound,SI,1,1,NONE,II,
 
 }
 
-#ifdef NEED_STACK_CHK_GUARD
+#include <dlfcn.h>
 
-unsigned long __stack_chk_guard=0;
+size_t
+dir_name_length(const char *s) {
+  size_t m;
+  const char *z;
+  for (m=strlen(s),z=kcl_self+m;z>s && z[-1]!='/';z--,m--);
+  return m;
+}
 
-static unsigned long
-random_ulong() {
- 
-  object y;
-  
-  vs_top=vs_base;
-  vs_push(Ct);
-  Lmake_random_state();
-  y=vs_pop;
-  vs_push(number_negate(find_symbol(make_simple_string("MOST-NEGATIVE-FIXNUM"),system_package)->s.s_dbind));
-  vs_push(y);
-  Lrandom();
+void
+init_boot(void) {
 
-  return fixint(vs_pop);
+  char *sysd=getenv("GCL_SYSDIR"),*d=sysd ? sysd : kcl_self;
+  void *v,*q;
+  char *z,*s="libboot.so";
+  size_t m=sysd ? strlen(sysd) : dir_name_length(kcl_self),n=m+strlen(s)+1;
+
+  z=alloca(n);
+  snprintf(z,n,"%-*.*s%s",(int)m,(int)m,d,s);
+  if (!(v=dlopen(z,RTLD_LAZY|RTLD_GLOBAL)))
+    printf("%s\n",dlerror());
+  if (!(q=dlsym(v,"gcl_init_boot")))
+    printf("%s\n",dlerror());
+  ((void (*)())q)();
 
 }
-#endif
+
+int in_pre_gcl=0;
+object def_env1[2]={(object)1,Cnil},*def_env=def_env1+1;
+object src_env1[2]={(object)1,Cnil},*src_env=src_env1+1;
 
 #ifdef HAVE_MPROTECT
 #include <sys/mman.h>
@@ -422,7 +464,7 @@ gcl_cleanup(int gc) {
 
   if (getenv("GCL_WAIT"))
     sleep(30);
-  
+
 #if defined(USE_CLEANUP)
   {extern void _cleanup(void);_cleanup();}
 #endif
@@ -434,9 +476,9 @@ gcl_cleanup(int gc) {
     saving_system=TRUE;
     GBC(t_other);
     saving_system=FALSE;
-    
+
     minimize_image();
-    
+
     raw_image=FALSE;
     cs_org=0;
     initial_sbrk=core_end;
@@ -460,12 +502,12 @@ stack_ret(char *s,unsigned long size) {
 
 int
 get_stack_to_be_allocated(unsigned long size) {
-   stack_to_be_allocated=alloca(size);
-   memset(stack_to_be_allocated,0,size);
-   return stack_ret(stack_to_be_allocated,size);
- }
+  stack_to_be_allocated=alloca(size);
+  memset(stack_to_be_allocated,0,size);
+  return stack_ret(stack_to_be_allocated,size);
+}
 
-DEFUN_NEW("EQUAL-TAIL-RECURSION-CHECK",object,fSequal_tail_recursion_check,SI,1,1,NONE,II,OO,OO,OO,(fixnum s),"") {
+DEFUN("EQUAL-TAIL-RECURSION-CHECK",object,fSequal_tail_recursion_check,SI,1,1,NONE,II,OO,OO,OO,(fixnum s),"") {
   object x0=make_list(s/sizeof(object)),x1=make_list(s/sizeof(object));
   char *w;
   get_stack_to_be_allocated(s);
@@ -513,9 +555,6 @@ which(const char *n,char *o) {
 
 #endif
 
-static int ARGC;
-static char **ARGV;
-
 int
 main(int argc, char **argv, char **envp) {
 
@@ -541,6 +580,7 @@ main(int argc, char **argv, char **envp) {
 #endif
   ARGC = argc;
   ARGV = argv;
+  ENVP = envp;
   
   vs_top = vs_base = vs_org;
   ihs_top = ihs_org-1;
@@ -549,16 +589,19 @@ main(int argc, char **argv, char **envp) {
 
   if (raw_image) {
 
+#ifdef GCL_GPROF
+    sigset_t prof;
+    int r;
+    sigemptyset(&prof);
+    sigaddset(&prof,SIGPROF);
+    sigprocmask(SIG_BLOCK,&prof,NULL);
+#endif
+
     printf("GCL (GNU Common Lisp)  %s  %ld pages\n",LISP_IMPLEMENTATION_VERSION,real_maxpage);
     fflush(stdout);
     
-    if (ARGC>1) {
-      massert(ARGV[1][strlen(ARGV[1])-1]=='/');
-      system_directory=ARGV[1];
-    }
-
     initlisp();
-    lex_new();
+    ihs_top++;lex_new();/*FIXME*/
     
     GBC_enable = TRUE;
     
@@ -579,9 +622,15 @@ main(int argc, char **argv, char **envp) {
 #ifdef NEED_STACK_CHK_GUARD
     __stack_chk_guard=random_ulong();/*Cannot be safely set inside a function which returns*/
 #endif
-  
+
+    if (in_pre_gcl) init_boot();
+
   }
 
+#ifdef _WIN32
+  detect_wine();
+#endif
+
   sSAlisp_maxpagesA->s.s_dbind = make_fixnum(real_maxpage);
 
   ihs_push(Cnil);
@@ -623,22 +672,27 @@ error(char *s)
         if (catch_fatal>0 && interrupt_enable )
             {catch_fatal = -1;
 #ifdef SGC
-	   if (sgc_enabled)
-	     { sgc_quit();}
-	   if (sgc_enabled==0)
+    if (sgc_enabled)
+      sgc_quit();
+    if (sgc_enabled==0)
+#endif
+      install_segmentation_catcher();
+    {
+      FEerror("Caught fatal error [memory may be damaged]: ~a",1,str(s));
+    }
+  }
+  printf("\nUnrecoverable error: %s.\n", s);
+  fflush(stdout);
+#ifdef UNIX
+  gcl_abort();
 #endif
-	     { install_segmentation_catcher() ;}
-	   FEerror("Caught fatal error [memory may be damaged]",0); }
-	printf("\nUnrecoverable error: %s.\n", s);
-	fflush(stdout);
-	do_gcl_abort();
 }
 
 static void
 initlisp(void) {
 
         void *v=&v;
-	
+
 	if (NULL_OR_ON_C_STACK(v) == 0
 #if defined(IM_FIX_BASE)
              || NULL_OR_ON_C_STACK(IM_FIX_BASE) == 0
@@ -662,31 +716,32 @@ initlisp(void) {
 		  core_end-1,NULL_OR_ON_C_STACK(core_end-1));
 	  error("NULL_OR_ON_C_STACK macro invalid");
 	}
-	
+
 	Cnil->fw=0;
 	set_type_of(Cnil,t_symbol);
  	Cnil->c.c_cdr=Cnil;
-	Cnil_body.s.s_dbind = Cnil;
-	Cnil_body.s.s_sfdef = NOT_SPECIAL;
-	Cnil_body.s.s_fillp = 3;
-	Cnil_body.s.s_self = "NIL";
-	Cnil_body.s.s_gfdef = OBJNULL;
-	Cnil_body.s.s_plist = Cnil;
-	Cnil_body.s.s_hpack = Cnil;
-	Cnil_body.s.s_stype = (short)stp_constant;
-	Cnil_body.s.s_mflag = FALSE;
+ 	Cnil->s.s_dbind = Cnil;
+	Cnil->s.s_sfdef = NOT_SPECIAL;
+	Cnil->s.s_name = make_simple_string("NIL");
+ 	Cnil->s.s_gfdef = OBJNULL;
+ 	Cnil->s.s_plist = Cnil;
+ 	Cnil->s.s_hpack = Cnil;
+ 	Cnil->s.s_stype = (short)stp_constant;
+ 	Cnil->s.s_mflag = FALSE;
+	Cnil->s.s_hash = ihash_equal1(Cnil,0);
 	
 	Ct->fw=0;
 	set_type_of(Ct,t_symbol);
-	Ct_body.s.s_dbind = Ct;
-	Ct_body.s.s_sfdef = NOT_SPECIAL;
-	Ct_body.s.s_fillp = 1;
-	Ct_body.s.s_self = "T";
-	Ct_body.s.s_gfdef = OBJNULL;
-	Ct_body.s.s_plist = Cnil;
-	Ct_body.s.s_hpack = Cnil;
-	Ct_body.s.s_stype = (short)stp_constant;
-	Ct_body.s.s_mflag = FALSE;
+ 	Ct->s.s_dbind = Ct;
+	Ct->s.tt=1;
+ 	Ct->s.s_sfdef = NOT_SPECIAL;
+	Ct->s.s_name = make_simple_string("T");
+ 	Ct->s.s_gfdef = OBJNULL;
+ 	Ct->s.s_plist = Cnil;
+ 	Ct->s.s_hpack = Cnil;
+ 	Ct->s.s_stype = (short)stp_constant;
+ 	Ct->s.s_mflag = FALSE;
+	Ct->s.s_hash = ihash_equal1(Ct,0);
 	
 	gcl_init_symbol();
 
@@ -706,8 +761,9 @@ initlisp(void) {
 	sSlambda_block_closure = make_si_ordinary("LAMBDA-BLOCK-CLOSURE");
 	sLspecial = make_ordinary("SPECIAL");
 
-	
 	NewInit();
+	init_boot();
+
 	gcl_init_typespec();
 	gcl_init_number();
 	gcl_init_character();
@@ -817,57 +873,36 @@ ihs_overflow(void) {
 }
 
 void
-segmentation_catcher(int i) {
+segmentation_catcher(int i, long code, void *scp, char *addr) {
+#ifndef _WIN32
+  void *faddr;
+  faddr=GET_FAULT_ADDR(sig,code,scp,addr); 
+
+  if ((cstack_dir==-1 && faddr < (void *)cs_limit && (void *)cs_limit-faddr <= PAGESIZE) ||
+      (cstack_dir==1 && faddr > (void *)cs_limit && faddr-(void *)cs_limit <= PAGESIZE))
+    FEerror("Control stack overflow.",0); /*FIXME -- provide getrlimit here.*/
+  else 
+    printf("Segmentation violation: c stack ok:signalling error");
+#endif
   error("Segmentation violation.");
 }
 
-/* static void */
-/* cs_overflow(void) { */
-/* #ifdef AV */
-/* 	if (cs_limit < cs_org - cssize) */
-/* 		error("control stack overflow"); */
-/* 	cs_limit -= CSGETA; */
-/* #endif */
-/* #ifdef MV */
+DEFUN("BYE",object,fSbye,SI,0,1,NONE,OI,OO,OO,OO,(fixnum exit_code,...),"") {
 
+  if (!INIT_NARGS(0)) exit_code=0;
 
-
-/* #endif */
-/* 	FEerror("Control stack overflow.", 0); */
-/* } */
-
-/* static void */
-/* end_of_file(void) { */
-/* 	error("end of file"); */
-/* } */
-
-DEFUNO_NEW("BYE",object,fSbye,SI
-       ,0,1,NONE,OO,OO,OO,OO,void,Lby,(object exitc),"")
-{	int n=VFUN_NARGS;
-	int exit_code;
-	if (n>=1) exit_code=fix(exitc);else exit_code=0;
-
-/*	printf("Bye.\n"); */
-	exit(exit_code);
+#ifdef UNIX
+  exit(exit_code);
+#else
+  RETURN(1,int,exit_code, 0); 
+#endif
 
 }
 
-
-DEFUN_NEW("QUIT",object,fSquit,SI
-       ,0,1,NONE,OO,OO,OO,OO,(object exitc),"")
-{	return FFN(fSbye)(exitc); }
+DEFUN("QUIT",object,fSquit,SI,0,1,NONE,OI,OO,OO,OO,(fixnum exitc),"") {
+  return FFN(fSbye)(exitc); 
+}
  
-/* DEFUN_NEW("EXIT",object,fLexit,LISP */
-/*        ,0,1,NONE,OI,OO,OO,OO,(fixnum exitc),"") */
-/* {	return fLbye(exitc); } */
- 
-
-/*  c_trace(void) */
-/*  { */
-/*  #ifdef AOSVS */
-
-/*  #endif */
-/*  } */
 
 static void
 FFN(siLargc)(void) {
@@ -898,13 +933,16 @@ FFN(siLgetenv)(void) {
   
   check_arg(1);
   check_type_string(&vs_base[0]);
-  if (vs_base[0]->st.st_fillp >= 256)
+  if (VLEN(vs_base[0]) >= 256)
     FEerror("Too long name: ~S.", 1, vs_base[0]);
-  for (i = 0;  i < vs_base[0]->st.st_fillp;  i++)
+  for (i = 0;  i < VLEN(vs_base[0]);  i++)
     name[i] = vs_base[0]->st.st_self[i];
   name[i] = '\0';
-  if ((value = getenv(name)) != NULL)
-    {vs_base[0] = make_simple_string(value);
+  if ((value = getenv(name)) != NULL) {
+#ifdef _WIN32
+    fix_filename(NULL, value);
+#endif    
+    vs_base[0] = make_simple_string(value);
 #ifdef FREE_GETENV_RESULT
     free(value);
     
@@ -932,17 +970,13 @@ FFN(siLcheck_vs)(void) {
   vs_base[0] = Cnil;
 }
 
-static object
-FFN(siLcatch_fatal)(int i) {
+DEFUN("CATCH-FATAL",object,fScatch_fatal,SI,1,1,NONE,OI,OO,OO,OO,(fixnum i),"") {
   catch_fatal=i;
   return Cnil;
 }
 
-LFD(siLreset_stack_limits)(void)
-{
-  long i=0;
+DEFUN("RESET-STACK-LIMITS",object,fSreset_stack_limits,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
 
-  check_arg(0);
   if(catch_fatal <0) catch_fatal=1;
 #ifdef SGC	
   {extern int fault_count ; fault_count = 0;}
@@ -963,17 +997,15 @@ LFD(siLreset_stack_limits)(void)
     ihs_limit = ihs_org + stack_multiple *  IHSSIZE;
   else
     error("can't reset ihs_limit");
-  if (cs_base==cs_org)
-    cs_org=(void *)&i;
+  cs_org=alloca(1);
 #ifdef __ia64__
  {
    extern void * GC_save_regs_in_stack();
-   if (cs_base2==cs_org2)
-     cs_org2=GC_save_regs_in_stack();
+   cs_org2=GC_save_regs_in_stack();
  }
 #endif
   /* reset_cstack_limit(i); */
-  vs_base[0] = Cnil;
+ RETURN1(Cnil);
 }
 
 #define COPYSTACK(org,p,typ,lim,top,geta,size) \
@@ -1014,6 +1046,7 @@ LFD(siLinit_system)(void) {
   check_arg(0);
   gcl_init_system(sSAno_initA);
   vs_base[0] = Cnil;
+  vs_top=vs_base+1;
 }
 
 static void
@@ -1024,38 +1057,18 @@ FFN(siLuser_init)(void) {
   vs_base[0] = Cnil;
 }
 
-/* static void */
-/* FFN(siLaddress)(void) { */
-/*   check_arg(1); */
-/*   vs_base[0] = make_fixnum((long)vs_base[0]); */
-/* } */
-
-DEFUN_NEW("NANI",object,fSnani,SI,1,1,NONE,OI,OO,OO,OO,(fixnum address),"") {
+DEFUN("NANI",object,fSnani,SI,1,1,NONE,OI,OO,OO,OO,(fixnum address),"") {
 
   RETURN1((object)address);
 
 }
 
-DEFUN_NEW("ADDRESS",object,fSaddress,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+DEFUN("ADDRESS",object,fSaddress,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
 
   RETURN1(x);
 
 }
 
-/* static void */
-/* FFN(siLnani)(void) { */
-/*   check_arg(1); */
-
-/*   switch (type_of(vs_base[0])) { */
-/*   case t_fixnum: */
-/*     vs_base[0]=(object)fix(vs_base[0]); */
-/*     break; */
-/*   default: */
-/*     FEerror("Cannot coerce ~s to an address",1,vs_base[0]); */
-/*   } */
-
-/* } */
-
 static void
 FFN(siLinitialization_failure)(void) {
   check_arg(0);
@@ -1063,24 +1076,16 @@ FFN(siLinitialization_failure)(void) {
   do_gcl_abort();
 }
 
-DEFUNO_NEW("IDENTITY",object,fLidentity,LISP
-       ,1,1,NONE,OO,OO,OO,OO,void,Lidentity,(object x0),"")
-{
-	/* 1 args */
+DEFUN("IDENTITY",object,fLidentity,LISP,1,1,NONE,OO,OO,OO,OO,(object x0),"") {
   RETURN1 (x0);
 }
 
-DEFUNO_NEW("LDB1",object,fSldb1,SI
-       ,3,3,NONE,OI,II,OO,OO,void,Lldb1,(fixnum a,fixnum b, fixnum c),"")
-{
-  RETURN1 (make_fixnum(((((~(-1 << (a))) << (b)) & (c)) >> (b))));
+DEFUN("GCL-COMPILE-TIME",object,fSgcl_compile_time,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+  RETURN1 (make_simple_string(__DATE__ " " __TIME__));
 }
 
-DEFUN_NEW("LISP-IMPLEMENTATION-VERSION",object,fLlisp_implementation_version,LISP
-       ,0,0,NONE,OO,OO,OO,OO,(void),"")
-{
-	/* 0 args */
-	RETURN1((make_simple_string(LISP_IMPLEMENTATION_VERSION)));
+DEFUN("LISP-IMPLEMENTATION-VERSION",object,fLlisp_implementation_version,LISP,0,0,NONE,OO,OO,OO,OO,(void),"") {
+  RETURN1((make_simple_string(LISP_IMPLEMENTATION_VERSION)));
 }
 
 static void
@@ -1107,7 +1112,7 @@ FFN(siLsave_system)(void) {
 }
 
 DEFVAR("*LISP-MAXPAGES*",sSAlisp_maxpagesA,SI,make_fixnum(real_maxpage),"");
-DEFVAR("*SYSTEM-DIRECTORY*",sSAsystem_directoryA,SI,make_simple_string(system_directory),"");
+DEFVAR("*SYSTEM-DIRECTORY*",sSAsystem_directoryA,SI,Cnil,"");
 DEFVAR("*MULTIPLY-STACKS*",sSAmultiply_stacksA,SI,Cnil,"");
 DEF_ORDINARY("TOP-LEVEL",sStop_level,SI,"");
 DEFVAR("*COMMAND-ARGS*",sSAcommand_argsA,SI,sLnil,"");
@@ -1115,7 +1120,7 @@ DEFVAR("*COMMAND-ARGS*",sSAcommand_argsA,SI,sLnil,"");
 static void
 init_main(void) {
 
-  make_si_function("BY", Lby);
+  /* make_function("BY", Lby); */
   make_si_function("ARGC", siLargc);
   make_si_function("ARGV", siLargv);
   
@@ -1123,11 +1128,8 @@ init_main(void) {
   
   make_si_function("MARK-VS", siLmark_vs);
   make_si_function("CHECK-VS", siLcheck_vs);
-  make_si_function("RESET-STACK-LIMITS", siLreset_stack_limits);
   make_si_function("INIT-SYSTEM", siLinit_system);
   make_si_function("USER-INIT", siLuser_init);
-  /* make_si_function("ADDRESS", siLaddress); */
-  /* make_si_function("NANI", siLnani); */
   make_si_function("INITIALIZATION-FAILURE",
 		   siLinitialization_failure);
   
@@ -1154,11 +1156,15 @@ init_main(void) {
   ADD_FEATURE("GPROF");
 #endif	 
   
-#ifndef _WIN32	 
+#ifndef _WIN32
   ADD_FEATURE("UNIX");
-#endif	
+#endif
 
-#ifdef _WIN32
+#if defined ( DARWIN )
+  ADD_FEATURE("DARWIN");
+#endif
+
+#if defined ( _WIN32 ) 
   ADD_FEATURE("WINNT");
   ADD_FEATURE("WIN32");
 #endif
@@ -1173,9 +1179,7 @@ init_main(void) {
 #ifdef SGC
   ADD_FEATURE("SGC");
 #endif	 
-/* #ifdef  ADDITIONAL_FEATURES */
-/*   ADDITIONAL_FEATURES; */
-/* #endif */
+
   ADD_FEATURE(HOST_CPU);
   ADD_FEATURE(HOST_KERNEL);
 #ifdef HOST_SYSTEM
@@ -1205,7 +1209,7 @@ init_main(void) {
 	 && ((-Seven)/ (-Three)) == 2)
       { ADD_FEATURE("TRUNCATE_USE_C");
       }  }
-#endif	 
+#endif
   
 #ifdef USE_READLINE
 #ifdef READLINE_IS_EDITLINE
@@ -1229,10 +1233,20 @@ init_main(void) {
   ADD_FEATURE("GNU-LD");
 #endif
   
+#ifndef NO_C99
+  ADD_FEATURE("C99");
+#endif	 
+
 #ifdef STATIC_LINKING
   ADD_FEATURE("STATIC");
 #endif	 
 
+#if SIZEOF_LONG==8
+  ADD_FEATURE("64BIT");
+#endif	 
+  ADD_FEATURE("INTDIV");
+  ADD_FEATURE("DYNAMIC-EXTENT");
+
 #ifdef LARGE_MEMORY_MODEL
   ADD_FEATURE("LARGE-MEMORY-MODEL");
 #endif
@@ -1240,8 +1254,8 @@ init_main(void) {
   make_special("*FEATURES*",features);}
   
   make_si_function("SAVE-SYSTEM", siLsave_system);
-  make_si_sfun("CATCH-FATAL",siLcatch_fatal,ARGTYPE1(f_fixnum));
-  make_si_function("WARN-VERSION",Lidentity);
+/*   make_si_sfun("CATCH-FATAL",siLcatch_fatal,ARGTYPE1(f_fixnum)); */
+  /* make_si_function("WARN-VERSION",Lidentity); */
   
 }
 
@@ -1285,7 +1299,7 @@ my_pa(bfd_vma addr,struct disassemble_info *dinfo) {
 #endif
 
 
-DEFUN_NEW("DISASSEMBLE-INSTRUCTION",object,fSdisassemble_instruction,SI,1,1,NONE,OI,OO,OO,OO,(fixnum addr),"") {
+DEFUN("DISASSEMBLE-INSTRUCTION",object,fSdisassemble_instruction,SI,1,1,NONE,OI,OO,OO,OO,(fixnum addr),"") {
 
 #if defined(HAVE_DIS_ASM_H) && defined(OUTPUT_ARCH)
 
@@ -1334,12 +1348,12 @@ typedef struct {
 #define tblookup(a_,b_) ({Tbl *tb=tb1;(b_)=(a_);for (;tb->tt && tb->b_!=(b_);tb++);tb->tt;})
 #define mtm_of(a_)      (a_)>=t_other ? NULL : tm_of(a_)
 
-DEFUN_NEW("FUNCTION-BY-ADDRESS",object,fSfunction_by_address,SI,1,1,NONE,OI,OO,OO,OO,(fixnum ad),"") {
+DEFUN("FUNCTION-BY-ADDRESS",object,fSfunction_by_address,SI,1,1,NONE,OI,OO,OO,OO,(fixnum ad),"") {
 
   ufixnum m=-1,mm,j;
   void *o;
   object x,xx=Cnil;
-  Tbl tb1[]={Tblof(t_sfun),Tblof(t_cfun),Tblof(t_vfun),Tblof(t_afun),Tblof(t_gfun),Tblof(t_closure),Tblof(t_cclosure),{0}};
+  Tbl tb1[]={Tblof(t_function),{0}};
   struct typemanager *tp;
   enum type tt;
   struct pageinfo *v;
@@ -1349,11 +1363,11 @@ DEFUN_NEW("FUNCTION-BY-ADDRESS",object,fSfunction_by_address,SI,1,1,NONE,OI,OO,O
       if (tblookup(mtm_of(v->type),tp))
 	for (o=pagetochar(page(v)),j=tp->tm_nppage;j--;o+=tp->tm_size)
 	  if (tblookup(type_of((x=o)),tt))
-	    if (!is_free(x) && (mm=ad-(ufixnum)x->sfn.sfn_self)<m) {
+	    if (!is_free(x) && (mm=ad-(ufixnum)x->fun.fun_self)<m) {
 	      m=mm;
 	      xx=x;
 	    }
-  
+
   return xx;
 
 }
