@@ -31,21 +31,26 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 LFD(Lvalues)(void)
 {
 	if (vs_base == vs_top) vs_base[0] = Cnil;
+	if (vs_top-vs_base > MULTIPLE_VALUES_LIMIT)
+	  FEerror("Too many function call values", 0);
 }
 
 LFD(Lvalues_list)(void)
 {
 
-	object list;
+	object x;
 
 	check_arg(1);
-	list = vs_base[0];
+	x = vs_base[0];
 	vs_top = vs_base;
-	while (!endp_prop(list)) {	
-		vs_push(MMcar(list));
-		list = MMcdr(list);
+	while (!endp(x)) {	
+		vs_push(MMcar(x));
+		x = MMcdr(x);
 	}
 	if (vs_top == vs_base) vs_base[0] = Cnil;
+	if (vs_top-vs_base > MULTIPLE_VALUES_LIMIT)
+	  FEerror("Too many function call values", 0);
+
 }
 
 static void
@@ -129,7 +134,7 @@ FFN(Fmultiple_value_prog1)(object forms)
 void
 gcl_init_multival(void)
 {
-	make_constant("MULTIPLE-VALUES-LIMIT",make_fixnum(32));
+	make_constant("MULTIPLE-VALUES-LIMIT",make_fixnum(1+MULTIPLE_VALUES_LIMIT));
 	make_function("VALUES",Lvalues);
 	make_function("VALUES-LIST",Lvalues_list);
 	make_special_form("MULTIPLE-VALUE-CALL",Fmultiple_value_call);
