@@ -23,13 +23,11 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 	Arithmetic operations
 */
 #define NEED_MP_H
+#define NEED_ISFINITE
 #include "include.h"
 
 #include "num_include.h"
 
-void
-zero_divisor(void);
-
 object fixnum_add(fixnum i, fixnum j)
 {
 
@@ -55,14 +53,14 @@ object fixnum_sub(fixnum i, fixnum j)
       }
    MPOP(return,subss,i,j);
    } else { /* i < 0 */
-     if ((MOST_NEGATIVE_FIX -i) <= -j) {
+     if (j <= (i-MOST_NEGATIVE_FIX)) {
        return make_fixnum(i-j);
      }
    MPOP(return,subss,i,j);
    }
 }
 
-object 
+inline object 
 fixnum_times(fixnum i, fixnum j) {
 
 #ifdef HAVE_CLZL
@@ -103,6 +101,237 @@ number_to_complex(object x)
 	}
 }
 
+static object
+integer_exact_quotient(object r,object x,object y) {
+
+  if (y==small_fixnum(1) || x==small_fixnum(0))
+    return x;
+
+  if (type_of(x)==t_fixnum)  /* no in_place for fixnums as could be small */
+    return make_fixnum((type_of(y)==t_fixnum ? fix(x)/fix(y) : -1));
+  /* Only big dividing a fix is most-negative-fix/abs(most-negative-fix)*/
+
+  if (type_of(y)==t_fixnum)
+    mpz_divexact_ui(MP(r),MP(x),fix(y));
+  else
+    mpz_divexact(MP(r),MP(x),MP(y));
+
+  return normalize_big(r);
+
+}
+
+static object
+fixnum_abs(object x) {
+
+  if (type_of(x)==t_fixnum) {
+
+    fixnum f=fix(x);
+
+    return f==MOST_NEGATIVE_FIX ? sSPminus_most_negative_fixnumP->s.s_dbind : (f<0 ? make_fixnum(-f) : x);
+
+  }
+
+  return x;
+
+}
+
+
+
+
+static object
+get_gcd_r_abs(object r,object x,object y) {
+
+  if (x==small_fixnum(1) || y==small_fixnum(1))
+    return small_fixnum(1);
+
+  switch(type_of(x)) {
+  case t_fixnum:
+    switch(type_of(y)) {
+    case t_fixnum:
+      return make_fixnum(fixnum_gcd(fix(x),fix(y)));
+    default:
+      mpz_gcd_ui(MP(r),MP(y),fix(x));
+      return normalize_big(r);
+    }
+  default:
+    switch(type_of(y)) {
+    case t_fixnum:
+      mpz_gcd_ui(MP(r),MP(x),fix(y));
+      return normalize_big(r);
+    default:
+      mpz_gcd(MP(r),MP(x),MP(y));
+      return normalize_big(r);
+
+    }
+  }
+}
+
+static object
+get_gcd_r(object r,object x,object y) {
+
+  return get_gcd_r_abs(r,fixnum_abs(x),fixnum_abs(y));
+
+}
+
+
+object
+get_gcd(object x,object y) {
+
+  x=get_gcd_r(big_fixnum1,x,y);
+  return x==big_fixnum1 ? replace_big(x) : x;
+
+}
+
+static object
+get_gcd_abs(object x,object y) {
+
+  x=get_gcd_r_abs(big_fixnum1,x,y);
+  return x==big_fixnum1 ? replace_big(x) : x;
+
+}
+
+static object
+integer_times(object r,object a,object b) {
+
+  if (a==small_fixnum(1))
+    return b;
+
+  if (b==small_fixnum(1))
+    return a;
+
+  if (type_of(a)==t_fixnum)
+    if (type_of(b)==t_fixnum)
+      return fixnum_times(fix(a),fix(b));
+    else {
+      mpz_mul_si(MP(r),MP(b),fix(a));
+      return normalize_big(r);
+    }
+  else
+    if (type_of(b)==t_fixnum) {
+      mpz_mul_si(MP(r),MP(a),fix(b));
+      return normalize_big(r);
+    } else {
+      mpz_mul(MP(r),MP(a),MP(b));
+      return normalize_big(r);
+    }
+}
+
+#define mneg(a_) ((a_)==MOST_NEGATIVE_FIX ? (ufixnum)(a_) : (ufixnum)(-(a_)))
+#define mpz_add_si(a_,b_,c_) ((c_)<0 ? mpz_sub_ui(a_,b_,mneg(c_)) : mpz_add_ui(a_,b_,c_))
+#define mpz_sub_si(a_,b_,c_) ((c_)<0 ? mpz_add_ui(a_,b_,mneg(c_)) : mpz_sub_ui(a_,b_,c_))
+
+
+static object
+integer_add(object r,object a,object b) {
+
+  if (a==small_fixnum(0))
+    return b;
+
+  if (b==small_fixnum(0))
+    return a;
+
+  if (type_of(a)==t_fixnum)
+    if (type_of(b)==t_fixnum)
+      return fixnum_add(fix(a),fix(b));
+    else {
+      mpz_add_si(MP(r),MP(b),fix(a));
+      return normalize_big(r);
+    }
+  else
+    if (type_of(b)==t_fixnum) {
+      mpz_add_si(MP(r),MP(a),fix(b));
+      return normalize_big(r);
+    } else {
+      mpz_add(MP(r),MP(a),MP(b));
+      return normalize_big(r);
+    }
+}
+
+static object
+integer_sub(object r,object a,object b) {
+
+  /* if (a==small_fixnum(0)) */
+  /*   return b; */
+
+  if (b==small_fixnum(0))
+    return a;
+
+  if (type_of(a)==t_fixnum)
+    if (type_of(b)==t_fixnum)
+      return fixnum_sub(fix(a),fix(b));
+    else {
+      mpz_sub_si(MP(r),MP(b),fix(a));
+      mpz_neg(MP(r),MP(r));
+      return normalize_big(r);
+    }
+  else
+    if (type_of(b)==t_fixnum) {
+      mpz_sub_si(MP(r),MP(a),fix(b));
+      return normalize_big(r);
+    } else {
+      mpz_sub(MP(r),MP(a),MP(b));
+      return normalize_big(r);
+    }
+}
+
+static object
+ratio_mult_with_cancellation(object a,object b,object c,object d) {
+
+  object gad,gbc;
+
+  gad=get_gcd_r(big_fixnum2,a,d);
+  gbc=get_gcd_r(big_fixnum5,b,c);
+
+  a=integer_exact_quotient(big_fixnum3,a,gad);
+  c=integer_exact_quotient(big_fixnum4,c,gbc);
+  a=integer_times(big_fixnum3,a,c);/*integer_times can clobber big_fixnum1*/
+  if (a==big_fixnum3 || a==big_fixnum4)
+    a=replace_big(a);
+
+  b=integer_exact_quotient(big_fixnum3,b,gbc);
+  d=integer_exact_quotient(big_fixnum4,d,gad);
+  b=integer_times(big_fixnum3,b,d);/*integer_times can clobber big_fixnum1*/
+  if (b==big_fixnum3 || b==big_fixnum4)
+    b=replace_big(b);
+
+  return make_ratio(a,b,1);
+
+}
+
+static object
+ratio_op_with_cancellation(object a,object b,object c,object d,object (*op)(object,object,object)) {
+
+  object b0,d0,g,t,g1;
+
+  b0=b;
+  d0=d;
+
+  g=get_gcd_r(big_fixnum2,b,d);
+
+  b=integer_exact_quotient(big_fixnum3,b,g);
+  d=integer_exact_quotient(big_fixnum4,d,g);
+
+  c=integer_times(big_fixnum3,b,c);/*integer_times can clobber big_fixnum1*/
+  a=integer_times(big_fixnum5,a,d);/*integer_times can clobber big_fixnum1*/
+  t=op(big_fixnum3,a,c);
+
+  g1=get_gcd_r(big_fixnum2,t,g);
+
+  t=integer_exact_quotient(big_fixnum3,t,g1);
+  if (t==big_fixnum3 || t==big_fixnum4 || t==big_fixnum5)
+    t=replace_big(t);
+
+  b=integer_exact_quotient(big_fixnum2,b0,g1);
+  b=integer_times(big_fixnum2,b,d);/*integer_times can clobber big_fixnum1*/
+  if (b==big_fixnum2 || b==big_fixnum4)
+    b=replace_big(b);
+
+  return make_ratio(t,b,1);
+
+}
+
+
+
 object
 number_plus(object x, object y)
 {
@@ -116,9 +345,9 @@ number_plus(object x, object y)
 		case t_bignum:
 		  MPOP(return, addsi,fix(x),MP(y));
 		case t_ratio:
-			z = number_plus(number_times(x, y->rat.rat_den),
-					y->rat.rat_num);
-			return make_ratio(z, y->rat.rat_den);
+		  return ratio_op_with_cancellation(x,small_fixnum(1),
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_add);
 		case t_shortfloat:
 			dx = (double)(fix(x));
 			dy = (double)(sf(y));
@@ -140,8 +369,9 @@ number_plus(object x, object y)
 		case t_bignum:
 		  MPOP(return,addii,MP(y),MP(x)); 
 		case t_ratio:
-			z = number_plus(number_times(x, y->rat.rat_den), y->rat.rat_num);
-			return make_ratio(z, y->rat.rat_den);
+		  return ratio_op_with_cancellation(x,small_fixnum(1),
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_add);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -160,17 +390,13 @@ number_plus(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		case t_bignum:
-			
-			z = number_plus(x->rat.rat_num,
-					number_times(x->rat.rat_den, y));
-			z = make_ratio(z, x->rat.rat_den);
-			return(z);
+		  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						    y,small_fixnum(1),
+						    integer_add);
 		case t_ratio:
-
-			z = number_plus(number_times(x->rat.rat_num,y->rat.rat_den),
-					number_times(x->rat.rat_den,y->rat.rat_num));
-			z = make_ratio(z,number_times(x->rat.rat_den,y->rat.rat_den));
-			return(z);
+   		  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_add);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -208,7 +434,7 @@ number_plus(object x, object y)
 		}
 	SHORTFLOAT:
 		z = alloc_object(t_shortfloat);
-		sf(z) = (shortfloat)(dx + dy);
+		sf(z) = (shortfloat)(dx + dy);/*FPE*/
 		return(z);
 
 	case t_longfloat:
@@ -262,10 +488,9 @@ one_plus(object x)
 	case t_bignum:
 	  MPOP(return,addsi,1,MP(x));
 	case t_ratio:
-		z = number_plus(x->rat.rat_num, x->rat.rat_den);
-		z = make_ratio(z, x->rat.rat_den);
-		return(z);
-
+	  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+					    small_fixnum(1),small_fixnum(1),
+					    integer_add);
 	case t_shortfloat:
 		dx = (double)(sf(x));
 		z = alloc_object(t_shortfloat);
@@ -301,13 +526,13 @@ number_minus(object x, object y)
 		switch(type_of(y)) {
 		case t_fixnum:
 		  return fixnum_sub(fix(x),fix(y));
-		  /* MPOP(return,subss,fix(x),fix(y)); */
+/* 		  MPOP(return,subss,fix(x),fix(y)); */
 		case t_bignum:
 		  MPOP(return, subsi,fix(x),MP(y));
 		case t_ratio:
-			z = number_minus(number_times(x, y->rat.rat_den), y->rat.rat_num);
-			z = make_ratio(z, y->rat.rat_den);
-			return(z);
+		  return ratio_op_with_cancellation(x,small_fixnum(1),
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_sub);
 		case t_shortfloat:
 			dx = (double)(fix(x));
 			dy = (double)(sf(y));
@@ -329,9 +554,9 @@ number_minus(object x, object y)
 		case t_bignum:
 		  MPOP(return,subii,MP(x),MP(y));
 		case t_ratio:
-			z = number_minus(number_times(x, y->rat.rat_den), y->rat.rat_num);
-			z = make_ratio(z, y->rat.rat_den);
-			return(z);
+		  return ratio_op_with_cancellation(x,small_fixnum(1),
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_sub);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -350,14 +575,13 @@ number_minus(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		case t_bignum:
-			z = number_minus(x->rat.rat_num, number_times(x->rat.rat_den, y));
-			z = make_ratio(z, x->rat.rat_den);
-			return(z);
+		  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						    y,small_fixnum(1),
+						    integer_sub);
 		case t_ratio:
-			z = number_minus(number_times(x->rat.rat_num,y->rat.rat_den),
-					 (number_times(x->rat.rat_den,y->rat.rat_num)));
-			z = make_ratio(z,number_times(x->rat.rat_den,y->rat.rat_den));
-			return(z);
+   		  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						    y->rat.rat_num,y->rat.rat_den,
+						    integer_sub);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -395,7 +619,7 @@ number_minus(object x, object y)
 		}
 	SHORTFLOAT:
 		z = alloc_object(t_shortfloat);
-		sf(z) = (shortfloat)(dx - dy);
+		sf(z) = (shortfloat)(dx - dy);/*FPE*/
 		return(z);
 
 	case t_longfloat:
@@ -446,10 +670,9 @@ one_minus(object x)
 	case t_bignum:
 	  MPOP(return,addsi,-1,MP(x));
 	case t_ratio:
-		z = number_minus(x->rat.rat_num, x->rat.rat_den);
-		z = make_ratio(z, x->rat.rat_den);
-		return(z);
-
+	  return ratio_op_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+					    small_fixnum(1),small_fixnum(1),
+					    integer_sub);
 	case t_shortfloat:
 		dx = (double)(sf(x));
 		z = alloc_object(t_shortfloat);
@@ -475,23 +698,19 @@ one_minus(object x)
 object
 number_negate(object x)
 {
-	object	z, z1;
+	object	z;
 
 	switch (type_of(x)) {
 
 	case t_fixnum:
 		if(fix(x) == MOST_NEGATIVE_FIX)
-		  return fixnum_add(1,MOST_POSITIVE_FIX);
+		  return sSPminus_most_negative_fixnumP->s.s_dbind; /* fixnum_add(1,MOST_POSITIVE_FIX); */
 		else
 		  return(make_fixnum(-fix(x)));
 	case t_bignum:
 		return big_minus(x);
 	case t_ratio:
-		z1 = number_negate(x->rat.rat_num);
-		z = alloc_object(t_ratio);
-		z->rat.rat_num = z1;
-		z->rat.rat_den = x->rat.rat_den;
-		return(z);
+	  return make_ratio(number_negate(x->rat.rat_num),x->rat.rat_den,1);
 
 	case t_shortfloat:
 		z = alloc_object(t_shortfloat);
@@ -526,12 +745,11 @@ number_times(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		  return fixnum_times(fix(x),fix(y));
-		  /* MPOP(return,mulss,fix(x),fix(y)); */
 		case t_bignum:
 		  MPOP(return,mulsi,fix(x),MP(y));
 		case t_ratio:
-			z = make_ratio(number_times(x, y->rat.rat_num), y->rat.rat_den);
-			return(z);
+		  return ratio_mult_with_cancellation(x,small_fixnum(1),
+						      y->rat.rat_num,y->rat.rat_den);
 		case t_shortfloat:
 			dx = (double)(fix(x));
 			dy = (double)(sf(y));
@@ -553,8 +771,8 @@ number_times(object x, object y)
 		case t_bignum:
 		  MPOP(return,mulii,MP(y),MP(x));
 		case t_ratio:
-			z = make_ratio(number_times(x, y->rat.rat_num), y->rat.rat_den);
-			return(z);
+		  return ratio_mult_with_cancellation(x,small_fixnum(1),
+						      y->rat.rat_num,y->rat.rat_den);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -573,12 +791,11 @@ number_times(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		case t_bignum:
-			z = make_ratio(number_times(x->rat.rat_num, y), x->rat.rat_den);
-			return(z);
+		  return ratio_mult_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						      y,small_fixnum(1));
 		case t_ratio:
-			z = make_ratio(number_times(x->rat.rat_num,y->rat.rat_num),
-				       number_times(x->rat.rat_den,y->rat.rat_den));
-			return(z);
+		  return ratio_mult_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						      y->rat.rat_num,y->rat.rat_den);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -616,7 +833,9 @@ number_times(object x, object y)
 		}
 	SHORTFLOAT:
 		z = alloc_object(t_shortfloat);
-		sf(z) = (shortfloat)(dx * dy);
+		sf(z) = (shortfloat)(dx * dy);/*FPE*/
+		/* if (number_zerop(z) && dx && dy) FLOATING_POINT_UNDERFLOW(sLA,list(2,x,y)); */
+		/* if (!ISFINITE(sf(z)) && ISFINITE(dx) && ISFINITE(dy)) FLOATING_POINT_OVERFLOW(sLA,list(2,x,y)); */
 		return(z);
 
 	case t_longfloat:
@@ -638,7 +857,9 @@ number_times(object x, object y)
 		}
 	LONGFLOAT:
 		z = alloc_object(t_longfloat);
-		lf(z) = dx * dy;
+		lf(z) = dx * dy;/*FPE*/
+		/* if (number_zerop(z) && dx && dy) FLOATING_POINT_UNDERFLOW(sLA,list(2,x,y)); */
+		/* if (!ISFINITE(lf(z)) && ISFINITE(dx) && ISFINITE(dy)) FLOATING_POINT_OVERFLOW(sLA,list(2, x,y));*/
 		return(z);
 
 	case t_complex:
@@ -677,19 +898,17 @@ number_divide(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		case t_bignum:
-			if(number_zerop(y) == TRUE)
-				zero_divisor();
-			if (number_minusp(y) == TRUE) {
-				x = number_negate(x);
-				y = number_negate(y);
-			}
-			z = make_ratio(x, y);
-			return(z);
+/* 			if(number_zerop(y) == TRUE) */
+/* 				zero_divisor(); */
+/* 			if (number_minusp(y) == TRUE) { */
+/* 				x = number_negate(x); */
+/* 				y = number_negate(y); */
+/* 			} */
+/* 			z = make_ratio(x, y, 0); */
+			return(make_ratio(x, y, 0));
 		case t_ratio:
-			if(number_zerop(y->rat.rat_num))
-				zero_divisor();
-			z = make_ratio(number_times(x, y->rat.rat_den), y->rat.rat_num);
-			return(z);
+		  /* if(number_zerop(y->rat.rat_num)) DIVISION_BY_ZERO(sLD,list(2,x,y)); */
+		  return ratio_mult_with_cancellation(x,small_fixnum(1),y->rat.rat_den,y->rat.rat_num);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -708,14 +927,12 @@ number_divide(object x, object y)
 		switch (type_of(y)) {
 		case t_fixnum:
 		case t_bignum:
-			if (number_zerop(y))
-				zero_divisor();
-			z = make_ratio(x->rat.rat_num, number_times(x->rat.rat_den, y));
-			return(z);
+		  /* if (number_zerop(y)) DIVISION_BY_ZERO(sLD,list(2,x,y)); */
+			return ratio_mult_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+							    small_fixnum(1),y);
 		case t_ratio:
-			z = make_ratio(number_times(x->rat.rat_num,y->rat.rat_den),
-				       number_times(x->rat.rat_den,y->rat.rat_num));
-			return(z);
+		  return ratio_mult_with_cancellation(x->rat.rat_num,x->rat.rat_den,
+						      y->rat.rat_den,y->rat.rat_num);
 		case t_shortfloat:
 			dx = number_to_double(x);
 			dy = (double)(sf(y));
@@ -753,9 +970,8 @@ number_divide(object x, object y)
 		}
 	SHORTFLOAT:
 		z = alloc_object(t_shortfloat);
-		if (dy == 0.0)
-			zero_divisor();
-		sf(z) = (shortfloat)(dx / dy);
+		/* if (dy == 0.0) DIVISION_BY_ZERO(sLD,list(2,x,y)); */
+		sf(z) = (shortfloat)(dx / dy);/*FPE ?*/
 		return(z);
 
 
@@ -778,8 +994,7 @@ number_divide(object x, object y)
 		}
 	LONGFLOAT:
 		z = alloc_object(t_longfloat);
-		if (dy == 0.0)
-			zero_divisor();
+		/* if (dy == 0.0) DIVISION_BY_ZERO(sLD,list(2,x,y)); */
 		lf(z) = dx / dy;
 		return(z);
 
@@ -792,8 +1007,8 @@ number_divide(object x, object y)
 		y = number_to_complex(y);
 		z1 = number_times(y->cmp.cmp_real, y->cmp.cmp_real);
 		z2 = number_times(y->cmp.cmp_imag, y->cmp.cmp_imag);
-		if (number_zerop(z3 = number_plus(z1, z2)))
-			zero_divisor();
+		z3 = number_plus(z1, z2);
+		/* if (number_zerop(z3 = number_plus(z1, z2))) DIVISION_BY_ZERO(sLD,list(2,x,y)); */
 		z1 = number_times(x->cmp.cmp_real, y->cmp.cmp_real);
 		z2 = number_times(x->cmp.cmp_imag, y->cmp.cmp_imag);
 		z1 = number_plus(z1, z2);
@@ -812,6 +1027,30 @@ number_divide(object x, object y)
 	}
 }
 
+object
+number_recip(object x) {
+
+  switch (type_of(x)) {
+  case t_fixnum:
+  case t_bignum:
+    return(make_ratio(small_fixnum(1), x, 1));
+  case t_ratio:
+    return(make_ratio(x->rat.rat_den,x->rat.rat_num, 1));
+  case t_shortfloat:
+    return make_shortfloat(1.0/sf(x));
+  case t_longfloat:
+    return make_longfloat(1.0/lf(x));
+  case t_complex:
+    return number_divide(make_complex(x->cmp.cmp_real,number_negate(x->cmp.cmp_imag)),
+			 number_plus(number_times(x->cmp.cmp_real,x->cmp.cmp_real),
+				     number_times(x->cmp.cmp_imag,x->cmp.cmp_imag)));
+  default:
+    FEwrong_type_argument(sLnumber, x);
+    return(Cnil);
+  }
+
+}
+
 object
 integer_divide1(object x, object y,fixnum d) {
   object q;
@@ -830,40 +1069,72 @@ integer_divide2(object x, object y,fixnum d,object *r) {
 
 }
 
-object
-get_gcd_abs(object x,object y) {
+DEFUN("P2",object,fSp2,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
 
-  object r;
-  
-  for (;;) {
-    
-    if (type_of(x) == t_fixnum && type_of(y) == t_fixnum)
-      return make_fixnum(fixnum_gcd(fix(x),fix(y)));
-    
-    if (number_compare(x, y) < 0) {
-      r = x;
-      x = y;
-      y = r;
-    }
-    if (type_of(y) == t_fixnum && fix(y) == 0)
-      return(x);
+  RETURN1(number_plus(x,y));
 
-    integer_quotient_remainder_1(x, y, NULL, &r,0);
-    x = y;
-    y = r;
+}
 
-  }
+DEFUN("M2",object,fSs2,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_minus(x,y));
+
+}
+
+DEFUN("*2",object,fSt2,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_times(x,y));
+
+}
+
+DEFUN("/2",object,fSd2,SI
+	  ,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_divide(x,y));
 
 }
 
 
-object
-get_gcd(object x, object y) {
-  
-  return get_gcd_abs(number_abs(x),number_abs(y));
+DEFUN("NUMBER-PLUS",object,fSnumber_plus,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_plus(x,y));
 
 }
 
+DEFUN("NUMBER-MINUS",object,fSnumber_minus,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_minus(x,y));
+
+}
+
+DEFUN("NUMBER-NEGATE",object,fSnumber_negate,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+
+  RETURN1(number_negate(x));
+
+}
+
+DEFUN("NUMBER-TIMES",object,fSnumber_times,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_times(x,y));
+
+}
+
+DEFUN("NUMBER-DIVIDE",object,fSnumber_divide,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+
+  RETURN1(number_divide(x,y));
+
+}
+
+DEFUN("NUMBER-RECIP",object,fSnumber_recip,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+
+  RETURN1(number_recip(x));
+
+}
+
+
 LFD(Lplus)(void)
 {
         fixnum i, j;
@@ -925,6 +1196,7 @@ LFD(Ldivide)(void)
 		check_type_number(&vs_base[i]);
 	if (j == 1) {
 		vs_base[0] = number_divide(small_fixnum(1), vs_base[0]);
+		vs_top = vs_base+1;
 		return;
 	}
 	for (i = 1; i < j; i++)
@@ -979,7 +1251,7 @@ LFD(Lgcd)(void) {
   vs_push(number_abs(vs_base[0]));
   
   for (i = 1;  i < narg;  i++)
-    vs_base[0] = get_gcd_abs(vs_base[0], number_abs(vs_base[i]));
+    vs_base[0] = get_gcd_abs(vs_base[0],number_abs(vs_base[i]));
 
 }
 
@@ -1019,12 +1291,21 @@ LFD(Llcm)(void) {
 
 }
 
-void
-zero_divisor(void)
-{
-	FEerror("Zero divisor.", 0);
-}
+DEFUN("FACTORIAL",object,fSfactorial,SI,1,1,NONE,OI,OO,OO,OO,(fixnum x),"") {
 
+  object r;
+
+  if (x<0) {
+    object y=make_fixnum(x);
+    TYPE_ERROR(y,sSnon_negative_fixnum);
+    x=fix(y);
+  }
+  r=new_bignum();
+  mpz_fac_ui(MP(r),x);
+  RETURN1(normalize_big(r));
+
+}
+  
 void
 gcl_init_num_arith(void)
 {
