@@ -48,12 +48,12 @@ FFD(Ftagbody)(object body)
 {
 
 	object *old_top = vs_top;
-	object *new_top;
+	object * VOL new_top;
 	object *tinf;
-	object *tinf_base;
+	object * VOL tinf_base;
 	object *env = lex_env;
 	object id = alloc_frame_id();
-	object bodysv = body;
+	VOL object bodysv = body;
 	object label;
 	enum type item_type;
 
@@ -180,7 +180,8 @@ FFN(Fgo)(object args)
 		FEtoo_many_argumentsF(args);
 	lex_tag = lex_tag_sch(MMcar(args));
 	if (MMnull(lex_tag))
-		FEerror("~S is an undefined tag.", 1, MMcar(args));
+/* 		FEerror("~S is an undefined tag.", 1, MMcar(args)); */
+		PROGRAM_ERROR("~S is an undefined tag.", MMcar(args));
 	fr = frs_sch(MMcaddr(lex_tag));
 	if (fr == NULL)
 		FEerror("The tag ~S is missing.", 1, MMcar(args));
