@@ -16,7 +16,10 @@ License for more details.
 
 */
 
-#include <string.h>
+#include <sys/types.h>
+#ifndef _WIN32
+#include <sys/wait.h>
+#endif
 
 #define IN_RUN_PROCESS
 #include "include.h"
@@ -32,6 +35,8 @@ License for more details.
 #include <sys/sockio.h>
 #endif
 
+#include "page.h"
+
 #ifdef RUN_PROCESS
 
 void setup_stream_buffer(object);
@@ -199,12 +204,12 @@ void run_process ( char *name )
 #endif
 
     stream_in = (object) alloc_object(t_stream);
-    stream_in->sm.sm_mode = smm_input;
+    stream_in->sm.tt=stream_in->sm.sm_mode = smm_input;
     stream_in->sm.sm_fp = ofp;
     stream_in->sm.sm_buffer = 0;
     stream_in->sm.sm_flags=0;
     stream_out = (object) alloc_object(t_stream);
-    stream_out->sm.sm_mode = smm_output;
+    stream_out->sm.tt=stream_out->sm.sm_mode = smm_output;
     stream_out->sm.sm_fp = ifp;
     stream_out->sm.sm_buffer = 0;
     stream_out->sm.sm_flags=0;
@@ -344,31 +349,21 @@ gcl_init_socket_function()
 
 
 
-/* LISP - Lisp Wrapper for the "c" code.
- *
- * The lisp OBJECT is passed to the code and a string must be extracted
- * and null terminated to make it work with the "C" code.
- *
- * Lisp Interface code.
- */
+static char *lisp_to_string(object string) {
 
-static char *lisp_to_string(string)
-object string;
-{
-	int	i, len;
-	char	*sself;
-	char	*cstr;
+  int	i, len;
+  char	*sself;
+  char	*cstr;
 
-	len = string->st.st_fillp;
+  len = string->st.st_fillp;
+
+  cstr = (char *) malloc (len+1);
+  sself = &(string->st.st_self[0]);
+  for (i=0; i<len; i++)
+      cstr[i] = sself[i];
+  cstr[i] = 0;
+  return (cstr);
 
-	cstr = (char *) malloc (len+1);
-	sself = &(string->st.st_self[0]);
-	for (i=0; i<len; i++)
-	{
-		cstr[i] = sself[i];
-	}
-	cstr[i] = 0;
-	return (cstr);
 }
 
 /* open_connection - Open_Connection a socket to a server that you know by port number.
@@ -382,6 +377,7 @@ static int open_connection(host,server)
 char	*host;
 int	server;
 {
+	int res;
 	int pid;
 	int	sock;
 	struct	hostent	*hp;
@@ -423,10 +419,13 @@ int	server;
 	}
 
 #ifdef OVM_IO
-	fcntl(sock,F_SETFL,FASYNC | FNDELAY);
+	res = fcntl(sock,F_SETFL,FASYNC | FNDELAY);
 #else
-	fcntl(sock,F_SETFL,FASYNC);
+	res = fcntl(sock,F_SETFL,FASYNC);
 #endif
+	if (res==-1)
+	  FEerror("fnctl F_SETFL error",0);
+
 	return(sock);
 }
 
@@ -455,7 +454,7 @@ enum smmode smm;
 
 	fp = fdopen(socket,mode);
 	stream = (object)  alloc_object(t_stream);
-	stream->sm.sm_mode = (short)smm;
+	stream->sm.tt=stream->sm.sm_mode = (short)smm;
 	stream->sm.sm_fp = fp;
 	stream->sm.sm_buffer = 0;
 
@@ -527,14 +526,14 @@ make_socket_pair()
 
 
   stream_in = (object) alloc_object(t_stream);
-  stream_in->sm.sm_mode = smm_input;
+  stream_in->sm.tt=stream_in->sm.sm_mode = smm_input;
   stream_in->sm.sm_fp = fp1;
   stream_in->sm.sm_buffer = 0;
   stream_in->sm.sm_int = sockets_in[1];
   stream_in->sm.sm_object0=stream_in->sm.sm_object1=OBJNULL;
   stream_in->sm.sm_flags = 0;
   stream_out = (object) alloc_object(t_stream);
-  stream_out->sm.sm_mode = smm_output;
+  stream_out->sm.tt=stream_out->sm.sm_mode = smm_output;
   stream_out->sm.sm_fp = fp2;
   stream_out->sm.sm_buffer = 0;
   setup_stream_buffer(stream_in);
@@ -635,9 +634,159 @@ FFN(siLmake_socket_pair)()
   make_socket_pair();
 }
 
+#define unpack_handle(a_,b_,c_) ({if (!consp(a_))\
+                                    TYPE_ERROR(a_,sLcons);\
+                                  if (type_of(a_->c.c_car)!=t_fixnum)\
+                                    TYPE_ERROR(a_->c.c_car,sLfixnum);\
+                                  b_=fix(a_->c.c_car);\
+                                  if (type_of(a_->c.c_cdr) != t_fixnum)\
+                                    TYPE_ERROR(a_->c.c_cdr,sLfixnum);\
+                                  c_=fix(a_->c.c_cdr);})
+
+
+DEFUN("KILL",object,fSkill,SI,2,2,NONE,OO,IO,OO,OO,(object x,fixnum err),"") {
+
+  fixnum k,l;
+  int e,status;
+
+  unpack_handle(x,k,l);
+
+  if (l>=0) {
+    ASSERT((e=waitpid(k,&status,WNOHANG))>=0);
+    if (e) {
+      if (!WIFEXITED(status)) {
+	ASSERT(WIFSIGNALED(status));
+	FEerror("Child %u died with signal %u\n",k,WTERMSIG(status));
+      } else if ((e=WEXITSTATUS(status)))
+	  FEerror("Child %u exited with error status %d\n",k,e);
+    } else {
+      ASSERT(!kill(k,SIGTERM));
+      ASSERT(waitpid(k,&status,0)==k);
+      if (WIFSIGNALED(status)) {
+	ASSERT(WTERMSIG(status)==SIGTERM);
+      } else {
+	ASSERT(WIFEXITED(status));
+	if ((e=WEXITSTATUS(status)))
+	  FEerror("Child %u exited with error status %d\n",k,e);
+      }
+    }
+/*     ASSERT(!close(l)); */
+    close(l);/*FIXME*/
+    x->c.c_cdr=make_fixnum(-1);
+  }
+  return Cnil;
+}
+  
+DEFUN("SELECT-READ",object,fSselect_read,SI,2,2,NONE,IO,IO,OO,OO,(object x,fixnum usec),"") {
+
+  fd_set fds;
+  fixnum max=-1,k,mask,i;
+  object y=x;
+  struct timeval tv={usec/1000000,usec%1000000};
+
+  FD_ZERO(&fds);
+  if (x!=Cnil && !consp(x))
+    TYPE_ERROR(x,sLlist);
+  for (;!endp(x);x=x->c.c_cdr) {
+    unpack_handle(x->c.c_car,i,k);
+    if (k<0) continue;/*closed stream*/
+    max=max<k ? k : max;
+    FD_SET(k,&fds);
+  }
+  select(max+1,&fds,NULL,NULL,usec < 0 ? NULL : &tv);
+  for (x=y,i=mask=0;!endp(x);x=x->c.c_cdr,i++) {
+    k=fix(x->c.c_car->c.c_cdr);
+    if (k<0) continue;
+    if (FD_ISSET(k,&fds))
+      mask|=(1<<i);
+  }
+  return (object)mask;
+}
+  
+DEFUN("WRITE-POINTER-OBJECT",object,fSwrite_pointer_object,SI,2,2,NONE,OO,OO,OO,OO,(object x,object z),"") {
+
+  object y;
+  fixnum pid,s;
+
+  unpack_handle(z,pid,s);
+  ASSERT(pid);
+
+  y=x;
+  if (stack_alloc_end<=stack_alloc_start || !writable_ptr(x)) 
+    y=OBJNULL;
+  ASSERT(write(s,&y,sizeof(y))==sizeof(y)); 
+  if (y==OBJNULL) {
+    char b[BUFSIZ];
+    stack_alloc_off();
+    y=make_fd_stream(s,smm_output,"w",b);
+    prin1(x,y);
+    fclose(y->sm.sm_fp);
+  }
+  return Cnil;
+}
+
+DEFUN("READ-POINTER-OBJECT",object,fSread_pointer_object,SI,1,1,NONE,OO,OO,OO,OO,(object z),"") {
+
+  object x;
+  fixnum pid,s;
+
+  unpack_handle(z,pid,s);
+  ASSERT(pid);
+
+  ASSERT(read(s,&x,sizeof(x))==sizeof(x));
+  if (x==OBJNULL) {
+    object y;
+    char b[BUFSIZ];
+    /*FIXME this could be somewhat faster if the malloc induced by
+      fdopen could be avoided.*/
+    y=make_fd_stream(s,smm_input,"r",b);
+    x=read_object(y);
+    fclose(y->sm.sm_fp);
+  }
+  FFN(fSkill)(z,1);
+  return x;
+
+}
+
+DEFVAR("*CHILD-STACK-ALLOC*",sSAchild_stack_allocA,SI,make_shortfloat(0.8),"");
+
+DEFUN("FORK",object,fSfork,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
+  int p[2],j=0;
+  pid_t pid;
+
+  ASSERT(!pipe(p));
+
+  ASSERT((pid=fork())>=0);
+  
+  if (!pid) {
+
+    j=1;
+    close(STDIN_FILENO);
+    close(STDOUT_FILENO);
+    close(STDERR_FILENO);
+      
+  }
+
+  close(p[1-j]);
+
+  return MMcons(make_fixnum(pid),make_fixnum(p[j]));
+
+}
+
+
+
 void
 gcl_init_socket_function()
-{
+{ 
+ 
+/*   struct sigaction sa; */
+/*   sa.sa_handler=SIG_IGN; */
+/*   sa.sa_flags=SA_NOCLDWAIT; */
+/*   sigemptyset(&sa.sa_mask); */
+  
+/*   sigaction(SIGCHLD,&sa,NULL); */
+  
   make_si_function("MAKE-SOCKET-STREAM", siLmake_socket_stream); 
   make_si_function("MAKE-SOCKET-PAIR", siLmake_socket_pair);
   make_si_function("RUN-PROCESS", siLrun_process);
