@@ -18,19 +18,18 @@ extern void _cleanup();
 
 LFD(siLsave)(void) {
 
-  char filename[256];
   extern char *kcl_self;
-  
+
   check_arg(1);
   check_type_or_pathname_string_symbol_stream(&vs_base[0]);
-  coerce_to_filename(vs_base[0], filename);
+  coerce_to_filename(vs_base[0], FN1);
 
   gcl_cleanup(1);
   
 #ifdef MEMORY_SAVE
-  MEMORY_SAVE(kcl_self,filename);
+  MEMORY_SAVE(kcl_self,FN1);
 #else	  
-  memory_save(kcl_self, filename);
+  memory_save(kcl_self, FN1);
 #endif	
 
   /*  no return  */
