@@ -38,7 +38,6 @@ License for more details.
 
 #include "ext_sym.h"
 #include "gclincl.h"
-#include <string.h>
 
 #if defined(DARWIN)
 asection *
@@ -60,21 +59,19 @@ round_up(void *address, unsigned long n)
 
 static MY_BFD_BOOLEAN
 madd_archive_element (struct bfd_link_info * link_info,
-		       bfd *abfd,
-		       const char *name) {
+		      bfd *abfd,
+		      const char *name,
+		      bfd **subsbfd) {
 
 return MY_BFD_FALSE;
 
 }
 
 static MY_BFD_BOOLEAN
-mmultiple_definition (struct bfd_link_info * link_info,
-		      const char *name,
-		      bfd *obfd,
-		      asection *osec,
-		      bfd_vma oval,
-		      bfd *nbfd,
-		      asection *nsec,
+mmultiple_definition (struct bfd_link_info *li, 
+		      struct bfd_link_hash_entry *h,
+		      bfd *nbfd, 
+		      asection *nsec, 
 		      bfd_vma nval) {
 
 return MY_BFD_FALSE;
@@ -82,13 +79,10 @@ return MY_BFD_FALSE;
 }
 
 static MY_BFD_BOOLEAN
-mmultiple_common (struct bfd_link_info * link_info,
-		  const char *name,
-		  bfd *obfd,
-		  enum bfd_link_hash_type otype,
-		  bfd_vma osize,
-		  bfd *nbfd,
-		  enum bfd_link_hash_type ntype,
+mmultiple_common (struct bfd_link_info *li, 
+		  struct bfd_link_hash_entry *h,
+		  bfd *nbfd, 
+		  enum bfd_link_hash_type ntype, 
 		  bfd_vma nsize) {
 
 return MY_BFD_FALSE;
@@ -183,7 +177,6 @@ int
 fasload(object faslfile) {
 
   object data;
-  char filename[256];
   int init_address=-1;
   object memory;
   int max_align=0;
@@ -231,9 +224,9 @@ fasload(object faslfile) {
 
   }
   
-  coerce_to_filename(faslfile, filename);
+  coerce_to_filename(faslfile, FN1);
 
-  if (!(b=bfd_openr(filename,0)))
+  if (!(b=bfd_openr(FN1,0)))
     FEerror("Cannot open bfd",0);
   if ((myerr=bfd_get_error()) && myerr!=3) 
     FEerror("Unknown bfd error code on openr",0);
@@ -248,6 +241,66 @@ fasload(object faslfile) {
     FEerror ("Could not craft fp register preservation stubs",0);
 #endif
 
+  if ((u=bfd_get_symtab_upper_bound(b))<0)
+    FEerror("Cannot get symtab uppoer bound",0);
+  q=(asymbol **)ZALLOCA(u);
+  if ((v=bfd_canonicalize_symtab(b,q))<0)
+    FEerror("cannot canonicalize symtab",0);
+
+  *entry_name=bfd_get_symbol_leading_char(b);
+  entry_name_ptr=*entry_name ? entry_name : entry_name+1;
+
+  {
+
+    for (u=0;u<v;u++) {
+      
+      struct bfd_link_hash_entry *h;
+      
+      if (!strncmp(entry_name_ptr,q[u]->name,5)) {
+	init_address=u;
+	continue;
+      }
+      
+      if (link_info.hash) {
+
+	if (!(h=bfd_link_hash_lookup(link_info.hash,q[u]->name,MY_BFD_FALSE,MY_BFD_FALSE,MY_BFD_TRUE))) 
+	  continue;
+
+	if (h->type!=bfd_link_hash_defined) 
+	  FEerror("Undefined symbol ~S",1,make_simple_string(q[u]->name));
+      
+	if (h->u.def.section) {
+	  q[u]->value=h->u.def.value+h->u.def.section->vma;
+	  q[u]->flags|=BSF_WEAK;
+	} else 
+	  FEerror("Symbol without section",0);
+
+      } else {
+
+	struct cons *x;
+/* 	unsigned long ad; */
+
+	if ((x=gethash(str((char *)q[u]->name),sSAlink_hash_tableA->s.s_dbind))->c_cdr==OBJNULL)
+	  continue;
+
+	q[u]->value=fix(x->c_car);
+	q[u]->flags|=BSF_WEAK;
+
+      }
+
+    }
+
+  }
+
+
+  /* Perhaps all arch dependent stuff could be placed here, e.g. the
+     darwin code below.  Aurelien?  This snippet should likely be a
+     CPP define in config.h.*/
+
+#ifdef SPECIAL_BFD
+  SPECIAL_BFD(b,q,v);
+#endif
+
   current=NULL;
   for (s=b->sections;s;s=s->next) {
 
@@ -278,6 +331,7 @@ fasload(object faslfile) {
   start_address = ROUND_UP(start_address,max_align);
   memory->cfd.cfd_size = memory->cfd.cfd_size - (start_address - the_start);
   memory->cfd.cfd_start = (void *)start_address;	
+  memory->cfd.cfd_name=make_simple_string(FN1);
 
   for (m=start_address,s=b->sections;s;s=s->next) {
 
@@ -308,16 +362,16 @@ fasload(object faslfile) {
       continue;
     }
 
-    if (!(h=bfd_link_hash_lookup(link_info.hash,q[u]->name,MY_BFD_FALSE,MY_BFD_FALSE,MY_BFD_TRUE))) 
+    if (!(h=bfd_link_hash_lookup(link_info.hash,q[u]->name,MY_BFD_FALSE,MY_BFD_FALSE,MY_BFD_TRUE)))
       continue;
 
-    if (h->type!=bfd_link_hash_defined) 
+    if (h->type!=bfd_link_hash_defined)
       FEerror("Undefined symbol ~S",1,make_simple_string(q[u]->name));
-      
+
     if (h->u.def.section) {
       q[u]->value=h->u.def.value+h->u.def.section->vma;
       q[u]->flags|=BSF_WEAK;
-    } else 
+    } else
       FEerror("Symbol without section",0);
 
   }
@@ -330,27 +384,28 @@ fasload(object faslfile) {
 #ifndef HAVE_ALLOCA
 #error Cannot use bfd relocations without alloca at present
 #endif
-  /* We have to do this  to avoid the possibility that 
-     bfd_get_relocated_section_contents will run GBC via its alloc, thereby 
-     write protecting the pages of memory->cfd again and causing bfd reads of 
+  /* We have to do this  to avoid the possibility that
+     bfd_get_relocated_section_contents will run GBC via its alloc, thereby
+     write protecting the pages of memory->cfd again and causing bfd reads of
      the section contents to return an error code after a 'stratified' segfault */
  {
    void *v=alloca(memory->cfd.cfd_size);
-   
+
    if (!v)
      FEerror("Cannot alloca for bfd",0);
 
    for (s=b->sections;s;s=s->next) {
      
-     unsigned long ss=bfd_section_size(b,s);
-
-     if (!(s->flags & SEC_LOAD))
+     if (!(s->flags & SEC_LOAD)) {
+       if (s->flags & SEC_ALLOC)
+	 memset((void *)(unsigned long)s->output_section->vma,0,s->size);
        continue;
+     }
      
      link_order.u.indirect.section=s;
      
      if (!bfd_get_relocated_section_contents(b,&link_info,&link_order,
-					     v,0,q)) 
+					     v,0,q))
        FEerror("Cannot get relocated section contents\n",0);
 
      memcpy((void *)(unsigned long)s->output_section->vma,v,ss);
@@ -358,7 +413,7 @@ fasload(object faslfile) {
    }
 
  }
-   
+
   dum.sm.sm_object1=faslfile;
   dum.sm.sm_fp=b->iostream;
 
@@ -377,14 +432,16 @@ fasload(object faslfile) {
   CLEAR_CACHE;
 #endif
   
+  if(symbol_value(sLAload_verboseA)!=Cnil) {
+    printf(" ;; start address -T %p ",memory->cfd.cfd_start);
+    fflush(stdout);
+  }
+
   call_init(init_address,memory,data,0);
   
   vs_base=old_vs_base;
   vs_top=old_vs_top;
   
-  if(symbol_value(sLAload_verboseA)!=Cnil)
-    printf("start address -T %p ",memory->cfd.cfd_start);
-
   return memory->cfd.cfd_size;
 
  }
