@@ -128,7 +128,14 @@ use_symbols(double d,...) {
 
 #endif
 
-  return (int)(d+d2);
+  { dcomplex c=d+I*d;
+    c=c*c;
+    d=c+I*c;
+    d=c/d;
+    d=d/(c+I*d);
+  }
+  
+  return (int)d;
 
 }
 #endif
