@@ -18,8 +18,6 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 */
 
-#include <string.h>
-#include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -39,18 +37,6 @@ object sSAmacosx_ldcmdA = 0L;
 
 #define sfasl_error(a,b...) {emsg(a,b);do_gcl_abort();}
 
-/* static void get_init_name (object faslfile, char *init_fun) */
-/* { */
-/*     object path = coerce_to_pathname (faslfile); */
-/*     char *p; */
-  
-/*     strcpy (init_fun, "_init_"); */
-/*     coerce_to_filename (path->pn.pn_name, init_fun + 6); */
-  
-/*     for (p = init_fun + 6 ; *p ; p++) */
-/*       if (*p == '-') *p = '_'; */
-/* } */
-
 static NSSymbol
 get_init_sym(NSModule module,object ff) {
 
@@ -63,32 +49,23 @@ get_init_sym(NSModule module,object ff) {
   if (!inf) {
 
     object x;
-    static struct string st;
-    set_type_of(&st,t_string);
-    st.st_self="COMPILER";
-    st.st_dim=st.st_fillp=strlen(st.st_self);
-    if ((x=find_package((object)&st))==Cnil)
+    if ((x=find_package(str("COMPILER")))==Cnil)
       sfasl_error("Cannot find compiler package\n");
-    st.st_self="INIT-NAME";
-    st.st_dim=st.st_fillp=strlen(st.st_self);
-    if ((inf=find_symbol((object)&st,x))==Cnil) {
+    if ((inf=find_symbol(str("INIT-NAME"),x))==Cnil) {
       inf=NULL;
       sfasl_error("Cannot find function COMPILER::INIT-NAME\n");
     }
     
   }
 
-  set_type_of(&st,t_string);
-  st.st_self=ff->st.st_self;
-  st.st_dim=st.st_fillp=ff->st.st_dim;
-  x=ifuncall1(inf,(object)&st);
-  if (x->d.t!=t_string)
+  x=ifuncall1(inf,ff);
+  if (!stringp(x))
     sfasl_error("INIT-NAME error\n");
   assert(snprintf(ib,sizeof(ib),"_init_%-.*s",x->st.st_dim,x->st.st_self)>0);
 
   if (!(v=NSLookupSymbolInModule(module, ib))) {
     x=ifuncall2(inf,(object)&st,Ct);
-    if (x->d.t!=t_string)
+    if (!stringp(x))
       sfasl_error("INIT-NAME error\n");
     assert(snprintf(ib,sizeof(ib),"_init_%-.*s",x->st.st_dim,x->st.st_self)>0);
     if (!(v=NSLookupSymbolInModule(module, ib)))
@@ -231,9 +208,12 @@ int fasload (object faslfile)
     close_stream (faslstream);
     
     memory=new_cfdata();
+    memory->cfd.cfd_name=faslstream->sm.sm_object1;
     
-    if (symbol_value (sLAload_verboseA) != Cnil)	
-        printf (" start address (dynamic) %p ", fptr);
+    if (symbol_value (sLAload_verboseA) != Cnil) {
+        printf (" ;; start address (dynamic) %p ", fptr);
+	fflush(stdout);
+    }
     
     call_init (0,memory,faslstream);
     
