@@ -1,5 +1,4 @@
 /*  Copyright William Schelter. All rights reserved.
-    Copyright 2024 Camm Maguire
     
     Stratified Garbage Collection  (SGC)
     
@@ -673,7 +672,7 @@ sgc_start(void) {
 
   }
 
-  sSAwritableA->s.s_dbind=fSmake_vector(sLbit,(page(heap_end)-first_data_page),Ct,Cnil,Cnil,0,Ct,Cnil);
+  sSAwritableA->s.s_dbind=fSmake_vector(sLbit,(page(rb_start)-first_data_page),Ct,Cnil,Cnil,0,Ct,Cnil);
   wrimap=(void *)sSAwritableA->s.s_dbind->v.v_self;
 
   /* now move the sgc free lists into place.   alt_free should
@@ -912,7 +911,7 @@ sgc_quit(void) {
 
 fixnum debug_fault =0;
 fixnum fault_count =0;
-
+extern char etext;
 static void
 memprotect_handler(int sig, long code, void *scp, char *addr) {
   
