@@ -416,7 +416,6 @@ Filesz      Memsz       Flags       Align
 #include <stdio.h>
 #include <sys/stat.h>
 #include <memory.h>
-#include <string.h>
 #include <errno.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -588,6 +587,8 @@ unexec (new_name, old_name, data_start, bss_start, entry_address)
 #if defined(emacs) || !defined(DEBUG)
   bss_end = (unsigned int) sbrk (0);
   new_bss_addr = (Elf32_Addr) bss_end;
+  /* add for gcl */
+  core_end = (char *) bss_end;
 #else
   new_bss_addr = old_bss_addr + old_bss_size + 0x1234;
 #endif
