@@ -44,6 +44,8 @@ get_string(object x) {
 
   switch(type_of(x)) {
   case t_symbol:
+    return x->s.s_name;
+  case t_simple_string:
   case t_string:
     return x;
   case t_pathname:
@@ -55,7 +57,7 @@ get_string(object x) {
     case smm_probe:
     case smm_io:
       return get_string(x->sm.sm_object1);
-    case smm_synonym:
+    case smm_file_synonym:
       return get_string(x->sm.sm_object0->s.s_dbind);
     }
   }
@@ -69,10 +71,14 @@ coerce_to_filename1(object spec, char *p,unsigned sz) {
 
   object namestring=get_string(spec);
 
-  massert(type_of(namestring)==t_string);
-  massert(namestring->st.st_fillp<sz);
-  memcpy(p,namestring->st.st_self,namestring->st.st_fillp);
-  p[namestring->st.st_fillp]=0;
+  massert(stringp(namestring));
+  massert(VLEN(namestring)<sz);
+  memcpy(p,namestring->st.st_self,VLEN(namestring));
+  p[VLEN(namestring)]=0;
+
+#ifdef FIX_FILENAME
+  FIX_FILENAME(spec,p);
+#endif
 
 }
 
@@ -80,7 +86,7 @@ coerce_to_filename1(object spec, char *p,unsigned sz) {
 static char GETPW_BUF[16384];
 #endif
 
-DEFUN_NEW("UID-TO-NAME",object,fSuid_to_name,SI,1,1,NONE,OI,OO,OO,OO,(fixnum uid),"") {
+DEFUN("UID-TO-NAME",object,fSuid_to_name,SI,1,1,NONE,OI,OO,OO,OO,(fixnum uid),"") {
 #ifndef __MINGW32__
   struct passwd *pwent,pw;
   long r;
@@ -138,12 +144,16 @@ home_namestring1(const char *n,int s,char *o,int so) {
 
 }
 
+#include <sys/types.h>
+#include <dirent.h>
+#include <fcntl.h>
+#include <unistd.h>
 
-DEFUN_NEW("HOME-NAMESTRING",object,fShome_namestring,SI,1,1,NONE,OO,OO,OO,OO,(object nm),"") {
+DEFUN("HOME-NAMESTRING",object,fShome_namestring,SI,1,1,NONE,OO,OO,OO,OO,(object nm),"") {
 
   check_type_string(&nm);
 
-  massert(!home_namestring1(nm->st.st_self,nm->st.st_fillp,FN1,sizeof(FN1)));
+  massert(!home_namestring1(nm->st.st_self,VLEN(nm),FN1,sizeof(FN1)));
   RETURN1(make_simple_string(FN1));
 
 }
@@ -195,7 +205,7 @@ DEF_ORDINARY("FILE",sKfile,KEYWORD,"");
 static int
 stat_internal(object x,struct stat *ssp) {
 
-  if (type_of(x)==t_string) {
+  if (({enum type _t=type_of(x);_t==t_string || _t==t_simple_string;})) {
 
     coerce_to_filename(x,FN1);
 
@@ -219,7 +229,7 @@ stat_mode_key(struct stat *ssp) {
 
 }
 
-DEFUN_NEW("STAT1",object,fSstat1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+DEFUN("STAT1",object,fSstat1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
   struct stat ss;
 
@@ -228,8 +238,10 @@ DEFUN_NEW("STAT1",object,fSstat1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 }
 
 
-DEFUNM_NEW("STAT",object,fSstat,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+DEFUNM("STAT",object,fSstat,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
+  object *vals=(object *)fcall.valp;
+  object *base=vs_top;
   struct stat ss;
 
   if (stat_internal(x,&ss))
@@ -242,24 +254,25 @@ DEFUNM_NEW("STAT",object,fSstat,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
 }
 
-DEFUN_NEW("FTELL",object,fSftell,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+DEFUN("FTELL",object,fSftell,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
 
   RETURN1((x=file_stream(x))!=Cnil&&x->sm.sm_fp ? (object)ftell(x->sm.sm_fp) : (object)0);
 
 }
 
-DEFUN_NEW("FSEEK",object,fSfseek,SI,2,2,NONE,OO,IO,OO,OO,(object x,fixnum pos),"") {
+DEFUN("FSEEK",object,fSfseek,SI,2,2,NONE,OO,IO,OO,OO,(object x,fixnum pos),"") {
 
   RETURN1((x=file_stream(x))!=Cnil&&x->sm.sm_fp&&!fseek(x->sm.sm_fp,pos,SEEK_SET) ? Ct : Cnil);
 
 }
 
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <unistd.h>
 
-DEFUN_NEW("READLINKAT",object,fSreadlinkat,SI,2,2,NONE,OI,OO,OO,OO,(fixnum d,object s),"") {
+DEFUN("READLINKAT",object,fSreadlinkat,SI,2,2,NONE,OI,OO,OO,OO,(fixnum d,object s),"") {
   ssize_t l,z1;
 
   check_type_string(&s);
@@ -278,12 +291,14 @@ DEFUN_NEW("READLINKAT",object,fSreadlinkat,SI,2,2,NONE,OI,OO,OO,OO,(fixnum d,obj
 
 }
 
-DEFUN_NEW("GETCWD",object,fSgetcwd,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
-  massert((getcwd(FN1,sizeof(FN1))));
+DEFUN("GETCWD",object,fSgetcwd,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
+  massert(getcwd(FN1,sizeof(FN1)));
   RETURN1(make_simple_string(FN1));
+
 }
 
-DEFUN_NEW("SETENV",object,fSsetenv,SI,2,2,NONE,OO,OO,OO,OO,(object variable,object value),"Set environment VARIABLE to VALUE")
+DEFUN("SETENV",object,fSsetenv,SI,2,2,NONE,OO,OO,OO,OO,(object variable,object value),"Set environment VARIABLE to VALUE")
 
 {
 
@@ -308,14 +323,14 @@ DEFUN_NEW("SETENV",object,fSsetenv,SI,2,2,NONE,OO,OO,OO,OO,(object variable,obje
 #include <sys/types.h>
 #include <dirent.h>
 
-DEFUN_NEW("OPENDIR",object,fSopendir,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+DEFUN("OPENDIR",object,fSopendir,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
   check_type_string(&x);
   coerce_to_filename(x,FN1);
   return (object)opendir(strlen(FN1) ? FN1 : "./");
 }
 
 
-DEFUN_NEW("D-TYPE-LIST",object,fSd_type_list,SI,0,0,NONE,OI,OO,OO,OO,(void),"") {
+DEFUN("D-TYPE-LIST",object,fSd_type_list,SI,0,0,NONE,OI,OO,OO,OO,(void),"") {
   RETURN1(
 
 #ifdef HAVE_D_TYPE
@@ -347,8 +362,7 @@ DEFUN_NEW("D-TYPE-LIST",object,fSd_type_list,SI,0,0,NONE,OI,OO,OO,OO,(void),"")
 
 
 
-DEFUN_NEW("READDIR",object,fSreaddir,SI,3,3,NONE,OI,IO,OO,OO,(fixnum x,fixnum y,object s),"") {
-
+DEFUN("READDIR",object,fSreaddir,SI,3,3,NONE,OI,IO,OO,OO,(fixnum x,fixnum y,object s),"") {
   struct dirent *e;
   object z;
   long tl;
@@ -391,12 +405,12 @@ DEFUN_NEW("READDIR",object,fSreaddir,SI,3,3,NONE,OI,IO,OO,OO,(fixnum x,fixnum y,
 
 }
 
-DEFUN_NEW("CLOSEDIR",object,fSclosedir,SI,1,1,NONE,OI,OO,OO,OO,(fixnum x),"") {
+DEFUN("CLOSEDIR",object,fSclosedir,SI,1,1,NONE,OI,OO,OO,OO,(fixnum x),"") {
   closedir((DIR *)x);
   return Cnil;
 }
 
-DEFUN_NEW("RENAME",object,fSrename,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
+DEFUN("RENAME",object,fSrename,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"") {
 
   check_type_string(&x);
   check_type_string(&y);
@@ -407,8 +421,8 @@ DEFUN_NEW("RENAME",object,fSrename,SI,2,2,NONE,OO,OO,OO,OO,(object x,object y),"
   RETURN1(rename(FN1,FN2) ? Cnil : Ct);
 
 }
-
-DEFUN_NEW("UNLINK",object,fSunlink,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  
+DEFUN("UNLINK",object,fSunlink,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
   check_type_string(&x);
 
@@ -417,9 +431,8 @@ DEFUN_NEW("UNLINK",object,fSunlink,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
   RETURN1(unlink(FN1) ? Cnil : Ct);
 
 }
-
-
-DEFUN_NEW("CHDIR1",object,fSchdir1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+  
+DEFUN("CHDIR1",object,fSchdir1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
   check_type_string(&x);
 
@@ -429,7 +442,7 @@ DEFUN_NEW("CHDIR1",object,fSchdir1,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
 }
 
-DEFUN_NEW("MKDIR",object,fSmkdir,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+DEFUN("MKDIR",object,fSmkdir,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
   check_type_string(&x);
 
@@ -443,7 +456,7 @@ DEFUN_NEW("MKDIR",object,fSmkdir,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
 
 }
 
-DEFUN_NEW("RMDIR",object,fSrmdir,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
+DEFUN("RMDIR",object,fSrmdir,SI,1,1,NONE,OO,OO,OO,OO,(object x),"") {
   check_type_string(&x);
 
   coerce_to_filename(x,FN1);
