@@ -105,16 +105,16 @@ char *original_file, *save_file;
 */	
 
 	if (stdin != original || original->_file != 0) {
-		fprintf(stderr, "Can't open the original file.\n");
-		exit(1);
+		emsg("Can't open the original file.\n");
+		do_gcl_abort();
 	}
 	setbuf(original, stdin_buf);
 	fclose(stdout);
 	unlink(save_file);
 	n = open(save_file, O_CREAT|O_WRONLY, 0777);
 	if (n != 1 || (save = fdopen(n, "w")) != stdout) {
-		fprintf(stderr, "Can't open the save file.\n");
-		exit(1);
+		emsg("Can't open the save file.\n");
+		do_gcl_abort();
 	}
 	setbuf(save, stdout_buf);
 
@@ -138,33 +138,19 @@ char *original_file, *save_file;
 	fclose(save);
 }
 
-LFD(Lsave)()
-{
-	char filename[256];
+extern void _cleanup();
 
-	check_arg(1);
-	check_type_or_pathname_string_symbol_stream(&vs_base[0]);
-	coerce_to_filename(vs_base[0], filename);
-
-	_cleanup();
-/*
-	{
-		FILE *p;
-		int nfile;
-
-
-		nfile = NUMBER_OPEN_FILES;
-
-		for (p = &_iob[3];  p < &_iob[nfile];  p++)
-			fclose(p);
-	}
-*/
-	memory_save(kcl_self, filename);
-/*
-	_exit(0);
-*/
-	exit(0);
-	/*  no return  */
+LFD(Lsave)() {
+  
+  check_arg(1);
+  check_type_or_pathname_string_symbol_stream(&vs_base[0]);
+  coerce_to_filename(vs_base[0], FN1);
+  
+  _cleanup();
+  
+  memory_save(kcl_self, FN1);
+  exit(0);
+  /*  no return  */
 }
 
 #endif /* UNIXSAVE include */
@@ -172,6 +158,6 @@ LFD(Lsave)()
 void
 gcl_init_unixsave(void)
 {
-	make_function("SAVE", Lsave);
+	make_si_function("SAVE", siLsave);
 }
 
