@@ -62,7 +62,7 @@ gcl_signal(int signo, void (*handler) (/* ??? */))
   
 #ifdef HAVE_SIGACTION
     struct sigaction action;
-    action.sa_handler = handler;
+    action.sa_sigaction = handler;
 /*    action.sa_flags =  SA_RESTART | ((signo == SIGSEGV || signo == SIGBUS) ? SV_ONSTACK : 0) */
    action.sa_flags = SA_RESTART | ((signo == SIGSEGV || signo == SIGBUS) ? SA_ONSTACK : 0)  
 #ifdef SA_SIGINFO
@@ -140,7 +140,7 @@ DEFCONST("+MC-CONTEXT-OFFSETS+",sSPmc_context_offsetsP,SI,FPE_INIT,"");
 
 #define ASM __asm__ __volatile__
 
-DEFUN_NEW("FLD",object,fSfld,SI,1,1,NONE,OI,OO,OO,OO,(fixnum val),"") {
+DEFUN("FLD",object,fSfld,SI,1,1,NONE,OI,OO,OO,OO,(fixnum val),"") {
   volatile double d;
   ASM ("fldt %1;fstpl %0" : "=m" (d): "m" (*(char *)val));
   RETURN1(make_longfloat(d));
@@ -148,19 +148,7 @@ DEFUN_NEW("FLD",object,fSfld,SI,1,1,NONE,OI,OO,OO,OO,(fixnum val),"") {
 
 #endif
 
-/* For now ignore last three args governing offsets and data modification, just to
-   support fpe sync with master*/
-DEFUN_NEW("*FIXNUM",object,fSAfixnum,SI,4,4,NONE,II,OO,OO,OO,(fixnum addr,object x,object y,object z),"") {
-  RETURN1((object)*(fixnum *)addr);
-}
-DEFUN_NEW("*FLOAT",object,fSAfloat,SI,4,4,NONE,OI,OO,OO,OO,(fixnum addr,object x,object y,object z),"") {
-  RETURN1(make_shortfloat(*(float *)addr));
-}
-DEFUN_NEW("*DOUBLE",object,fSAdouble,SI,4,4,NONE,OI,OO,OO,OO,(fixnum addr,object x,object y,object z),"") {
-  RETURN1(make_longfloat(*(double *)addr));
-}
-
-DEFUN_NEW("FEENABLEEXCEPT",object,fSfeenableexcept,SI,1,1,NONE,II,OO,OO,OO,(fixnum x),"") {
+DEFUN("FEENABLEEXCEPT",object,fSfeenableexcept,SI,1,1,NONE,II,OO,OO,OO,(fixnum x),"") {
 
 #ifdef HAVE_FEENABLEEXCEPT
 
@@ -177,14 +165,14 @@ DEFUN_NEW("FEENABLEEXCEPT",object,fSfeenableexcept,SI,1,1,NONE,II,OO,OO,OO,(fixn
     ASM("stmxcsr %0" :: "m" (i));
     i=(i|(FE_ALL_EXCEPT<<7))&(~(x<<7));
     ASM("ldmxcsr %0" : "=m" (i));
-  }    
+  }
 #endif
 
   RETURN1((object)x);
 
 }
 
-DEFUN_NEW("FEDISABLEEXCEPT",object,fSfedisableexcept,SI,0,0,NONE,IO,OO,OO,OO,(void),"") {
+DEFUN("FEDISABLEEXCEPT",object,fSfedisableexcept,SI,0,0,NONE,IO,OO,OO,OO,(void),"") {
 
   fixnum x;
 
@@ -210,26 +198,23 @@ DEFUN_NEW("FEDISABLEEXCEPT",object,fSfedisableexcept,SI,0,0,NONE,IO,OO,OO,OO,(vo
 
 #if defined(__x86_64__) || defined(__i386__)
 
-#define FE_TEST(x87sw_,mxcsr_,excepts_) ((x87sw_)&(excepts_))|(~((mxcsr_)>>7)&excepts_)
+#define FE_TEST(x87sw_,mxcsr_,excepts_) (((x87sw_)&(excepts_))|(((mxcsr_))&(excepts_)))
 
-DEFUN_NEW("FPE_CODE",object,fSfpe_code,SI,2,2,NONE,II,OO,OO,OO,(fixnum x87sw,fixnum mxcsr),"") {
+DEFUN("FPE_CODE",object,fSfpe_code,SI,2,2,NONE,II,IO,OO,OO,(fixnum x87sw,fixnum mxcsr),"") {
+
+  RETURN1((object)FE_TEST(x87sw,mxcsr,FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW|FE_UNDERFLOW|FE_INEXACT));
 
-  RETURN1((object)(long)(FE_TEST(x87sw,mxcsr,FE_INVALID) ? FPE_FLTINV :
-			 (FE_TEST(x87sw,mxcsr,FE_DIVBYZERO) ? FPE_FLTDIV :
-			  (FE_TEST(x87sw,mxcsr,FE_OVERFLOW) ? FPE_FLTOVF :
-			   (FE_TEST(x87sw,mxcsr,FE_UNDERFLOW) ? FPE_FLTUND :
-			    (FE_TEST(x87sw,mxcsr,FE_INEXACT) ? FPE_FLTRES : 0))))));
 }
 
 #if defined(__MINGW32__) || defined(__CYGWIN__)
 
-DEFUN_NEW("FNSTSW",object,fSfnstsw,SI,0,0,NONE,II,OO,OO,OO,(void),"") {
-  volatile unsigned short t=0;
+DEFUN("FNSTSW",object,fSfnstsw,SI,0,0,NONE,II,OO,OO,OO,(void),"") {
+  volatile unsigned short t;
   ASM ("fnstsw %0" :: "m" (t));
   RETURN1((object)(long)t);
 }
-DEFUN_NEW("STMXCSR",object,fSstmxcsr,SI,0,0,NONE,II,OO,OO,OO,(void),"") {
-  volatile unsigned int t=0;
+DEFUN("STMXCSR",object,fSstmxcsr,SI,0,0,NONE,II,OO,OO,OO,(void),"") {
+  volatile unsigned int t;
   ASM ("stmxcsr %0" :: "m" (t));
   RETURN1((object)(long)t);
 }
@@ -239,7 +224,7 @@ DEFUN_NEW("STMXCSR",object,fSstmxcsr,SI,0,0,NONE,II,OO,OO,OO,(void),"") {
 
 
 static void
-sigfpe3(int sig,void *i,void *v) {
+sigfpe3(int sig,siginfo_t *i,void *v) {
 
   unblock_signals(SIGFPE,SIGFPE);
 #ifdef __MINGW32__
@@ -247,19 +232,22 @@ sigfpe3(int sig,void *i,void *v) {
 #endif
   ifuncall3(sSfloating_point_error,FPE_CODE(i,v),FPE_ADDR(i,v),FPE_CTXT(v));
 
+  FPE_SET_CTXT_ADDR(v,FPE_ADDR(i,v));
+  FPE_CLR_CTXT_CWD(v);
+
 }
 
 DEFCONST("+FE-LIST+",sSPfe_listP,SI,list(5,
+					 list(3,sLfloating_point_invalid_operation,make_fixnum(FPE_FLTINV),make_fixnum(FE_INVALID)),
 					 list(3,sLdivision_by_zero,make_fixnum(FPE_FLTDIV),make_fixnum(FE_DIVBYZERO)),
 					 list(3,sLfloating_point_overflow,make_fixnum(FPE_FLTOVF),make_fixnum(FE_OVERFLOW)),
 					 list(3,sLfloating_point_underflow,make_fixnum(FPE_FLTUND),make_fixnum(FE_UNDERFLOW)),
-					 list(3,sLfloating_point_inexact,make_fixnum(FPE_FLTRES),make_fixnum(FE_INEXACT)),
-					 list(3,sLfloating_point_invalid_operation,make_fixnum(FPE_FLTINV),make_fixnum(FE_INVALID))),"");
+					 list(3,sLfloating_point_inexact,make_fixnum(FPE_FLTRES),make_fixnum(FE_INEXACT))),"");
 
 DEF_ORDINARY("FLOATING-POINT-ERROR",sSfloating_point_error,SI,"");
 
 static void
-sigpipe(void)
+sigpipe(int s,siginfo_t *a,void *b)
 {
   unblock_signals(SIGPIPE,SIGPIPE);
   perror("");
@@ -267,14 +255,14 @@ sigpipe(void)
 }
 
 void
-sigint(void)
+sigint(int s,siginfo_t *a,void *b)
 {
   unblock_signals(SIGINT,SIGINT);
   terminal_interrupt(1);
 }
 
 static void
-sigalrm(void)
+sigalrm(int s,siginfo_t *a,void *b)
 {
   unblock_signals(SIGALRM,SIGALRM);
   raise_pending_signals(sig_try_to_delay);
@@ -286,16 +274,16 @@ DEF_ORDINARY("SIGUSR1-INTERRUPT",sSsigusr1_interrupt,SI,"");
 DEF_ORDINARY("SIGIO-INTERRUPT",sSsigio_interrupt,SI,"");
 
 static void
-sigusr1(void)
+sigusr1(int s,siginfo_t *a,void *b)
 {ifuncall1(sSsigusr1_interrupt,Cnil);}
 
 static void
-sigio(void)
+sigio(int s,siginfo_t *a,void *b)
 {ifuncall1(sSsigio_interrupt,Cnil);}
 
 
 static void
-sigterm(void)
+sigterm(int s,siginfo_t *a,void *b)
 {do_gcl_abort();}
 
 
