@@ -30,12 +30,12 @@ License for more details.
 #include <rpc/rpc.h>
 #endif /* __CYGWIN__ */
 
-extern short aet_sizes[];
-static object
-FFN(siGxdr_open)(f)
-     object f;
-{ XDR *xdrs;
-  object ar= alloc_simple_string(sizeof(XDR));
+extern aet_type_struct aet_types[];
+
+DEFUN("XDR-OPEN",object,fSxdr_open,SI,1,1,NONE,OO,OO,OO,OO,(object f),"") {
+
+  XDR *xdrs;
+  object ar= alloc_string(sizeof(XDR));
   array_allocself(ar,1,OBJNULL);
   xdrs= (XDR *) ar->a.a_self;
   if (f->sm.sm_fp == 0) FEerror("stream not ok for xdr io",0);
@@ -47,8 +47,7 @@ FFN(siGxdr_open)(f)
   return ar;
 }
 
-static object
-FFN(siGxdr_write)(object str,object elt) {
+DEFUN("XDR-WRITE",object,fSxdr_write,SI,2,2,NONE,OO,OO,OO,OO,(object str,object elt),"") {
 
   XDR *xdrp= (XDR *) str->ust.ust_self;
   xdrproc_t e;
@@ -66,6 +65,7 @@ FFN(siGxdr_write)(object str,object elt) {
   case t_shortfloat:
     if(xdr_float(xdrp,&sf(elt))) goto error;
     break;
+  case t_simple_vector:
   case t_vector:
     
     switch(elt->v.v_elttype) {
@@ -87,13 +87,13 @@ FFN(siGxdr_write)(object str,object elt) {
       break;
     }
     {
-      u_int tmp=elt->v.v_fillp;
-      if (tmp!=elt->v.v_fillp)
+      u_int tmp=VLEN(elt);
+      if (tmp!=VLEN(elt))
 	goto error;
       if(xdr_array(xdrp,(void *)&elt->v.v_self,
 		    &tmp,
 		    elt->v.v_dim,
-		    aet_sizes[elt->v.v_elttype],
+		    aet_types[elt->v.v_elttype].size,
 		    e))
 	goto error;
     }
@@ -108,8 +108,7 @@ FFN(siGxdr_write)(object str,object elt) {
   return elt;
 }
 
-static object
-FFN(siGxdr_read)(object str,object elt) {
+DEFUN("XDR-READ",object,fSxdr_read,SI,2,2,NONE,OO,OO,OO,OO,(object str,object elt),"") {
 
   XDR *xdrp= (XDR *) str->ust.ust_self;
   xdrproc_t e;
@@ -128,6 +127,7 @@ FFN(siGxdr_read)(object str,object elt) {
     {float x;
     if(xdr_float(xdrp,&x)) goto error;
     return make_shortfloat(x);}
+  case t_simple_vector:
   case t_vector:
     switch(elt->v.v_elttype) {
     case aet_lf:
@@ -149,13 +149,13 @@ FFN(siGxdr_read)(object str,object elt) {
     }
 
     {
-      u_int tmp=elt->v.v_fillp;
-      if (tmp!=elt->v.v_fillp)
+      u_int tmp=VLEN(elt);
+      if (tmp!=VLEN(elt))
 	goto error;
       if(xdr_array(xdrp,(void *)&elt->v.v_self,
 		    &tmp,
 		    elt->v.v_dim,
-		    aet_sizes[elt->v.v_elttype],
+		    aet_types[elt->v.v_elttype].size,
 		    e))
 	goto error;
     }
@@ -172,13 +172,13 @@ FFN(siGxdr_read)(object str,object elt) {
 }
 static void
 gcl_init_xdrfuns()
-{ make_si_sfun("XDR-WRITE",siGxdr_write,
-	       ARGTYPE2(f_object,f_object)|RESTYPE(f_object));
+{/*  make_si_sfun("XDR-WRITE",siGxdr_write, */
+/* 	       ARGTYPE2(f_object,f_object)|RESTYPE(f_object)); */
 
-  make_si_sfun("XDR-READ",siGxdr_read,
-	       ARGTYPE2(f_object,f_object)|RESTYPE(f_object));
-  make_si_sfun("XDR-OPEN",siGxdr_open,
-	       ARGTYPE1(f_object)|RESTYPE(f_object));
+/*   make_si_sfun("XDR-READ",siGxdr_read, */
+/* 	       ARGTYPE2(f_object,f_object)|RESTYPE(f_object)); */
+/*   make_si_sfun("XDR-OPEN",siGxdr_open, */
+/* 	       ARGTYPE1(f_object)|RESTYPE(f_object)); */
   
 }
 #else
