@@ -52,10 +52,27 @@
 
 (in-package :user)
 
-(load "package.lisp")
+(unless (find-package :walker)
+  (make-package :walker :use '(:lisp)))
+(unless (find-package :iterate)
+  (make-package :iterate :use '(:lisp :walker)))
+(unless (find-package :pcl)
+  (make-package :pcl :use '(:lisp :walker :iterate)))
 
 (eval-when (compile load eval)
 
+(if (find-package ':walker)
+    (use-package '(:lisp) ':walker)
+    (make-package ':walker :use '(:lisp)))
+
+(if (find-package ':iterate)
+    (use-package '(:lisp :walker) ':iterate)
+    (make-package ':iterate :use '(:lisp :walker)))
+
+(if (find-package ':pcl)
+    (use-package '(:walker :iterate :lisp) ':pcl)
+    (make-package ':pcl :use '(:walker :iterate :lisp)))
+
 (export (intern (symbol-name :iterate)		;Have to do this here,
 		(find-package :iterate))	;because in the defsystem
 	(find-package :iterate))		;(later in this file)
@@ -70,13 +87,13 @@
 ;;; about order of export and all that stuff, we can't put it in PKG before
 ;;; we want to use it.
 ;;; 
-(defvar *the-pcl-package* (find-package :pcl))
+#-gcl(defvar *the-pcl-package* (find-package :pcl))
 
 (defvar *pcl-system-date* "September 16 92 PCL (g)")
 
 (eval-when (compile load eval)
 (defvar *pcl-proclaim*
-  '(optimize (speed 3) (safety 1) (space 0)
+  '(optimize (speed 3) (safety #+kcl 0 #-kcl 1) (space 0)
              #+lucid (compilation-speed 0)))
 )
 
@@ -247,6 +264,7 @@ and load your system with:
 	    #+Xerox-Medley         (Xerox-Medley xerox)
 	    #+TI                   TI 
 	    #+(and dec vax common) Vaxlisp
+	    #+KCL                  KCL
 	    #+IBCL                 IBCL
 	    #+gcl                  gcl
 	    #+excl                 (excl franz)
@@ -290,6 +308,7 @@ and load your system with:
 	 #+Cloe-Runtime                      ("l"     . "fasl")
 	 #+(and dec common vax (not ultrix)) ("LSP"   . "FAS")
 	 #+(and dec common vax ultrix)       ("lsp"   . "fas")
+	 #+KCL                               ("lsp"   . "o")
 	 #+IBCL                              ("lsp"   . "o")
 	 #+Xerox                             ("lisp"  . "dfasl")
 	 #+(and Lucid MC68000)               ("lisp"  . "lbin")
@@ -451,7 +470,7 @@ and load your system with:
 (defun operation-transformations (name mode &optional arg)
   (let ((system (get-system name)))
     (unless system (error "Can't find system with name ~S." name))
-    (let ((*system-directory* (funcall (the function (car system))))
+    (let ((*system-directory* (funcall (car system))) ;(the function (car system))
 	  (modules (cadr system)))
       (ecase mode
 	(:compile
@@ -520,11 +539,10 @@ and load your system with:
 
 #+cmu17
 (defparameter *byte-files* '(defclass defcombin iterate env))
-
 (defun operate-on-system (name mode &optional arg print-only)
   (let ((system (get-system name)))
     (unless system (error "Can't find system with name ~S." name))
-    (let* ((*system-directory* (funcall (the function (car system))))
+    (let* ((*system-directory* (funcall (car system))) ; (the function (car system))
 	   (transformations (operation-transformations name mode arg)))
       (labels ((load-binary (name pathname)
 		 (format t "~&Loading binary of ~A...~%" name)
@@ -584,9 +602,11 @@ and load your system with:
 				(:source (car *system-directory*))
 				(:binary (cdr *system-directory*)))))))
 	 (dir (pathname-directory directory))
-	 (ldir (if (consp dir)
-		   dir
-		   (pathname-directory (truename directory))))
+	 (ldir nil)
+
+;	 (if (consp dir)
+;		   dir
+;		   (pathname-directory (truename directory))))
 
 	 (port-dname (when (and port 
 				(or *put-impl-binaries-in-impl-directory-p*
@@ -596,10 +616,20 @@ and load your system with:
 			     (append ldir (list "impl" port-dname))
 			     ldir))
 			     
+;	 (port-directory (if port-dname
+;			     (append ldir (list "impl" port-dname))
+;			     ldir))
+			     
+	 (port-directory ldir)
+	 (port-device (if (null port-directory)
+			  nil
+			(pathname-device port-directory)))
+			     
          (pathname
            (make-pathname
              :name (string-downcase (string name))
              :type extension
+	     :device port-device
 	     :directory port-directory
              :defaults directory)))
 
@@ -612,14 +642,14 @@ and load your system with:
 (defun system-source-files (name)
   (let ((system (get-system name)))
     (unless system (error "Can't find system with name ~S." name))
-    (let ((*system-directory* (funcall (the function (car system))))
+    (let ((*system-directory* (funcall (car system))) ;(the function (car system))
 	  (modules (cadr system)))
       (mapcar #'make-source-pathname modules))))
 
 (defun system-binary-files (name)
   (let ((system (get-system name)))
     (unless system (error "Can't find system with name ~S." name))
-    (let ((*system-directory* (funcall (the function (car system))))
+    (let ((*system-directory* (funcall (car system))) ;(the function (car system))
 	  (modules (cadr system)))
       (mapcar #'make-binary-pathname modules))))
 
@@ -643,7 +673,7 @@ and load your system with:
 ;;;
 ;;; ***                                                                   ***
 
-(defun load-truename (&optional (errorp nil))
+#-gcl(defun load-truename (&optional (errorp nil))
   #+cmu (declare (ignore errorp))
   (flet (#+(or Lispm Xerox LUCID)
 	 (bad-time ()
@@ -659,12 +689,13 @@ and load your system with:
     ;; 3.0 it's in the LUCID-COMMON-LISP package.
     ;;
     #+LUCID (or lucid::*source-pathname* (bad-time))
-    #+akcl  *load-pathname*
+    #+gcl *load-pathname*
+    #+(and akcl (not gcl))  si:*load-pathname*
     #+cmu17 *load-truename*
-    #-(or Lispm excl Xerox (and dec vax common) LUCID akcl cmu17) nil))
+    #-(or Lispm excl Xerox (and dec vax common) LUCID akcl gcl cmu17) nil))
 
 #-(or cmu Symbolics)
-(defvar *pcl-directory* (concatenate 'string user::*system-directory* "../pcl/"))
+(defvar *pcl-directory* (concatenate 'string si::*system-directory* "../pcl/"))
 ;	(or (load-truename t)
 ;	    (error "Because load-truename is not implemented in this port~%~
 ;                    of PCL, you must manually edit the definition of the~%~
@@ -715,7 +746,7 @@ and load your system with:
    (sys-proclaim    t            t            ()                kcl)
    (gcl_pcl_walk        (gcl_pcl_pkg)    (gcl_pcl_pkg)    ())
    (gcl_pcl_iterate     t            t            ())
-   (gcl_pcl_macros      t            t            ())
+   (gcl_pcl_macros    t         t        ())
    (gcl_pcl_low         (gcl_pcl_pkg gcl_pcl_macros) t            (gcl_pcl_macros))
    
    
