@@ -88,7 +88,7 @@
 ;;; of caches will require corresponding port-specific modifications to the
 ;;; lap code assembler.
 ;;;
-;; #+gcl(import 'si::non-negative-fixnum)
+#+gcl(import 'si::non-negative-fixnum)
 
 (defmacro cache-vector-ref (cache-vector location)
   `(svref (the simple-vector ,cache-vector)
@@ -418,13 +418,6 @@
 
 )
 
-(defmacro mdotimes ((var form &optional ret) &rest body &aux (v (gensym)))
-  `(do ((,v ,form)
-	(,var 0 (1+ ,var)))
-       ((>= ,var ,v) ,ret)
-     (declare (fixnum ,var ,v))
-     ,@body))
-
 #+structure-wrapper
 (progn
 
@@ -432,7 +425,7 @@
 (defun make-wrapper-cache-number-vector ()
   (let ((cnv (make-array #.wrapper-cache-number-vector-length
 			 :element-type 'fixnum)))
-    (mdotimes (i #.wrapper-cache-number-vector-length)
+    (dotimes (i #.wrapper-cache-number-vector-length)
       (setf (aref cnv i) (get-wrapper-cache-number)))
     cnv))
 
@@ -521,7 +514,7 @@
 	  (values si::*all-t-s-type* si::*standard-slot-positions*)
 	  (values (make-array size :element-type 'unsigned-char)
 		  (let ((array (make-array size :element-type 'unsigned-short)))
-		    (mdotimes (i size)
+		    (dotimes (i size)
 		      (declare (fixnum i))
 		      (setf (aref array i) (* #.(si::size-of t) i))))))
     (make-wrapper-internal :length size
@@ -798,7 +791,7 @@
 	 (old-vector (cache-vector old-cache))
 	 (new-vector (get-cache-vector size)))
     (declare (simple-vector old-vector new-vector))
-    (mdotimes (i size)
+    (dotimes (i size)
       (setf (svref new-vector i) (svref old-vector i)))
     (setf (cache-vector new-cache) new-vector)
     new-cache))
@@ -920,7 +913,7 @@
     (declare (type field-type field)
 	     (type non-negative-fixnum result mask nkeys)
 	     (simple-vector cache-vector))
-    (mdotimes (i nkeys)
+    (dotimes (i nkeys)
       (let* ((wrapper (cache-vector-ref cache-vector (+ i from-location)))
 	     (wcn (wrapper-cache-number-vector-ref wrapper field)))
 	(declare (type non-negative-fixnum wcn))
@@ -1147,7 +1140,7 @@
 
 (defconstant *local-cache-functions*
   '((cache () .cache.)
-    (nkeys () (the non-negative-fixnum (cache-nkeys .cache.)))
+    (nkeys () (cache-nkeys .cache.))
     (line-size () (cache-line-size .cache.))
     (vector () (cache-vector .cache.))
     (valuep () (cache-valuep .cache.))
@@ -1213,7 +1206,7 @@
 	  (let ((list (make-list (nkeys)))
 		(vector (vector)))
 	    (declare (simple-vector vector))
-	    (mdotimes (i (nkeys) list)
+	    (dotimes (i (nkeys) list)
 	      (setf (nth i list) (cache-vector-ref vector (+ location i)))))))
     ;;
     ;; Given a line number, return true IFF the line's
@@ -1230,7 +1223,7 @@
 	(declare (simple-vector cache-vector))
 	(if (= (nkeys) 1)
 	    (eq wrappers (cache-vector-ref cache-vector loc))
-	    (mdotimes (i (nkeys) t)
+	    (dotimes (i (nkeys) t)
 	      (unless (eq (pop wrappers)
 			  (cache-vector-ref cache-vector (+ loc i)))
 		(return nil))))))
@@ -1271,7 +1264,7 @@
       (let ((cache-vector (vector))
 	    (wrappers-mismatch-p (null wrappers)))
 	(declare (simple-vector cache-vector))
-	(mdotimes (i (nkeys) wrappers-mismatch-p)
+	(dotimes (i (nkeys) wrappers-mismatch-p)
 	  (let ((wrapper (cache-vector-ref cache-vector (+ loc i))))
 	    (when (or (null wrapper)
 		      (invalid-wrapper-p wrapper))
@@ -1382,7 +1375,7 @@
   (with-local-cache-functions (cache)
     (let ((location (if (= (nkeys) 1) 0 1))
 	  (limit (funcall (limit-fn) (nlines))))
-      (mdotimes (i (nlines) cache)
+      (dotimes (i (nlines) cache)
 	(when (and (not (location-reserved-p location))
 		   (line-full-p i))
 	  (let* ((home-loc (compute-primary-cache-location-from-location 
@@ -1406,7 +1399,7 @@
       (declare (type non-negative-fixnum location limit))
       (when (location-reserved-p location)
 	(setq location (next-location location)))
-      (mdotimes (i (the non-negative-fixnum (1+ limit)))
+      (dotimes (i (the non-negative-fixnum (1+ limit)))
 	(when (location-matches-wrappers-p location wrappers)
 	  (return-from probe-cache (or (not (valuep))
 				       (location-value location))))
@@ -1420,7 +1413,7 @@
 (defun map-cache (function cache &optional set-p)
   (with-local-cache-functions (cache)
     (let ((set-p (and set-p (valuep))))
-      (mdotimes (i (nlines) cache)
+      (dotimes (i (nlines) cache)
 	(unless (or (line-reserved-p i) (not (line-valid-p i nil)))
 	  (let ((value (funcall function (line-wrappers i) (line-value i))))
 	    (when set-p
@@ -1436,7 +1429,7 @@
   (with-local-cache-functions (cache)
     (let ((count 0))
       (declare (type non-negative-fixnum count))
-      (mdotimes (i (nlines) count)
+      (dotimes (i (nlines) count)
 	(unless (line-reserved-p i)
 	  (when (line-full-p i)
 	    (incf count)))))))
@@ -1444,7 +1437,7 @@
 (defun entry-in-cache-p (cache wrappers value)
   (declare (ignore value))
   (with-local-cache-functions (cache)
-    (mdotimes (i (nlines))
+    (dotimes (i (nlines))
       (unless (line-reserved-p i)
 	(when (equal (line-wrappers i) wrappers)
 	  (return t))))))
@@ -1505,12 +1498,12 @@
 		(to-line free))
 	    (declare (type non-negative-fixnum to-line))
 	    (if (line-reserved-p to-line)
-		(error "transferring something into a reserved cache line.")
+		(error "transfering something into a reserved cache line.")
 		(let ((from-loc (line-location from-line))
 		      (to-loc (line-location to-line)))
 		  (declare (type non-negative-fixnum from-loc to-loc))
 		  (modify-cache to-cache-vector
-				(mdotimes (i (line-size))
+				(dotimes (i (line-size))
 				  (setf (cache-vector-ref to-cache-vector
 							  (+ to-loc i))
 					(cache-vector-ref from-cache-vector
@@ -1541,7 +1534,7 @@
 		     (fill-cache-from-cache-p nil ncache cache line))
 		   (try-one-fill (wrappers value)
 		     (fill-cache-p nil ncache wrappers value)))
-	    (if (and (mdotimes (i (nlines) t)
+	    (if (and (dotimes (i (nlines) t)
 		       (when (and (null (line-reserved-p i))
 				  (line-valid-p i wrappers))
 			 (unless (try-one-fill-from-line i) (return nil))))
@@ -1570,7 +1563,7 @@
 				  (fill-cache-p t ncache wrappers value))))
 	       (try-one-fill (wrappers value)
 		 (fill-cache-p nil ncache wrappers value)))
-	(mdotimes (i (nlines))
+	(dotimes (i (nlines))
 	  (when (and (null (line-reserved-p i))
 		     (line-valid-p i wrappers))
 	    (do-one-fill-from-line i)))
@@ -1645,7 +1638,7 @@
 	   (declare (type non-negative-fixnum from-loc to-loc)
 		    (simple-vector cache-vector))
 	   (modify-cache cache-vector
-			 (mdotimes (i (line-size))
+			 (dotimes (i (line-size))
 			   (setf (cache-vector-ref cache-vector (+ to-loc i))
 				 (cache-vector-ref cache-vector (+ from-loc i)))
 			   (setf (cache-vector-ref cache-vector (+ from-loc i))
