@@ -39,9 +39,6 @@
 						;EARLY
 						;BRAID
 						;COMPLETE
-(defvar *fegf-started-p* nil)
-
-
 )
 
 (eval-when (load eval)
@@ -74,7 +71,7 @@
 (defmacro parse-gspec (spec
 		       (non-setf-var . non-setf-case)
 		       (setf-var . setf-case))
-  (declare (indentation 1 1))
+;  (declare (indentation 1 1))
   #+setf (declare (ignore setf-var setf-case))
   (once-only (spec)
     `(cond (#-setf (symbolp ,spec) #+setf t
@@ -105,14 +102,14 @@
 	      (symbol-function symbol))
   #+xerox (il:virginfn symbol)
   #+setf (fdefinition symbol)
-  #+kcl (symbol-function
-	  (let ((sym (get symbol 'si::traced)) first-form)
-	    (if (and sym
-		     (consp (symbol-function symbol))
-		     (consp (setq first-form (nth 3 (symbol-function symbol))))
-		     (eq (car first-form) 'si::trace-call))
-		sym
-		symbol)))
+;  #+kcl (symbol-function
+;	  (let ((sym (when (symbolp symbol) (get symbol 'si::traced))) first-form)
+;	    (if (and sym
+;		     (consp (symbol-function symbol))
+;		     (consp (setq first-form (nth 3 (symbol-function symbol))))
+;		     (eq (car first-form) 'si::trace-call))
+;		sym
+;		symbol)))
   #-(or Lispm Lucid excl Xerox setf kcl) (symbol-function symbol))
 
 ;;;
@@ -134,17 +131,10 @@
             (setf (symbol-function name) new-definition)
             (when brokenp (xcl:rebreak-function name))
             (when advisedp (xcl:readvise-function name)))
-  #+(and setf (not cmu)) (setf (fdefinition name) new-definition)
-  #+kcl (setf (symbol-function 
-	       (let ((sym (get name 'si::traced)) first-form)
-		 (if (and sym
-			  (consp (symbol-function name))
-			  (consp (setq first-form
-				       (nth 3 (symbol-function name))))
-			  (eq (car first-form) 'si::trace-call))
-		     sym
-		     name)))
-	      new-definition)
+  ;; FIXME add setf expander for fdefinition -- right now we go through
+  ;; the following code which expands to a call to si::fset
+  #+(and setf (not cmu) (not kcl)) (setf (fdefinition name) new-definition)
+  #+kcl (setf (symbol-function (or (si::traced-sym name) name)) new-definition)
   #+cmu (progn
 	  (c::%%defun name new-definition nil)
 	  (c::note-name-defined name :function)
@@ -174,7 +164,9 @@
     (name (fdefine-carefully (get-setf-function-name name) new-value))))
 
 
-(proclaim '(special *the-class-t* 
+(eval-when
+ (compile load eval)
+ (proclaim '(special *the-class-t*
 	            *the-class-vector* *the-class-symbol*
                     *the-class-string* *the-class-sequence*
                     *the-class-rational* *the-class-ratio*
@@ -202,7 +194,7 @@
                     *the-class-standard-effective-slot-definition*
 
                     *the-eslotd-standard-class-slots*
-                    *the-eslotd-funcallable-standard-class-slots*))
+                    *the-eslotd-funcallable-standard-class-slots*)))
 
 (proclaim '(special *the-wrapper-of-t*
                     *the-wrapper-of-vector* *the-wrapper-of-symbol*
@@ -310,7 +302,7 @@
   `(satisfies ,(make-class-eq-predicate class)))
 ||#
 
-#-(or excl cmu17)
+#-(or excl cmu17 gcl)
 (deftype eql (type-object)
   `(member ,type-object))
 
@@ -371,7 +363,7 @@
 	   type))))
 
 ;;; not used...
-#+nil
+;#+nil
 (defun *typep (object type)
   (setq type (*normalize-type type))
   (cond ((member (car type) '(eql wrapper-eq class-eq class))
@@ -413,29 +405,36 @@
 			 (convert-to-system-type type2))))))))
 
 (defun do-satisfies-deftype (name predicate)
-  #+cmu17 (declare (ignore name predicate))
-  #+(or :Genera (and :Lucid (not :Prime)) ExCL :coral)
-  (let* ((specifier `(satisfies ,predicate))
-	 (expand-fn #'(lambda (&rest ignore)
-			(declare (ignore ignore))
-			specifier)))
-    ;; Specific ports can insert their own way of doing this.  Many
-    ;; ports may find the expand-fn defined above useful.
-    ;;
-    (or #+:Genera
-	(setf (get name 'deftype) expand-fn)
-	#+(and :Lucid (not :Prime))
-	(system::define-macro `(deftype ,name) expand-fn nil)
-	#+ExCL
-	(setf (get name 'excl::deftype-expander) expand-fn)
-	#+:coral
-	(setf (get name 'ccl::deftype-expander) expand-fn)))
-  #-(or :Genera (and :Lucid (not :Prime)) ExCL :coral cmu17)
-  ;; This is the default for ports for which we don't know any
-  ;; better.  Note that for most ports, providing this definition
-  ;; should just speed up class definition.  It shouldn't have an
-  ;; effect on performance of most user code.
-  (eval `(deftype ,name () '(satisfies ,predicate))))
+  (declare (ignorable predicate))
+  (unless (get name 'si::deftype-definition)
+;    (print `(deftype ,name nil `(si::std-instance ,(si::coerce-to-standard-class ',name))))
+;    (print (si::coerce-to-standard-class name))
+    (eval `(deftype ,name nil t))
+    (remprop name 'si::simple-typep-fn)))
+
+  ;; #+cmu17 (declare (ignore name predicate))
+  ;; #+(or :Genera (and :Lucid (not :Prime)) ExCL :coral)
+  ;; (let* ((specifier `(satisfies ,predicate))
+  ;; 	 (expand-fn #'(lambda (&rest ignore)
+  ;; 			(declare (ignore ignore))
+  ;; 			specifier)))
+  ;;   ;; Specific ports can insert their own way of doing this.  Many
+  ;;   ;; ports may find the expand-fn defined above useful.
+  ;;   ;;
+  ;;   (or #+:Genera
+  ;; 	(setf (get name 'deftype) expand-fn)
+  ;; 	#+(and :Lucid (not :Prime))
+  ;; 	(system::define-macro `(deftype ,name) expand-fn nil)
+  ;; 	#+ExCL
+  ;; 	(setf (get name 'excl::deftype-expander) expand-fn)
+  ;; 	#+:coral
+  ;; 	(setf (get name 'ccl::deftype-expander) expand-fn)))
+  ;; #-(or :Genera (and :Lucid (not :Prime)) ExCL :coral cmu17)
+  ;; ;; This is the default for ports for which we don't know any
+  ;; ;; better.  Note that for most ports, providing this definition
+  ;; ;; should just speed up class definition.  It shouldn't have an
+  ;; ;; effect on performance of most user code.
+  ;; (unless (get name 'si::deftype-definition) (eval `(deftype ,name () '(satisfies ,predicate)))))
 
 (defun make-type-predicate-name (name &optional kind)
   (if (symbol-package name)
@@ -515,7 +514,7 @@
 
 (defun make-class-predicate-name (name)
   (intern (format nil "~A::~A class predicate"
-		  (package-name (symbol-package name))
+		  (package-name (or (symbol-package name) *package*))
 		  name)
 	  *the-pcl-package*))
 
@@ -535,38 +534,68 @@
   ;;
   ;; name       supers     subs                     cdr of cpl
   ;; prototype
-  '(;(t         ()         (number sequence array character symbol) ())
-    (number     (t)        (complex real)           (t))
-    (real       (number)   (float rational)         (number t))
-    (complex    (number)   ()                       (number t) #c(1 1))
-    (float      (real)     ()                       (real number t)  1.0)
+  `(;(t         ()         (number sequence array character symbol) ())
+    (number     (t)        (complex float rational) (t))
+    (complex    (number)   ()                       (number t)                   #c(1 1))
+    (float      (real)     ()                       (real number t)              1.0)
+    (real       (number)   (rational float)         (number t))
     (rational   (real)     (integer ratio)          (real number t))
-    (integer    (rational) ()                       (rational real number t) 1)
-    (ratio      (rational) ()                       (rational real number t) 1/2)
+    (integer    (rational) ()                       (rational real number t)     1)
+    (ratio      (rational) ()                       (rational real number t)     1/2)
 
     (sequence   (t)        (list vector)            (t))
     (list       (sequence) (cons null)              (sequence t))
-    (cons       (list)     ()                       (list sequence t)
-     (nil))
+    (cons       (list)     ()                       (list sequence t)            (nil))
+
+    (pathname   (t)        (logical-pathname)       (t)                          #p"foo")
+    (logical-pathname   
+                (pathname t)        ()              (pathname t)                 )
+    (readtable  (t)        ()                       (t)                          ,*readtable*)
+    (package    (t)        ()                       (t)                          ,*package*)
+    (hash-table (t)        ()                       (t)                          )
+    (function   (t)        ()                       (t)                          ,#'cons)
+;    (function   (t)        (interpreted-function
+;			    compiled-function)      (t)                          )
+;    (interpreted-function   
+;                (function t)
+;		           ()                       (function t)                 ,(eval `(function (lambda nil nil))))
+;    (compiled-function   
+;                (function t)
+;		           ()                       (function t)                 ,#'cons)
+    (synonym-stream  
+                (stream t) ()                       (stream t)                   ,*standard-output*)
+    (echo-stream  
+                (stream t) ()                       (stream t)                   )
+    (two-way-stream  
+                (stream t) ()                       (stream t)                   )
+    (string-stream  
+                (stream t) ()                       (stream t)                   )
+    (concatenated-stream  
+                (stream t) ()                       (stream t)                   )
+    (broadcast-stream  
+                (stream t) ()                       (stream t)                   )
+    (file-stream  
+                (stream t) ()                       (stream t)                   )
+    (stream     (t)        (synonym-stream 
+			    string-stream
+			    two-way-stream
+			    echo-stream
+			    file-stream
+			    concatenated-stream
+			    broadcast-stream)       (t))
     
 
-    (array      (t)        (vector)                 (t)
-     #2A((NIL)))
+    (array      (t)        (vector)                 (t)                          #2A((NIL)))
     (vector     (array
-		 sequence) (string bit-vector)      (array sequence t)
-     #())
-    (string     (vector)   ()                       (vector array sequence t)
-     "")
-    (bit-vector (vector)   ()                       (vector array sequence t)
-     #*1)
-    (character  (t)        ()                       (t)
-     #\c)
+		 sequence) (string bit-vector)      (array sequence t)           #())
+    (string     (vector)   ()                       (vector array sequence t)    "")
+    (bit-vector (vector)   ()                       (vector array sequence t)    #*1)
+    (character  (t)        ()                       (t)                          #\c)
    
-    (symbol     (t)        (null)                   (t)
-     symbol)
+    (symbol     (t)        (null)                   (t)                          symbol)
+    (random-state (t)      (null)                   (t)                          #$0)
     (null       (symbol 
-		 list)     ()                       (symbol list sequence t)
-     nil)))
+		 list)     ()                       (symbol list sequence t)     nil)))
 
 #+cmu17
 (labels ((direct-supers (class)
@@ -620,6 +649,8 @@
   (defclass kernel:funcallable-instance (function) ()
     (:metaclass built-in-class)))
 
+(push (make-early-class-definition 'function nil 'built-in-class '(t) nil nil) *early-class-definitions*)
+
 (defclass slot-object (#-cmu17 t #+cmu17 kernel:instance) ()
   (:metaclass slot-class))
 
@@ -629,12 +660,15 @@
 (defstruct (#-cmu17 structure-object #+cmu17 dead-beef-structure-object
 	     (:constructor |STRUCTURE-OBJECT class constructor|)))
 
-
 (defclass standard-object (slot-object) ())
 
 (defclass metaobject (standard-object) ())
 
-(defclass specializer (metaobject) 
+(defclass funcallable-standard-object (standard-object function)
+  ()
+  (:metaclass funcallable-standard-class))
+
+(defclass specializer (metaobject)
      ((type
         :initform nil
         :reader specializer-type)))
@@ -662,7 +696,7 @@
 ;;; have the class CLASS in its class precedence list.
 ;;; 
 (defclass class (documentation-mixin dependent-update-mixin definition-source-mixin
-		 specializer)
+		 specializer )
      ((name
 	:initform nil
 	:initarg  :name
@@ -767,7 +801,7 @@
 ;;;
 ;;; Slot definitions.
 ;;;
-(defclass slot-definition (metaobject) 
+(defclass slot-definition (metaobject )
      ((name
 	:initform nil
 	:initarg :name
@@ -855,7 +889,8 @@
 					       effective-slot-definition)
   ())
 
-(defclass method (metaobject) ())
+(defclass method (metaobject )
+  ())
 
 (defclass standard-method (definition-source-mixin plist-mixin method)
      ((generic-function
@@ -904,8 +939,8 @@
 			    definition-source-mixin
 			    documentation-mixin
 			    metaobject
-			    #+cmu17 kernel:funcallable-instance)
-     ()
+			    funcallable-standard-object)
+     ((initial-methods :initform () :accessor generic-function-initial-methods))
   (:metaclass funcallable-standard-class))
     
 (defclass standard-generic-function (generic-function)
@@ -931,12 +966,17 @@
       (pretty-arglist
 	:initform ()
 	:accessor gf-pretty-arglist)
+      (declarations
+       :initform ()
+       :initarg :declare
+       :reader generic-function-declarations)
       )
   (:metaclass funcallable-standard-class)
   (:default-initargs :method-class *the-class-standard-method*
 		     :method-combination *standard-method-combination*))
 
-(defclass method-combination (metaobject) ())
+(defclass method-combination (metaobject)
+  ())
 
 (defclass standard-method-combination
 	  (definition-source-mixin method-combination)
@@ -947,6 +987,14 @@
       (options       :reader method-combination-options
 	             :initarg :options)))
 
+(defclass long-method-combination (standard-method-combination)
+  ((function
+    :initarg :function
+    :reader long-method-combination-function)
+   (arguments-lambda-list
+    :initarg :arguments-lambda-list
+    :reader long-method-combination-arguments-lambda-list)))
+
 (defparameter *early-class-predicates*
   '((specializer specializerp)
     (exact-class-specializer exact-class-specializer-p)
@@ -966,5 +1014,29 @@
     (standard-boundp-method standard-boundp-method-p)
     (generic-function generic-function-p)
     (standard-generic-function standard-generic-function-p)
-    (method-combination method-combination-p)))
+    (method-combination method-combination-p)
+    (long-method-combination long-method-combination-p)))
 
+
+(defun early-find-class-symbol (x &optional errorp environment)
+  (declare (ignore errorp environment))
+  (when (or (member x *early-class-definitions* :key 'cadr)
+	    (gethash x *find-class*))
+    x))
+
+(defun mk-early-cpl (sym)
+  (let ((l (nth 4 (car (member sym *early-class-definitions* :key 'cadr)))))
+    (append l (reduce (lambda (&rest r) (when r (apply 'union r))) (mapcar 'mk-early-cpl l)))))
+
+(defun early-class-precedence-list-symbol (x &aux tem)
+  (cond ((mk-early-cpl x))
+	((setq tem (gethash x *find-class*))
+	 (early-class-precedence-list (car tem)))))
+
+(setf (get 'si::si-find-class 'si::early) 'early-find-class-symbol)
+(setf (get 'si::si-class-precedence-list 'si::early) 'early-class-precedence-list-symbol)
+(setf (get 'si::si-class-of 'si::early) 'early-class-name-of)
+;(setf (symbol-function 'si::find-class) (symbol-function 'early-find-class-symbol))
+;(setf (symbol-function 'si::class-precedence-list) (symbol-function 'early-class-precedence-list-symbol))
+;(setf (symbol-function 'si::class-of) (symbol-function 'early-class-name-of))
+;(setf (symbol-function 'si::class-direct-subclasses) (symbol-function 'early-class-direct-subclasses)) ;FIXME need class-name here
