@@ -61,7 +61,8 @@
 ;;;
 (eval-when (compile load eval)
 
-(defmacro memq (item list) `(member ,item ,list :test #'eq))
+(import '(si::memq) 'pcl)
+;(defmacro memq (item list) `(member ,item ,list :test #'eq))
 (defmacro assq (item list) `(assoc ,item ,list :test #'eq))
 (defmacro rassq (item list) `(rassoc ,item ,list :test #'eq))
 (defmacro delq (item list) `(delete ,item ,list :test #'eq))
@@ -80,8 +81,8 @@
 	(t (make-cdxr (- n 4) `(cddddr ,form)))))
 )
 
-(deftype non-negative-fixnum ()
-  '(and fixnum (integer 0 *)))
+;(deftype non-negative-fixnum ()
+;  '(and fixnum (integer 0 *)))
 
 (defun true (&rest ignore) (declare (ignore ignore)) t)
 (defun false (&rest ignore) (declare (ignore ignore)) nil)
@@ -127,34 +128,9 @@
 	  ,wrapped-body))))
 
 (eval-when (compile load eval)
-(defun extract-declarations (body &optional environment)
-  ;;(declare (values documentation declarations body))
-  (let (documentation declarations form)
-    (when (and (stringp (car body))
-	       (cdr body))
-      (setq documentation (pop body)))
-    (block outer
-      (loop
-	(when (null body) (return-from outer nil))
-	(setq form (car body))
-	(when (block inner
-		(loop (cond ((not (listp form))
-			     (return-from outer nil))
-			    ((eq (car form) 'declare)
-			     (return-from inner 't))
-			    (t
-			     (multiple-value-bind (newform macrop)
-				  (macroexpand-1 form environment)
-			       (if (or (not (eq newform form)) macrop)
-				   (setq form newform)
-				 (return-from outer nil)))))))
-	  (pop body)
-	  (dolist (declaration (cdr form))
-	    (push declaration declarations)))))
-    (values documentation
-	    (and declarations `((declare ,.(nreverse declarations))))
-	    body)))
-)
+  (defun extract-declarations (body &optional environment)
+    (multiple-value-bind (doc decls ctps body) (si::parse-body-header body)
+      (values doc decls (nconc ctps body)))))
 
 (defun get-declaration (name declarations &optional default)
   (dolist (d declarations default)
@@ -208,19 +184,20 @@
 ;;; Common Lisp BUG:
 ;;;    Common Lisp should have destructuring-bind.
 ;;;    
-;#-gcl
 ; FIXME use regular destructuring-bind
-(defmacro pcl-destructuring-bind (pattern form &body body)
-  (multiple-value-bind (ignore declares body)
-      (extract-declarations body)
-    (declare (ignore ignore))
-    (multiple-value-bind (setqs binds)
-	(destructure pattern form)
-      `(let ,binds
-	 ,@declares
-	 ,@setqs
-	 (progn .destructure-form.)
-	 . ,body))))
+;; #+gcl(setf (macro-function 'pcl-destructuring-bind) (macro-function 'destructuring-bind))
+;; #-gcl
+;; (defmacro pcl-destructuring-bind (pattern form &body body)
+;;   (multiple-value-bind (ignore declares body)
+;;       (extract-declarations body)
+;;     (declare (ignore ignore))
+;;     (multiple-value-bind (setqs binds)
+;; 	(destructure pattern form)
+;;       `(let ,binds
+;; 	 ,@declares
+;; 	 ,@setqs
+;; 	 (progn .destructure-form.)
+;; 	 . ,body))))
 
 (eval-when (compile load eval)
 (defun destructure (pattern form)
@@ -315,7 +292,11 @@
        (loop (when (null .plist-tail.) (return nil))
 	     (setq ,key (pop .plist-tail.))
 	     (when (null .plist-tail.)
-	       (error "Malformed plist in doplist, odd number of elements."))
+	       (error 'program-error :format-control
+			       "Malformed plist in doplist, odd number of elements."))
+	     (when (not (symbolp ,key))
+	       (error 'program-error :format-control 
+			       "Supplied key is not a symbol."))
 	     (setq ,val (pop .plist-tail.))
 	     (progn ,@bod)))))
 
@@ -352,7 +333,7 @@
        (printing-random-thing-internal ,thing ,stream)
        (format ,stream ">"))))
 
-(defun printing-random-thing-internal (thing stream)
+#-gcl(defun printing-random-thing-internal (thing stream)
   (declare (ignore thing stream))
   nil)
 
@@ -439,9 +420,18 @@
 
 (defvar *create-classes-from-internal-structure-definitions-p* t)
 
+#+gcl(defvar *structure-table* (make-hash-table :test 'eq))
+#+gcl(defun structure-type-p (type)
+       (or (not (null (gethash type *structure-table*)))
+	   (let ((s-data nil))
+	     (and (symbolp type)
+		  (setq s-data (get type 'si::s-data))
+		  (null (si::s-data-type s-data))))))
+
 (defun find-class-from-cell (symbol cell &optional (errorp t))
   (or (find-class-cell-class cell)
       (and *create-classes-from-internal-structure-definitions-p*
+	   (fboundp 'find-structure-class)
            (structure-type-p symbol)
            (find-structure-class symbol))
       (cond ((null errorp) nil)
@@ -456,8 +446,9 @@
   (find-class-cell-predicate cell))
 
 (defun legal-class-name-p (x)
-  (and (symbolp x)
-       (not (keywordp x))))
+  (symbolp x))
+;  (and (symbolp x)
+;       (not (keywordp x))))
 
 (defun find-class (symbol &optional (errorp t) environment)
   (declare (ignore environment))
@@ -473,9 +464,8 @@
 
 ; Use this definition in any CL implementation supporting 
 ; both define-compiler-macro and load-time-value.
-#+cmu ; Note that in CMU, lisp:find-class /= pcl:find-class
-(define-compiler-macro find-class (&whole form
-				   symbol &optional (errorp t) environment)
+#+(or gcl cmu) ; Note that in CMU, lisp:find-class /= pcl:find-class
+(define-compiler-macro find-class (&whole form symbol &optional (errorp t) environment)
   (declare (ignore environment))
   (if (and (constantp symbol) 
 	   (legal-class-name-p (eval symbol))
@@ -501,7 +491,8 @@
   (declare (ignore errorp environment))
   `(SETF\ PCL\ FIND-CLASS ,new-value ,symbol))
 
-(defun #-setf SETF\ PCL\ FIND-CLASS #+setf (setf find-class) (new-value symbol)
+(defun #-setf SETF\ PCL\ FIND-CLASS #+setf (setf find-class) (new-value symbol  &optional errorp environment)
+  (declare (ignore errorp environment))
   (if (legal-class-name-p symbol)
       (let ((cell (find-class-cell symbol)))
 	(setf (find-class-cell-class cell) new-value)
@@ -520,7 +511,8 @@
 	    (dolist (keys+aok (find-class-cell-make-instance-function-keys cell))
 	      (update-initialize-info-internal
 	       (initialize-info new-value (car keys+aok) nil (cdr keys+aok))
-	       'make-instance-function)))))
+	       'make-instance-function))))
+	new-value)
       (error "~S is not a legal class name." symbol)))
 
 #-setf
@@ -657,6 +649,7 @@
 (defun do-standard-defsetfs-for-defclass (accessors)
   (dolist (name accessors) (do-standard-defsetf-1 name)))
 
+;; FIXME remove this when all is well
 (defun do-standard-defsetf-1 (function-name)
   #+setf
   (declare (ignore function-name))
@@ -697,7 +690,8 @@
 			 (bindings (mapcar #'(lambda (x) `(,(gensym) ,x)) loc-args))
 			 (vars (mapcar #'car bindings)))
 		    `(let ,bindings
-		       (,setf-function-name ,(car (last form)) ,@vars)))))
+		       (funcall #',setf-function-name ,(car (last form)) ,@vars)))))
+	(format t "defsetfinf ~S~%" `(defsetf ,function-name ,helper))
 	(eval `(defsetf ,function-name ,helper)))
       #+Xerox
       (flet ((setf-expander (body env)
