@@ -403,7 +403,8 @@
 				     :lambda-list lambda-list
 				     other-initargs)))
 ;   (when existing (remove-method generic-function existing))
-    (add-method generic-function new)))
+    (add-method generic-function new)
+    new))
 
 	
 (defun make-specializable (function-name &key (arglist nil arglistp))
@@ -452,18 +453,30 @@
 
 (defun real-get-method (generic-function qualifiers specializers
 					 &optional (errorp t))
+  (when (generic-function-methods generic-function)
+    (unless (eql (length specializers)
+		 (length (arg-info-metatypes (gf-arg-info generic-function))))
+      (error 'program-error
+	     :format-control "Specializer list ~S does not match generic function ~S"
+	     :format-arguments (list specializers generic-function))))
+
   (let ((hit
-	  (dolist (method (generic-function-methods generic-function))
-	    (when (and (equal qualifiers (method-qualifiers method))
-		       (every #'same-specializer-p specializers
-			      (method-specializers method)))
-	      (return method)))))
+	 (dolist (method (generic-function-methods generic-function))
+	   (when (and (equal qualifiers (method-qualifiers method))
+		      (every #'same-specializer-p specializers
+			     (method-specializers method)))
+	     (return method)))))
     (cond (hit hit)
 	  ((null errorp) nil)
 	  (t
 	   (error "No method on ~S with qualifiers ~:S and specializers ~:S."
 		  generic-function qualifiers specializers)))))
 
+(defmethod find-method ((generic-function standard-generic-function)
+			qualifiers specializers &optional (errorp t))
+  (real-get-method generic-function qualifiers
+		   (parse-specializers specializers) errorp))
+  
 
 ;;;
 ;;; Compute various information about a generic-function's arglist by looking
@@ -620,14 +633,14 @@
 			  allocate-instance shared-initialize initialize-instance))
 	    (update-make-instance-function-table (type-class (car specializers))))
 	  (update-dfun generic-function))
-	method)))
+	generic-function)))
   
 (defun real-remove-method (generic-function method)
-  (if  (neq generic-function (method-generic-function method))
-       (error "The method ~S is attached to the generic function~@
-               ~S.  It can't be removed from the generic function~@
-               to which it is not attached."
-	      method (method-generic-function method))
+  (if (neq generic-function (method-generic-function method)) generic-function
+;;        (error "The method ~S is attached to the generic function~@
+;;                ~S.  It can't be removed from the generic function~@
+;;                to which it is not attached."
+;; 	      method (method-generic-function method))
        (let* ((name         (generic-function-name generic-function))
 	      (specializers (method-specializers method))
 	      (methods      (generic-function-methods generic-function))
@@ -693,8 +706,7 @@
     standard-object structure-object)   ;                         or symbol
    (number array character symbol	; direct subclasses of t, but not sequence
     standard-object structure-object)
-   (complex real)		        ; direct subclasses of number
-   (float rational)		        ; direct subclasses of real
+   (complex float rational)		; direct subclasses of number
    (integer ratio)			; direct subclasses of rational
    (list vector)			; direct subclasses of sequence
    (cons null)				; direct subclasses of list
@@ -740,9 +752,9 @@
       (dotimes (i nreq)
 	i
 	(unless arguments
-	  (error "The function ~S requires at least ~D arguments"
-		 (generic-function-name generic-function)
-		 nreq))
+	  (error 'program-error
+		 :format-control "The function ~S requires at least ~D arguments"
+		 :format-arguments (list (generic-function-name generic-function) nreq)))
 	(let ((arg (pop arguments)))
 	  (push (if type-modifier `(,type-modifier ,arg) arg) types-rev)))
       (values (nreverse types-rev) arg-info))))
@@ -1183,28 +1195,23 @@
 
 #+lcl3.0 (dont-use-production-compiler)
 
-(defun generate-discrimination-net-internal 
-    (gf methods types methods-function test-function type-function)
-  #+cmu
-  (declare (type function methods-function test-function type-function))
+(defun generate-discrimination-net-internal (gf methods types methods-function test-function type-function)
+  (declare (function methods-function test-function type-function))
   (let* ((arg-info (gf-arg-info gf))
 	 (precedence (arg-info-precedence arg-info))
 	 (nreq (arg-info-number-required arg-info))
 	 (metatypes (arg-info-metatypes arg-info)))
     (labels ((do-column (p-tail contenders known-types)
-	       (if p-tail
-		   (let* ((position (car p-tail))
-			  (known-type (or (nth position types) t)))
-		     (if (eq (nth position metatypes) 't)
-			 (do-column (cdr p-tail) contenders
-				    (cons (cons position known-type) known-types))
-			 (do-methods p-tail contenders 
-				     known-type () known-types)))
-		   (funcall methods-function contenders 
-			    (let ((k-t (make-list nreq)))
-			      (dolist (index+type known-types)
-				(setf (nth (car index+type) k-t) (cdr index+type)))
-			      k-t))))
+			(if p-tail
+			    (let* ((position (car p-tail))
+				   (known-type (or (nth position types) t)))
+			      (if (eq (nth position metatypes) t)
+				  (do-column (cdr p-tail) contenders (cons (cons position known-type) known-types))
+				(do-methods p-tail contenders known-type nil known-types)))
+			  (funcall methods-function contenders 
+				   (let ((k-t (make-list nreq)))
+				     (dolist (index+type known-types k-t)
+				       (setf (nth (car index+type) k-t) (cdr index+type)))))))
 	     (do-methods (p-tail contenders known-type winners known-types)
 	       ;;
                ;; <contenders>
@@ -1216,32 +1223,27 @@
 	       ;;   after the discrimination has been made.
 	       ;;   
                (if (null contenders)
-		   (do-column (cdr p-tail) winners
-			      (cons (cons (car p-tail) known-type) known-types))
-                   (let* ((position (car p-tail))
-			  (method (car contenders))
-			  (specl (nth position (method-specializers method)))
-                          (type (funcall type-function (type-from-specializer specl))))
-		     (multiple-value-bind (app-p maybe-app-p)
-			 (specializer-applicable-using-type-p type known-type)
-		       (flet ((determined-to-be (truth-value)
-				(if truth-value app-p (not maybe-app-p)))
-			      (do-if (truth &optional implied)
-				(let ((ntype (if truth type `(not ,type))))
-				  (do-methods p-tail
-				    (cdr contenders)
-				    (if implied
-					known-type
-					(augment-type ntype known-type))
-				    (if truth
-					(append winners `(,method))
-					winners)
-				    known-types))))
-			 (cond ((determined-to-be nil) (do-if nil t))
-			       ((determined-to-be t)   (do-if t   t))
-			       (t (funcall test-function position type 
-					   (do-if t) (do-if nil))))))))))
-      (do-column precedence methods ()))))
+		   (do-column (cdr p-tail) winners (cons (cons (car p-tail) known-type) known-types))
+		 (let* ((position (car p-tail))
+			(method (car contenders))
+			(specl (nth position (method-specializers method)))
+			(type (funcall type-function (type-from-specializer specl))))
+		   (multiple-value-bind
+		    (app-p maybe-app-p)
+		    (specializer-applicable-using-type-p type known-type)
+		    (flet ((determined-to-be (truth-value) (if truth-value app-p (not maybe-app-p)))
+			   (do-if (truth &optional implied)
+				  (let ((ntype (if truth type `(not ,type))))
+				    (do-methods p-tail
+						(cdr contenders)
+						(if implied known-type (augment-type ntype known-type))
+						(if truth (append winners `(,method)) winners)
+						known-types))))
+			  (cond ((determined-to-be nil) (do-if nil t))
+				((determined-to-be t)   (do-if t   t))
+				(t (funcall test-function position type (do-if t) (do-if nil))))))))))
+	    (do-column precedence methods nil))))
+      
 
 #+lcl3.0 (use-previous-compiler)
 
@@ -1381,14 +1383,19 @@
 	   (arg-info (gf-arg-info generic-function))
 	   (metatypes (arg-info-metatypes arg-info))
 	   (applyp (arg-info-applyp arg-info))
-	   (fmc-arg-info (cons (length metatypes) applyp)))
+	   (fmc-arg-info (cons (length metatypes) applyp))
+	   (arglist (if function-p
+			(make-dfun-lambda-list metatypes applyp)
+			(make-fast-method-call-lambda-list metatypes applyp))))
       (multiple-value-bind
 	  (cfunction constants)
-	  (get-function1 (make-dispatch-lambda 
-			  function-p metatypes applyp
-			  `((locally (declare #.*optimize-speed*)
-			      (let ((emf ,net))
-				,(make-emf-call metatypes applyp 'emf)))))
+	  (get-function1 `(lambda
+			    ,arglist
+			    ,@(unless function-p
+				`((declare (ignore .pv-cell. .next-method-call.))))
+			    (locally (declare #.*optimize-speed*)
+				     (let ((emf ,net))
+				       ,(make-emf-call metatypes applyp 'emf))))
 			 #'net-test-converter
 			 #'net-code-converter
 			 #'(lambda (form)
