@@ -2,7 +2,6 @@
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
-;;; Copyright (c) 2024 Camm Maguire
 ;;; All rights reserved.
 ;;;
 ;;; Use and copying of this software and preparation of derivative works
@@ -131,10 +130,9 @@
 (defun slot-value (object slot-name)
   (let* ((class (class-of object))
 	 (slot-definition (find-slot-definition class slot-name)))
-    (values
-     (if (null slot-definition)
-	 (slot-missing class object slot-name 'slot-value)
-	 (slot-value-using-class class object slot-definition)))))
+    (if (null slot-definition)
+	(slot-missing class object slot-name 'slot-value)
+	(slot-value-using-class class object slot-definition))))
 
 (setf (gdefinition 'slot-value-normal) #'slot-value)
 
@@ -142,7 +140,7 @@
   (if (and (constantp slot-name-form)
 	   (let ((slot-name (eval slot-name-form)))
 	     (and (symbolp slot-name) (symbol-package slot-name))))
-      `(values (accessor-slot-value ,object-form ,slot-name-form))
+      `(accessor-slot-value ,object-form ,slot-name-form)
       `(slot-value-normal ,object-form ,slot-name-form)))
 
 (defun set-slot-value (object slot-name new-value)
@@ -160,7 +158,7 @@
   (if (and (constantp slot-name-form)
 	   (let ((slot-name (eval slot-name-form)))
 	     (and (symbolp slot-name) (symbol-package slot-name))))
-      `(values (accessor-set-slot-value ,object-form ,slot-name-form ,new-value-form))
+      `(accessor-set-slot-value ,object-form ,slot-name-form ,new-value-form)
       `(set-slot-value-normal ,object-form ,slot-name-form ,new-value-form)))
 
 ;(defconstant *optimize-slot-boundp* nil)
@@ -179,7 +177,7 @@
   (if (and (constantp slot-name-form)
 	   (let ((slot-name (eval slot-name-form)))
 	     (and (symbolp slot-name) (symbol-package slot-name))))
-      `(values (accessor-slot-boundp ,object-form ,slot-name-form))
+      `(accessor-slot-boundp ,object-form ,slot-name-form)
       `(slot-boundp-normal ,object-form ,slot-name-form)))
 
 (defun slot-makunbound (object slot-name)
