@@ -95,7 +95,7 @@
 	(if (eq *boot-state* 'complete)
 	    (get-accessor-method-function gf type class slotd)
 	    (get-optimized-std-accessor-method-function class slotd type))
-      #+kcl (si:turbo-closure function)
+;      #+kcl (si:turbo-closure function)
       (setf (slot-accessor-std-p slotd type) std-p)
       (setf (slot-accessor-function slotd type) function))
     (when (and old-slotd (not (eq old-std-p (slot-accessor-std-p slotd 'all))))
@@ -113,28 +113,27 @@
   (when documentation-p
     (setf (plist-value object 'documentation) documentation)))
 
-(defmethod documentation (object &optional doc-type)
-  (lisp:documentation object doc-type))
+(defmethod documentation (object doc-type)
+  (real-documentation object doc-type))
 
-(defmethod (setf documentation) (new-value object &optional doc-type)
-  (declare (ignore new-value doc-type))
-  (error "Can't change the documentation of ~S." object))
+(defmethod (setf documentation) (new-value object doc-type)
+  (si::set-documentation object doc-type new-value))
 
 
-(defmethod documentation ((object documentation-mixin) &optional doc-type)
+(defmethod documentation ((object documentation-mixin) doc-type)
   (declare (ignore doc-type))
   (plist-value object 'documentation))
 
-(defmethod (setf documentation) (new-value (object documentation-mixin) &optional doc-type)
+(defmethod (setf documentation) (new-value (object documentation-mixin) doc-type)
   (declare (ignore doc-type))
   (setf (plist-value object 'documentation) new-value))
 
 
-(defmethod documentation ((slotd standard-slot-definition) &optional doc-type)
+(defmethod documentation ((slotd standard-slot-definition) doc-type)
   (declare (ignore doc-type))
   (slot-value slotd 'documentation))
 
-(defmethod (setf documentation) (new-value (slotd standard-slot-definition) &optional doc-type)
+(defmethod (setf documentation) (new-value (slotd standard-slot-definition) doc-type)
   (declare (ignore doc-type))
   (setf (slot-value slotd 'documentation) new-value))
 
@@ -332,21 +331,26 @@
 (setf (gdefinition 'load-defclass) #'real-load-defclass)
 
 (defun ensure-class (name &rest all)
-  (apply #'ensure-class-using-class name (find-class name nil) all))
+  (apply #'ensure-class-using-class name
+	 (let ((class (find-class name nil)))
+             (when (and class (eq name (class-name class)))
+               ;; NAME is the proper name of CLASS, so redefine it
+               class))
+	 all))
 
 (defmethod ensure-class-using-class (name (class null) &rest args &key)
   (multiple-value-bind (meta initargs)
       (ensure-class-values class args)
-    (inform-type-system-about-class (class-prototype meta) name);***
-    (setf class (apply #'make-instance meta :name name initargs)
-	  (find-class name) class)
-    (inform-type-system-about-class class name)	                ;***
-    class))
+      (inform-type-system-about-class (class-prototype meta) name);***
+      (let ((class (apply #'make-instance meta :name name initargs)))
+	(setf (find-class name) class)
+	(inform-type-system-about-class class name)	                ;***
+	class)))
 
 (defmethod ensure-class-using-class (name (class pcl-class) &rest args &key)
   (multiple-value-bind (meta initargs)
       (ensure-class-values class args)
-    (unless (eq (class-of class) meta) (change-class class meta))
+    (unless (eq (class-of class) meta) (apply #'change-class class meta initargs))
     (apply #'reinitialize-instance class initargs)
     (setf (find-class name) class)
     (inform-type-system-about-class class name)	                ;***
@@ -362,32 +366,62 @@
 	 (supplied-supers (getf initargs :direct-superclasses unsupplied))
 	 (supplied-slots  (getf initargs :direct-slots unsupplied))
 	 (meta
-	   (cond ((neq supplied-meta unsupplied)
-		  (find-class supplied-meta))
-		 ((or (null class)
-		      (forward-referenced-class-p class))
-		  *the-class-standard-class*)
-		 (t
-		  (class-of class)))))  
+	  (cond ((neq supplied-meta unsupplied)
+		 (find-class supplied-meta))
+		((or (null class)
+		     (forward-referenced-class-p class))
+		 *the-class-standard-class*)
+		(t
+		 (class-of class)))))  
     (flet ((fix-super (s)
-	     (cond ((classp s) s)
-		   ((not (legal-class-name-p s))
-		    (error "~S is not a class or a legal class name." s))
-		   (t
-		    (or (find-class s nil)
-			(setf (find-class s)
-			      (make-instance 'forward-referenced-class
-					     :name s)))))))      
-      (loop (unless (remf initargs :metaclass) (return)))
-      (loop (unless (remf initargs :direct-superclasses) (return)))
-      (loop (unless (remf initargs :direct-slots) (return)))
-      (values meta
-	      (list* :direct-superclasses
-		     (and (neq supplied-supers unsupplied)
-			  (mapcar #'fix-super supplied-supers))
-		     :direct-slots
-		     (and (neq supplied-slots unsupplied) supplied-slots)
-		     initargs)))))
+		      (cond ((classp s) s)
+			    ((not (legal-class-name-p s))
+			     (error "~S is not a class or a legal class name." s))
+			    (t
+			     (or (find-class s nil)
+				 (setf (find-class s)
+				       (make-instance 'forward-referenced-class
+						      :name s)))))))      
+	  ;;
+	  ;; CLHS: signal PROGRAM-ERROR, if
+	  ;; (a) there are any duplicate slot names
+	  ;; (b) any of the slot options :ALLOCATION, :INITFORM, :TYPE, or
+	  ;; :DOCUMENTATION appears more than one in a single slot description.
+	  (loop for (slot . more) on (getf initargs :direct-slots)
+		for slot-name = (getf slot :name)
+		if (some (lambda (s) (eq slot-name (getf s :name))) more) do
+		(error 'program-error :format-control
+				"More than one direct slot with name ~S."
+				:format-arguments (list slot-name))
+		else do
+		(loop for (option value . more) on slot by #'cddr
+		      when (and (member option '(:allocation :type :initform
+							     :documentation))
+				(not (eq unsupplied
+					 (getf more option unsupplied)))) do
+					 (error 'program-error :format-control
+							 "Duplicate slot option ~S for slot ~S."
+							 :format-arguments (list option slot-name))))
+	  ;;
+	  ;; CLHS: signal PROGRAM-ERROR, if an initialization argument name
+	  ;; appears more than once in :DEFAULT-INITARGS class option.
+	  (loop for (initarg . more) on (getf initargs :direct-default-initargs)
+		for name = (car initarg) 
+		when (some (lambda (a) (eq (car a) name)) more) do
+		(error 'program-error :format-control
+				"Duplicate initialization argument ~
+                name ~S in :default-initargs of class ~A." :format-arguments (list name class)))
+	  ;;
+	  (loop (unless (remf initargs :metaclass) (return)))
+	  (loop (unless (remf initargs :direct-superclasses) (return)))
+	  (loop (unless (remf initargs :direct-slots) (return)))
+	  (values meta
+		  (list* :direct-superclasses
+			 (and (neq supplied-supers unsupplied)
+			      (mapcar #'fix-super supplied-supers))
+			 :direct-slots
+			 (and (neq supplied-slots unsupplied) supplied-slots)
+			 initargs)))))
 
 
 ;;;
@@ -447,7 +481,10 @@
 			       (setf (slot-value class 'predicate-name)
 				     (make-class-predicate-name (class-name class))))))
   (add-direct-subclasses class direct-superclasses)
-  (update-class class nil)
+  (if (class-finalized-p class)
+      ;; required by AMOP, "Reinitialization of Class Metaobjects"
+      (finalize-inheritance class)
+      (update-class class nil))
   (make-class-predicate class predicate-name)
   (add-slot-accessors class direct-slots))
 
@@ -1204,7 +1241,7 @@
       #+new-kcl-wrapper
       (copy-structure-header ,instance)))
 
-(defun change-class-internal (instance new-class)
+(defun change-class-internal (instance new-class initargs)
   (let* ((old-class (class-of instance))
 	 (copy (allocate-instance new-class))
 	 (new-wrapper (get-wrapper copy))
@@ -1220,53 +1257,58 @@
     ;; Cfrom are retained.  If such a local slot was unbound, it remains
     ;; unbound."
     ;;     
-    (iterate ((new-slot (list-elements new-layout))
-	      (new-position (interval :from 0)))
-      (let* ((new-position new-position)
-	     (old-position (posq new-slot old-layout)))
-	(declare (fixnum new-position))
-	(when old-position
-	  (setf (instance-ref new-slots new-position)
-		(instance-ref old-slots old-position)))))
+    (loop for new-slot in new-layout and new-position from 0
+	  for old-position = (posq new-slot old-layout)
+	  when old-position do
+	    (setf (instance-ref new-slots new-position)
+		  (instance-ref old-slots old-position)))
 
     ;;
     ;; "The values of slots specified as shared in the class Cfrom and
     ;; as local in the class Cto are retained."
     ;;
-    (iterate ((slot-and-val (list-elements old-class-slots)))
-      (let ((position (posq (car slot-and-val) new-layout)))
-	(when position
-	  (setf (instance-ref new-slots position) (cdr slot-and-val)))))
+    (loop for (name . val) in old-class-slots
+	  for new-position = (posq name new-layout)
+	  when new-position do
+	    (setf (instance-ref new-slots new-position) val))
 
     ;; Make the copy point to the old instance's storage, and make the
     ;; old instance point to the new storage.
     (swap-wrappers-and-slots instance copy)
 
-    (update-instance-for-different-class copy instance)
+    (apply #'update-instance-for-different-class copy instance initargs)
     instance))
 
 (defmethod change-class ((instance standard-object)
-			 (new-class standard-class))
-  (unless (std-instance-p instance)
-    (error "Can't change the class of ~S to ~S~@
-            because it isn't already an instance with metaclass~%~S."
-	   instance
-	   new-class
-	   'standard-class))
-  (change-class-internal instance new-class))
+			 (new-class standard-class)
+			 &rest initargs)
+  (change-class-internal instance new-class initargs))
+
+;; FIXME add class funcallable-standard-object ??
+;(defmethod change-class ((instance funcallable-standard-object)
+;			 (new-class funcallable-standard-class)
+;			 &rest initargs)
+;  (change-class-internal instance new-class initargs))
 
 (defmethod change-class ((instance standard-object)
-			 (new-class funcallable-standard-class))
-  (unless (fsc-instance-p instance)
-    (error "Can't change the class of ~S to ~S~@
-            because it isn't already an instance with metaclass~%~S."
-	   instance
-	   new-class
-	   'funcallable-standard-class))
-  (change-class-internal instance new-class))
+			 (new-class funcallable-standard-class)
+			 &rest initargs)
+  (declare (ignore initargs))
+  (error "Can't change the class of ~S to ~S~@
+          because it isn't already an instance with metaclass ~S."
+	 instance new-class 'standard-class))
+
+;(defmethod change-class ((instance funcallable-standard-object)
+;			 (new-class standard-class)
+;			 &rest initargs)
+;  (declare (ignore initargs))
+;  (error "Can't change the class of ~S to ~S~@
+;          because it isn't already an instance with metaclass ~S."
+;	 instance new-class 'funcallable-standard-class))
+
+(defmethod change-class ((instance t) (new-class-name symbol) &rest initargs)
+  (apply #'change-class instance (find-class new-class-name) initargs))
 
-(defmethod change-class ((instance t) (new-class-name symbol))
-  (change-class instance (find-class new-class-name)))
 
 
 
