@@ -2,7 +2,6 @@
 ;;;
 ;;; *************************************************************************
 ;;; Copyright (c) 1985, 1986, 1987, 1988, 1989, 1990 Xerox Corporation.
-;;; Copyright (c) 2024 Camm Maguire
 ;;; All rights reserved.
 ;;;
 ;;; Use and copying of this software and preparation of derivative works
@@ -227,6 +226,7 @@
 
 (defun pv-table-lookup (pv-table pv-wrappers)
   (let* ((slot-name-lists (pv-table-slot-name-lists pv-table))
+	 (call-list (pv-table-call-list pv-table))
 	 (cache (or (pv-table-cache pv-table)
 		    (setf (pv-table-cache pv-table)
 			  (get-cache (- (length slot-name-lists)
@@ -236,7 +236,7 @@
 				     2)))))
     (or (probe-cache cache pv-wrappers)
 	(let* ((pv (compute-pv slot-name-lists pv-wrappers))
-	       (calls '#())
+	       (calls (compute-calls call-list pv-wrappers))
 	       (pv-cell (cons pv calls))
 	       (new-cache (fill-cache cache pv-wrappers pv-cell)))
 	  (unless (eq new-cache cache)
@@ -318,8 +318,6 @@
 		     cache))))))
 
 (defun update-slots-in-pv (wrappers pv cwrapper pv-size pv-map)
-  (declare (optimize (safety 1)))
-  (check-type pv-size si::seqbnd)
   (if (not (if (atom wrappers)
 	       (eq cwrapper wrappers)
 	       (dolist (wrapper wrappers nil)
@@ -983,24 +981,10 @@
 	  (make-method-initargs-form-internal1
 	   initargs (cddr lmf) args lmf-params restp)))))
 
-(defun split-declarations-moving-ignores (body req-args &aux r)
-  (multiple-value-bind (outer-decls inner-decls body)
-      (split-declarations body req-args)
-    (values
-     (mapcar (lambda (x)
-	       (cons 'declare
-		     (remove-if (lambda (y)
-				  (when (and (consp y) (member (car y) '(ignore ignorable)))
-				    (push y r)))
-				(cdr x))))
-	     outer-decls)
-     (cons (cons 'declare r) inner-decls)
-     body)))
-
 (defun make-method-initargs-form-internal1 
     (initargs body req-args lmf-params restp)
   (multiple-value-bind (outer-decls inner-decls body)
-      (split-declarations-moving-ignores body req-args)
+      (split-declarations body req-args)
     (let* ((rest-arg (when restp '.rest-arg.))
 	   (args+rest-arg (if restp (append req-args (list rest-arg)) req-args)))
       `(list* :fast-function
